SystemOrganization addCategory: #'FS-Git'!
SystemOrganization addCategory: #'FS-Git-Notifications'!
SystemOrganization addCategory: #'FS-Git-Plumbing'!
SystemOrganization addCategory: #'FS-Git-Plumbing-Notifications'!
SystemOrganization addCategory: #'FS-Git-Plumbing-Exceptions'!
SystemOrganization addCategory: #'FS-Git-Plumbing-Tests'!
SystemOrganization addCategory: #'FS-Git-Tests'!
SystemOrganization addCategory: #'FS-Git-Monticello'!
SystemOrganization addCategory: #'FS-Git-Monticello-Tests'!

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
classRemovals
	^ operations select: [ :operation| 
			operation isRemoval and: [operation definition isClassDefinition]]! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
methodRemovals
	^ operations select: [ :operation| 
			operation isRemoval and: [operation definition isMethodDefinition]]! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
packageRemovals
	^ operations select: [ :operation| 
			operation isRemoval and: [operation definition isOrganizationDefinition]]! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
removeMethodsOfClass: aClassName
	self methodRemovals do: [ :methodOperation|
		methodOperation definition className == aClassName
			ifTrue: [operations remove: methodOperation]]! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
simplifyClassRemovals
	self packageRemovals do: [ :packageRemoval|
		operations remove: packageRemoval.
		self removeMethodsOfClass: packageRemoval definition className].! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
simplifyHierarchy
	self simplifyClassRemovals.
	self simplifyMethodRemovals.! !

!MCPatch methodsFor: '*FS-Git-Monticello' stamp: 'CamilloBruni 8/9/2011 20:43'!
simplifyMethodRemovals
	self classRemovals do: [ :classRemoval|
		self removeMethodsOfClass: classRemoval definition className].! !

FSStore subclass: #FSGitStore
	instanceVariableNames: 'revision cache modManager repository cacheStore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitStore commentStamp: 'CamilloBruni 8/1/2011 18:33' prior: 0!
I am used to access files and directorys of a specific version of a git repository.!

!FSGitStore class methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 16:34'!
cache: aFilesystem
	^ self basicNew
		initializeWithCache: aFilesystem;
		yourself! !

!FSGitStore class methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 16:33'!
inMemory
	^ self cache: (FSFilesystem inMemory)! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:36'!
basenameFromEntry: entry
	^ cacheStore basenameFromEntry: entry! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 8/1/2011 16:35'!
basicCommit: aMessage to: branchName
	| commit parents author committer |
	"Disallow empty commits"
	modManager hasModifications ifFalse: [ ^ self ].
	modManager processChanges.
	
	commit := GitCommit
		withTree: modManager rootTree
		message: aMessage
		author: (author ifNil: [ author := GitStamp now ])
		committer: (committer ifNil: [ committer := GitStamp now ]).
		
	parents := OrderedCollection new.
	"don't add the fully loaded revision. add it without loaded dependencies:"
	revision ifNotNil: [ parents add: 
		(repository objectWithSignature: revision signature) ].
	commit parents: parents.
	
	repository storeObject: commit.
	repository updateRef: repository headsDir / branchName to: commit signature.
	"again, load the commit without attached dependecies:"
	revision := repository objectWithSignature: commit signature.

	self basicReset.! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 8/1/2011 16:30'!
basicCopy: source ifAbsent: aBlock to: destination ifPresent: pBlock  filesystem: aFilesystem
	self ensure: source.
	self ensure: destination.
	cacheStore basicCopy: source ifAbsent: aBlock to: destination ifPresent: pBlock  filesystem: aFilesystem.
	modManager markFileCreated: destination.! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:46'!
basicIsDirectory: aPath
	^ cacheStore basicIsDirectory: aPath! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:46'!
basicIsFile: entry
	^ cacheStore basicIsFile: entry! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 8/1/2011 16:30'!
basicOpen: aPath writable: isWritable
	isWritable ifTrue: [ modManager markFileCreated: aPath ].
	self ensure: aPath.
	^ cacheStore
		basicOpen: aPath
		writable: isWritable! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/19/2011 17:46'!
basicReset
	self flag: 'TODO: explicitely pass a revision/commit to the basicreset'.
	self flag: 'TODO: merge with ModificationManager >> clearRootTree'.
	
	cache workingDirectory children do: [ :reference|
		reference deleteAll].
	self repository: repository.
	self initializeWithCache: cache.
	! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/15/2011 15:49'!
basicSwitchToBranch: branchName
	repository updateHeadToBranch: branchName.
	self basicReset.! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/15/2011 15:49'!
basicTag: tagName
	repository updateRef: repository tagsDir / tagName to: revision signature.
	self basicReset.! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 19:42'!
branch: branchName message: aMessage
"Creates a commit in branch ==branchName== with message ==aMessage==. If the branch doesn't exist it will be created."

	self basicCommit: aMessage to: branchName.
	(branchName = self branchName)
		ifFalse: [ self basicSwitchToBranch: branchName ]! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 17:36'!
branch: branchName message: message andTag: tagName
	self
		branch: branchName
		message: message.
	self 
		tagRevision: self
		with: tagName! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 19:33'!
branchName
	self flag: 'this might be conceptually wrong'.
	^ repository activeBranchName! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 20:22'!
branchNames
	^ repository branchNames! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 20:23'!
branches
	^ repository branches! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 8/1/2011 17:20'!
commit: aMessage
	"Creates a commit with message ==aMessage== in the currently active branch."
	self basicCommit: aMessage to: self branchName! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 15:37'!
commit: aMessage andTag: tagName
"Creates a commit with message ==aMessage== and creates a tag for that commit with name ==tagName==."

	self 
		commit: aMessage;
		basicTag: tagName! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 8/1/2011 16:30'!
createDirectory: aPath
	self ensure: aPath.
	cache createDirectory: aPath.
	modManager markDirectoryCreated: aPath.! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 18:45'!
delete: anObject
	(cache exists: anObject)
			ifTrue: [ cache delete: anObject].
	modManager
		markDeleted: anObject.! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:46'!
delimiter
	^ cacheStore delimiter! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 18:03'!
detachWorkingCopy
	"removes all entries that exist in the parent tree from the
	working copy. Changes will stay intact.
	The next commit will not contain any objects 
	present in the parent tree."
	
	modManager clearRootTree! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/19/2011 18:00'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	self ensure: aPath.
	"load the directory into the cache"
	(modManager treeAt: aPath) entries do: [ :gitEntry|
			self ensure: aPath / gitEntry entryName.
		].
	cacheStore directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock.! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/19/2011 17:54'!
ensure: aPath 
	aPath isRoot ifTrue: [ ^ aPath ].
	"Make sure we have the lates version of the cache"
	(modManager isDeleted: aPath) ifTrue: [ ^ aPath ].
	(cacheStore exists: aPath) ifTrue: [ ^ aPath ].
	
	self loadFromRepository: aPath.
	^ aPath.! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 19:16'!
entryFromNode: anObject filesystem: aFilesystem path: aPath	
	self flag: 'TODO: check if we need to intercept here and update the modification manager and the cache'.
	^ cacheStore entryFromNode: anObject filesystem: aFilesystem path: aPath! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 19:16'!
entryFromNode: node path: path for: aFileystem	
	self flag: 'TODO: check if we need to intercept here and update the modification manager and the cache'.
	^ cacheStore entryFromNode: node path: path for: aFileystem! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 19:06'!
exists: aPath
	(modManager isDeleted: aPath)
		ifTrue: [ ^ false ].
	self ensure: aPath.
	^ cacheStore exists: aPath! !

!FSGitStore methodsFor: 'printing' stamp: 'CamilloBruni 7/15/2011 15:53'!
forReferencePrintOn: aStream
	aStream nextPutAll: 'git:'! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 19:16'!
growFile: path to: anInteger
	self flag: 'TODO: check if we need to intercept here and update the modification manager and the cache'.
	^ cacheStore growFile: path to: anInteger! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:47'!
handleClass
	^ cacheStore handleClass! !

!FSGitStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/19/2011 18:01'!
initializeWithCache: aFilessytem
	super initialize.
	cache := aFilessytem.
	cacheStore := cache store.
	modManager := FSGitModificationManager 
		initializeWith: repository
		with: cache
		for: revision.! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/15/2011 21:07'!
loadFromRepository: aPath
	"try to load the given path from the repositry into our cache"
	|currentGitTree|
	currentGitTree := self loadGitTreeAt: aPath.
	"path doesn't exist in the repository either"
	currentGitTree ifNil: [ ^ self ].
	
			! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/15/2011 21:13'!
loadGitTreeAt: aPath
	"recursivel walk through all the git trees and resolve the path"
	|currentGitTree|
	currentGitTree := modManager rootTree.
	
	(aPath withParents reject: [ :path | path isRoot]) do: [ :path |
		currentGitTree isTree ifFalse: [ ^ nil ]. 
		currentGitTree
			entryAt: path basename
			ifPresent: [ :entry | 
				currentGitTree := repository objectWithSignature: entry objectSignature.
				self updateCacheAt: path with: currentGitTree.]
			ifAbsent: [ 
				^ nil ] ].
	^ currentGitTree! !

!FSGitStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/19/2011 17:54'!
loadRootIntoCache
	revision ifNil: [ ^ self ].
	self halt.
	! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 15:11'!
message
	^ revision isNil
		ifTrue: [ '' ]
		ifFalse: [ revision message ]! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 8/1/2011 16:18'!
name
"Answers the sha1 name of the currently active commit (the checked out version of the repository)."

	^ revision
		ifNotNil: [ revision signature hexHash ]
		ifNil: [ '' ]! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 18:54'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	self ensure: aPath.
	^ cacheStore
		nodeAt:  aPath
		ifPresent: presentBlock
		ifAbsent: absentBlock! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:47'!
openFileStream: path writable: aBoolean
	^ cacheStore openFileStream: path writable: aBoolean! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 17:17'!
parents
	revision ifNil: [ ^ OrderedCollection new ].
	^ revision parents ! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:47'!
pathFromString: aString
	^ cacheStore pathFromString: aString! !

!FSGitStore methodsFor: 'delegate' stamp: 'CamilloBruni 7/15/2011 16:47'!
replaceFile: path in: aBlock
	^ cacheStore replaceFile: path in: aBlock! !

!FSGitStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 18:38'!
repository: aGitRepository
	repository := aGitRepository.
	modManager repository: repository.
	revision ifNil: [ self revision: repository head ].! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 19:48'!
reset
	self basicReset! !

!FSGitStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/13/2011 17:56'!
revision
	^ revision! !

!FSGitStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 18:42'!
revision: aGitCommit
	revision := aGitCommit.
	revision ifNil: [ ^ self ].
	repository ifNil: [ repository := aGitCommit repository ].
	self flag: 'TODO: directly use the commit as a reference'.
	self basicReset.! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 20:22'!
tagNames
	^ repository tagNames! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 17:36'!
tagRevision: aRevision with: aString
	aRevision basicTag: aString! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/13/2011 20:23'!
tags
	^ repository tags! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 15:33'!
timeStampOfCommit
	revision
		ifNotNil: [ ^ revision committer timeStamp ]
		ifNil: [ ^ nil ]! !

!FSGitStore methodsFor: 'git-delegating' stamp: 'CamilloBruni 7/15/2011 15:33'!
timeStampOfCreation
	revision
		ifNotNil: [ ^ revision author timeStamp ]
		ifNil: [ ^ nil ]! !

!FSGitStore methodsFor: 'internal' stamp: 'CamilloBruni 7/15/2011 21:09'!
updateCacheAt: aPath with: aGitTree
	self flag: 'TODO: merge the modmanager cache and the store cache into a single instance'.
	modManager updateCacheAt: aPath with: aGitTree.
	(cacheStore exists: aPath) ifTrue: [ ^ self ].
	aGitTree isTree
		ifTrue: [ cacheStore ensureDirectory: aPath ]
		ifFalse: [
			"its a file, hence load its contents into the cache"
			cacheStore ensureDirectory: aPath parent.
			(cache referenceTo: aPath) 
					writeStreamDo: [ :stream | stream nextPutAll: aGitTree bytes ]].
			! !

Notification subclass: #FSGitResetRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Notifications'!

!FSGitResetRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 16:15'!
defaultAction
	^ StandardWindow 	new questionWithoutCancel: 
		'You will loose all changes that you didn''t commit. Do you really want the modifications to be reset?'! !

Notification subclass: #FSGitSwitchBranchRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Notifications'!

!FSGitSwitchBranchRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 16:18'!
defaultAction
	^ StandardWindow new questionWithoutCancel: 
		'You will loose all changes you didn''t commit. Do you really want to switch branches?'! !

Notification subclass: #GitRepositoryDeleteRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Notifications'!

!GitRepositoryDeleteRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/24/2010 15:12'!
defaultAction
	^ StandardWindow new questionWithoutCancel: 
		'Do you really want to remove the repository? This cannot be undone!!'! !

Notification subclass: #GitStampEmailRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Notifications'!

!GitStampEmailRequest commentStamp: '<historical>' prior: 0!
I provide a mechanism to handle a request for the e-mail address used in a GitStamp object before it triggers an input form. This is especially useful for testing purposes and scripts.!

!GitStampEmailRequest methodsFor: 'exceptiondescription' stamp: 'CamilloBruni 6/22/2011 05:21'!
defaultAction
	| mail |
	mail := GitStamp requestEmail.
	GitStamp defaultEmail: mail.
	^ mail! !

Notification subclass: #GitStampNameRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Notifications'!

!GitStampNameRequest commentStamp: '<historical>' prior: 0!
I provide a mechanism to handle a request for the name used in a GitStamp object before it triggers an input form. This is especially useful for testing purposes and scripts.!

!GitStampNameRequest methodsFor: 'exceptiondescription' stamp: 'MaxLeske 7/13/2010 14:50'!
defaultAction
	| name |
	name := GitStamp requestName.
	GitStamp defaultName: name.
	^ name! !

MCRepository subclass: #MCGitRepository
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello'!

!MCGitRepository class methodsFor: 'configuration' stamp: 'lr 10/29/2010 14:41'!
creationTemplate
	^ self name , ' directory: ''repo.git'''! !

!MCGitRepository class methodsFor: 'configuration' stamp: 'lr 10/29/2010 14:35'!
description
	^ 'Git'! !

!MCGitRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:20'!
directory: aString
	^ self reference: (FSFilesystem onDisk referenceTo: aString)! !

!MCGitRepository class methodsFor: 'configuration' stamp: 'lr 10/29/2010 14:41'!
fillInTheBlankRequest
	^ 'Local Git Repository'! !

!MCGitRepository class methodsFor: 'testing' stamp: 'lr 10/29/2010 14:40'!
isAbstractClass
	self shouldBeImplemented! !

!MCGitRepository class methodsFor: 'configuration' stamp: 'lr 10/29/2010 14:40'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCGitRepository class methodsFor: 'instance creation' stamp: 'lr 10/29/2010 14:37'!
reference: aReference
	^ self new initializeReference: aReference! !

!MCGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2011 01:03'!
alwaysStoreDiffs
	^ true! !

!MCGitRepository methodsFor: 'private' stamp: 'CamilloBruni 8/9/2011 20:33'!
basicStoreVersion: aVersion
	| filesystem |
	filesystem := self gitHeadFilesystem.
	self flag: 'package names in git repos somehow doesnt fit. maybe use branches instead...'.
	MCGitWriter
		fileOut: aVersion
		on: (filesystem root / aVersion package name)
			ensureDirectory.
			
	filesystem
		commit: aVersion info message
		andTag: aVersion info name.
	filesystem validate.! !

!MCGitRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/28/2011 23:59'!
cacheAllFileNamesDuring: aBlock
	^ aBlock value! !

!MCGitRepository methodsFor: 'accessing' stamp: 'lr 10/29/2010 14:42'!
description
	^ 'git:/', reference asString! !

!MCGitRepository methodsFor: 'accessing-git' stamp: 'lr 10/29/2010 15:09'!
gitHeadFilesystem
	^ self gitRepository head! !

!MCGitRepository methodsFor: 'accessing-git' stamp: 'lr 10/29/2010 15:09'!
gitRepository
	^ FSGitRepository on: self reference! !

!MCGitRepository methodsFor: 'interface' stamp: 'CamilloBruni 4/29/2011 01:23'!
includesVersionNamed: aString
	self flag: '(aString copyUpToLast: $.)'.
	^ self gitRepository tagNames includes: aString.! !

!MCGitRepository methodsFor: 'initialization' stamp: 'lr 10/29/2010 14:38'!
initializeReference: aReference
	reference := aReference! !

!MCGitRepository methodsFor: 'morphic' stamp: 'lr 10/29/2010 15:07'!
morphicOpen: aWorkingCopy
	(MCFileRepositoryInspector repository: self workingCopy: aWorkingCopy)
		show! !

!MCGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 4/27/2011 15:47'!
readableFileNames
	^ self gitRepository tags gather: [ :each | 
		each tagNames collect: [ :name | name , '.mcz' ] ]! !

!MCGitRepository methodsFor: 'accessing' stamp: 'lr 10/29/2010 14:45'!
reference
	^ reference! !

!MCGitRepository methodsFor: 'interface' stamp: 'CamilloBruni 8/9/2011 18:14'!
versionFromFileNamed: aString
	| filesystem branch|
	filesystem := self gitRepository tagNamed: (aString).
	branch := aString copyUpToLast: $-.
	
	^ (MCGitReader on: filesystem root / branch) version! !

!MCGitRepository methodsFor: 'interface' stamp: 'CamilloBruni 8/9/2011 20:28'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	self halt.
	(self includesVersionNamed: aVersionInfo name)
		ifFalse: [ ^ aBlock value].
	^ self versionFromFileNamed: aVersionInfo name.! !

MCVersionInfo subclass: #MCGitVersionInfo
	instanceVariableNames: 'commit filesystem aCommit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello'!

!MCGitVersionInfo class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/8/2011 13:20'!
with: aGitFS
	^ self basicNew initializeWithFS: aGitFS! !

!MCGitVersionInfo methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/8/2011 12:41'!
commit
	^ commit! !

!MCGitVersionInfo methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/8/2011 13:24'!
initializeWithFS: aGitFS
	super initialize.
	filesystem := aGitFS.
	commit := aGitFS revision.
	self loadBasicProperties.
	! !

!MCGitVersionInfo methodsFor: 'loading' stamp: 'CamilloBruni 8/9/2011 20:40'!
loadBasicProperties
	id := commit signature hexHash.
	author := commit author name.
	date := commit author timeStamp date.
	time := commit author timeStamp time.
	message := commit message.
	name := self loadName.
	self flag: 'fix ancestors'.
	ancestors := #().! !

!MCGitVersionInfo methodsFor: 'loading' stamp: 'CamilloBruni 8/9/2011 20:40'!
loadName
	^ commit tags first name! !

Error subclass: #GitError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitInvalidBytesError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitInvalidObjectError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitInvalidPackfileError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitInvalidRepositoryError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitObjectNotFoundError
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

!GitObjectNotFoundError class methodsFor: 'exceptioninstantiator' stamp: 'MaxLeske 8/11/2011 22:05'!
signal: aString with: data
	^ self new
		signal: aString;
		data: data! !

!GitObjectNotFoundError methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 22:05'!
data
	^ data! !

!GitObjectNotFoundError methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 22:05'!
data: anObject
	data := anObject! !

!GitObjectNotFoundError methodsFor: 'printing' stamp: 'MaxLeske 8/11/2011 22:07'!
messageText
	^ String streamContents: [ :stream |
		stream
			nextPutAll: messageText;
			nextPutAll: 'Known object data: ';
			nextPutAll: data printString ] ! !

GitError subclass: #GitUnsupportedOperationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

GitError subclass: #GitValidationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Exceptions'!

MCReader subclass: #MCGitReader
	instanceVariableNames: 'infoCache definitions info dependencies package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello'!

!MCGitReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 15:25'!
classFolders
	^ self reference glob: [ :reference| 
		reference basename endsWith: '.cls']! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 15:41'!
definitions
	^ definitions ifNil: [ definitions := self loadDefinitions].! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 15:41'!
dependencies
	^ dependencies ifNil: [dependencies := self loadDependencies]! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 15:41'!
info
	^ info ifNil: [ info := self loadVersionInfo ]! !

!MCGitReader methodsFor: 'loading' stamp: 'CamilloBruni 8/9/2011 20:26'!
loadDefinitions
	|newDefinitions|
	"newDefinitions := OrderedCollection new.
	self classFolders do:
		[:classFolder |
		MCGitClassReader loadDefinitionsFrom: classFolder into: newDefinitions].
	^ newDefinitions
	"
	self flag: 'properly reconstruct the definitions from the repository data'.
	^ ( MCStReader on: (self reference / 'source.st') readStream) definitions! !

!MCGitReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 13:26'!
loadDependencies
	^ OrderedCollection new! !

!MCGitReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 15:22'!
loadPackage
	^ MCPackage named: self reference basename! !

!MCGitReader methodsFor: 'loading' stamp: 'CamilloBruni 8/9/2011 20:41'!
loadVersionInfo
	^ MCGitVersionInfo with: self reference filesystem! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 15:41'!
package
	^ package ifNil: [ package := self loadPackage ]! !

!MCGitReader methodsFor: 'accessing' stamp: 'lr 10/29/2010 17:55'!
reference
	^ stream! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 7/12/2011 13:25'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

!MCGitReader methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 20:21'!
version
	^ MCVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies! !

MCWriter subclass: #MCGitWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello'!

!MCGitWriter class methodsFor: 'as yet unclassified' stamp: 'lr 10/29/2010 17:31'!
fileOut: aVersion on: aStream
	| inst |
	inst := self on: aStream.
	inst writeVersion: aVersion! !

!MCGitWriter class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 15:26'!
readerClass
	^ MCStReader! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/29/2011 13:42'!
addDefinition: definition
	self writeDefinition: definition.! !

!MCGitWriter methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2011 01:47'!
classPathFor: aClass 
	^ self classPathForPackage: aClass category class: aClass name! !

!MCGitWriter methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2011 01:47'!
classPathForPackage: packageName class: className
	|path|
	path := self reference.
	($- split: packageName) do: [ :categorySegment|
		path := path / categorySegment].
	path := path / className, 'cls'.
	path ensureDirectory.
	^ path! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 8/9/2011 15:39'!
convert: aString
	^ aString replaceAll: Character cr with: Character lf! !

!MCGitWriter methodsFor: 'accessing' stamp: 'StefanMarr 5/18/2011 22:47'!
methodPathFor: aMethodDefinition
	|path|
	path := self classPathFor: (aMethodDefinition classIsMeta 
											ifTrue: [ aMethodDefinition actualClass theNonMetaClass ] 
											ifFalse: [ aMethodDefinition actualClass ]).
	
	aMethodDefinition actualClass.
	path := path / (aMethodDefinition classIsMeta 
											ifTrue: [ #classmethod] 
											ifFalse: [ #instancemethod]).
	path := path / aMethodDefinition category.
	^ path! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:25'!
modifyClass: oldDefinition to: newDefinition
	(self classPathFor: oldDefinition) delete.
	self writeClass: newDefinition.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:34'!
modifyDefinition: oldDefinition to: newDefinition
	oldDefinition isOrganizationDefinition
		ifTrue: [ ^ self modifyPackage: oldDefinition to:  newDefinition].
	oldDefinition isClassDefinition
		ifTrue: [ ^ self modifyClass: oldDefinition to:  newDefinition ].
	oldDefinition isMethodDefinition
		ifTrue: [ ^ self modifyMethod: oldDefinition to:  newDefinition].
	self error: 'unknown definition for removal'.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:25'!
modifyMethod: oldDefinition to: newDefinition
	(self methodPathFor: oldDefinition) delete.
	self writeMethod: newDefinition.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:35'!
modifyPackage: oldDefinition to:  newDefinition
	self shouldBeImplemented! !

!MCGitWriter methodsFor: 'accessing' stamp: 'lr 10/29/2010 17:32'!
reference
	^ stream! !

!MCGitWriter methodsFor: 'removing' stamp: 'CamilloBruni 4/29/2011 01:49'!
removeClass: aClassDefinition
	(self classPathForPackage: aClassDefinition category class: aClassDefinition className) delete
	! !

!MCGitWriter methodsFor: 'removing' stamp: 'CamilloBruni 4/29/2011 13:56'!
removeDefinition: aDefinition
	aDefinition isOrganizationDefinition
		ifTrue: [ ^ self removePackage: aDefinition ].
	aDefinition isClassDefinition
		ifTrue: [ ^ self removeClass: aDefinition ].
	aDefinition isMethodDefinition
		ifTrue: [ ^ self removeMethod: aDefinition ].
	self error: 'unknown definition for removal'.! !

!MCGitWriter methodsFor: 'removing' stamp: 'CamilloBruni 4/29/2011 15:54'!
removeMethod: aMCMethodDefinition 
	(self methodPathFor: aMCMethodDefinition) delete.! !

!MCGitWriter methodsFor: 'serializing' stamp: 'CamilloBruni 4/28/2011 23:33'!
serializeClassDefinition: aClassDefinition 
	^ String streamContents: [ :s |
		(MCStWriter on: s)
			writeClassDefinition: aClassDefinition]! !

!MCGitWriter methodsFor: 'legacy' stamp: 'CamilloBruni 8/9/2011 15:39'!
serializeDefinitions: aCollection
	^ String streamContents: [ :s |
		(MCStWriter on: s)
			writeDefinitions: aCollection ]! !

!MCGitWriter methodsFor: 'serializing' stamp: 'CamilloBruni 7/12/2011 16:00'!
serializeMetaClassDefinition: aClassDefinition 
	^ String streamContents: [ :s |
		(MCStWriter on: s)
			writeMetaclassDefinition: aClassDefinition]! !

!MCGitWriter methodsFor: 'serializing' stamp: 'CamilloBruni 4/28/2011 23:48'!
serializeMethodDefinition: aMethodDefinition 
	^ String streamContents: [ :s |
		(MCStWriter on: s)
			visitMethodDefinition: aMethodDefinition]! !

!MCGitWriter methodsFor: 'legacy' stamp: 'lr 10/29/2010 17:44'!
serializeVersionInfo: aVersionInfo
	^ String streamContents: [ :s |
		(MCVersionInfoWriter on: s)
			writeVersionInfo: aVersionInfo ]! !

!MCGitWriter methodsFor: 'accessing' stamp: 'lr 10/29/2010 17:22'!
snapshotWriterClass
	^ MCStWriter! !

!MCGitWriter methodsFor: 'syncing' stamp: 'CamilloBruni 7/6/2011 15:33'!
syncClassAt: directory
	|className|
	className := (directory basename copyUpToLast: $.) asSymbol.
	(Smalltalk includesKey: className)
		ifFalse: [ ^ directory delete ].
	self syncMethodsOfClass: (Smalltalk at: className) in: directory.! !

!MCGitWriter methodsFor: 'syncing' stamp: 'CamilloBruni 5/19/2011 21:38'!
syncMethods: selectorCollection withFiles: aDirectory 
	|selectors methodFiles|
	aDirectory exists
		ifFalse: [ ^ self ].
	selectors := selectorCollection asSet.
	methodFiles := aDirectory glob: [:path| path basename endsWith: '.st'].
	methodFiles do: [ :path|
		(selectors includes: (path basename copyUpToLast: $.) asSymbol)
			ifFalse: [ path reference delete ]].! !

!MCGitWriter methodsFor: 'updating' stamp: 'CamilloBruni 4/29/2011 15:47'!
syncMethodsOfClass: aClass in: aDirectory 
	self syncMethods: (aClass methodDict keys) withFiles: aDirectory / #instancemethod.
	self syncMethods: (aClass class methodDict) keys withFiles: aDirectory / #classmethod.
							! !

!MCGitWriter methodsFor: 'syncing' stamp: 'CamilloBruni 4/29/2011 14:55'!
syncPackageAt: directory
	self flag: 'to be implemented'.
	self syncPackagesAt: directory.! !

!MCGitWriter methodsFor: 'syncing' stamp: 'CamilloBruni 8/9/2011 15:42'!
syncPackagesAt: aPath
	aPath directories do: [ :directory|
		(directory basename endsWith: '.cls')
			ifFalse: [ self syncPackageAt: directory ]
			ifTrue: [	self syncClassAt: directory ]].! !

!MCGitWriter methodsFor: 'syncing' stamp: 'CamilloBruni 4/29/2011 14:49'!
syncSources
	self syncPackagesAt: self reference.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 8/9/2011 19:58'!
writeClass: aClassDefinition 
	|category|
	category := self classPathFor: aClassDefinition actualClass.
	(category / #classmethod) ensureDirectory.
	(category / #instancemethod) ensureDirectory.
	
	(category / 'definition.st') writeStreamDo: [ :s|
		self writeClassDefinitionFrom: aClassDefinition on: s].
	(category / 'metadefinition.st') writeStreamDo: [ :s|
		self writeMetaClassDefinitionFrom: aClassDefinition on: s].
	
	(category / 'README.txt') writeStreamDo: [ :s|
		s nextPutAll: (self convert: aClassDefinition comment)].
	! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/28/2011 23:32'!
writeClassDefinitionFrom: aClassDefinition on: aStream 
	|writer|
	self flag: 'todo'.
	aStream nextPutAll: (self convert: (self serializeClassDefinition: aClassDefinition)).! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/28/2011 18:45'!
writeClasses: aCollection 
	aCollection do: [ :class|
		self writeClass: class].! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:25'!
writeDefinition: aDefinition
	aDefinition isOrganizationDefinition
		ifTrue: [ ^ self writePackage: aDefinition ].
	aDefinition isClassDefinition
		ifTrue: [ ^ self writeClass: aDefinition ].
	aDefinition isMethodDefinition
		ifTrue: [ ^ self writeMethod: aDefinition ].
	self error: 'unknown definition for removal'.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:26'!
writeDefinitions: anOrderedCollection 
	self writePackages: (anOrderedCollection select: [:each|
 each isOrganizationDefinition]).
	self writeClasses: (anOrderedCollection select: [:each|
 each isClassDefinition]).
	self writeMethods: (anOrderedCollection select: [:each|
 each isMethodDefinition]).! !

!MCGitWriter methodsFor: 'legacy' stamp: 'CamilloBruni 8/9/2011 15:40'!
writeInfo: aVersionInfo
	(self reference / 'version') writeStreamDo: [ :s |
		s nextPutAll: (self convert: (self serializeVersionInfo: aVersionInfo)) ]! !

!MCGitWriter methodsFor: 'legacy' stamp: 'CamilloBruni 8/9/2011 15:39'!
writeLegacySnapshot: aSnapshot
	(self reference / 'source.st')
		writeStreamDo: [ :s | 
			s nextPutAll: (self convert: (self serializeDefinitions: aSnapshot definitions)) ].! !

!MCGitWriter methodsFor: 'legacy' stamp: 'CamilloBruni 8/9/2011 18:21'!
writeLegacyVersion: aVersion
	"lazy... write all formats then later on decide on what to load"
	self writeInfo: aVersion info.
	self writeLegacySnapshot: aVersion snapshot.
	
	(MCMczWriter on: (self reference / 'version.mcz'))
		writeVersion: aVersion.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/12/2011 15:59'!
writeMetaClassDefinitionFrom: aClassDefinition on: aStream 
	|writer|
	aStream nextPutAll: (self convert: (self serializeMetaClassDefinition: aClassDefinition)).! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:26'!
writeMethod: aMethodDefinition
	|path|
	path := self methodPathFor: aMethodDefinition.
	path ensureDirectory.
	
	(path / aMethodDefinition selector, 'st')
		writeStreamDo: [ :s| 
			self writeMethod: aMethodDefinition on: s].! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:26'!
writeMethod: aMethodDefinition on: aStream
	aStream nextPutAll: (self convert: (self serializeMethodDefinition: aMethodDefinition)).! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 7/6/2011 15:26'!
writeMethods: aCollection 
	aCollection do: [ :methodDefinition|
		self writeMethod: methodDefinition ].! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/29/2011 01:43'!
writeModificationPatch: aMCAddition
	self modifyDefinition: aMCAddition definition.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/28/2011 18:52'!
writePackage: aMCOrganizationDefinition 
	aMCOrganizationDefinition categories do: [ :category|
		(self reference / category) ensureDirectory].! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/28/2011 18:41'!
writePackages: aCollection 
	aCollection do: [:each| self writePackage: each].
	! !

!MCGitWriter methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 20:44'!
writePatch: aMCPatch 
	(aMCPatch simplifyHierarchy) operations do: [:operation|
		self writePatchOperation: operation].! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/29/2011 13:42'!
writePatchOperation: aPatchOperation
	aPatchOperation applyTo: self.! !

!MCGitWriter methodsFor: 'writing' stamp: 'CamilloBruni 4/29/2011 01:43'!
writeRemovalPatch: aMCAddition
	self removeDefinition: aMCAddition definition.! !

!MCGitWriter methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 15:40'!
writeSnapshot: aSnapshot
	"write out a nice file based representation so you can easily track changes with a common git tool"
	self writeDefinitions: aSnapshot definitions! !

!MCGitWriter methodsFor: 'visiting' stamp: 'CamilloBruni 8/9/2011 17:50'!
writeVersion: aVersion
	self writeLegacyVersion: aVersion.
	aVersion isDiffy
		ifTrue: [ self writePatch: aVersion patch ]
		ifFalse: [ self writeSnapshot: aVersion snapshot].
	self syncSources.! !

FSFilesystemTest subclass: #FSGitFilesystemTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitFilesystemTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testCopyDestExists; add: #testEntryAt; add: #testRootExists; add: #testEnsureDirectory; add: #testCopy; add: #testCreateDirectoryNoParent; add: #testDefaultWorkingDirectory; add: #testDirectory; add: #testRootIsNotAFile; add: #testSetWorkingDirectory; add: #testEnsureDirectoryExists; add: #testChildrenAt; add: #testReadStreamDo; add: #testChangeDirectory; add: #testReadStreamIfAbsent; add: #testSetRelativeWorkingDirectory; add: #testFileNames; add: #testWriteStream; add: #testChangeDirectoryString; add: #testCreateDirectoryExists; add: #testFile; add: #testReadStream; add: #testReadStreamDoIfAbsent; add: #testDelimiter; add: #testWriteStreamDoIfPresent; add: #testWriteStreamIfPresent; add: #testRoot; add: #testWriteStreamDo; add: #testRootIsDirectory; add: #testWorking; add: #testEnsureDirectoryCreatesParent; add: #testEntriesAt; add: #testFileStreamDo; add: #testCopySourceDoesntExist; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!FSGitFilesystemTests class methodsFor: 'accessing' stamp: 'MaxLeske 7/24/2010 17:26'!
packageNamesUnderTest
	^ #('FS-Git')! !

!FSGitFilesystemTests methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/13/2011 16:14'!
createFilesystem
	^ (FSGitRepository on: (FSFilesystem inMemory / 'testRepo.git')) head.! !

!FSGitFilesystemTests methodsFor: 'tests' stamp: 'CamilloBruni 7/20/2011 13:09'!
testCreateDirectoryNoParent
	| path |
	path := FSPath root / 'griffle' / 'glob'.
	self 
		should: [ filesystem createDirectory: path ]
		raise: FSDirectoryDoesNotExist! !

TestCase subclass: #GitTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

GitTestCase subclass: #FSGitFilesystemUsageTests
	instanceVariableNames: 'basicRepository fsRepository workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitFilesystemUsageTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new) add: #testDeletingMultipleDirectories; add: #testDeleting; add: #testDeletingMultipleWithIntermediateCreate; yourself)); add: (#passed->((Set new) add: #testRevisionChangeWithReload; add: #testEquals; add: #testLoadExistingFilesFromHead; add: #testParents; add: #testDeletingDirectory; add: #testCreateAndDeleteFile; add: #testReset; add: #testReadingNonBareRepository; add: #testGettingCommitTimestamp; add: #testTagging; add: #testCommittingCreateAndDelete; add: #testCopyingExistingFile; add: #testParentsWithReload; add: #testModifying; add: #testDeleteExistingFile; add: #testDeleteModifiedFile; add: #testBranchMessageAndTag; add: #testResetWithDeletions; add: #testBranching; add: #testReadExistingModifiedFile; add: #testDetachingWorkingCopy; add: #testTags; add: #testAddingMultipleDirectoriesFails; add: #testCreateAndReadFile; add: #testAddingWithReload; add: #testActiveBranch; add: #testReadNonexistentFile; add: #testCompleteUsecase; add: #testReadExistingFile; add: #testReadingBareRepository; add: #testAddingSingleDirectoryFails; add: #testCommittingWithReload; add: #testRevisionChange; add: #testAdding; add: #testCompleteUsecase2; add: #testLoadExistingFilesFromRevision; add: #testModifyingWithReload; add: #testCopyingWithReload; add: #testLoadExistingFiles; add: #testModifyExistingFile; add: #testCopying; add: #testCommitting; add: #testEmptyCommitTimeStamp; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 10/18/2010 16:11'!
branch: branchName message: aMessage
	[ [ workingCopy branch: branchName message: aMessage ] 
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'CamilloBruni 8/1/2011 16:54'!
commit: message
	workingCopy commit: message.
	GitFSCK validate: basicRepository.! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 10/18/2010 16:11'!
commit: aMessage andTag: tagName
	[ [ workingCopy commit: aMessage andTag: tagName ] 
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 8/11/2011 20:41'!
createFixedTimeStampCommitIn: anFSReference
	| repository blob tree commit committer author |
	repository := GitRepository on: anFSReference.
	
	blob := GitBlob bytes: 'blob1'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'first blob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature }.
		
	committer := (GitStamp new
		email: 'mail@server.ext';
		name: 'Homer Simpson';
		timeStamp: (DateAndTime year: 2011 month: 06 day: 28 hour: 22 minute: 10) asTimeStamp;
		yourself).
	author := (GitStamp new
		email: 'mail@server.ext';
		name: 'Homer Simpson';
		timeStamp: (DateAndTime year: 2010 month: 06 day: 27 hour: 21 minute: 11) asTimeStamp;
		yourself). 
					
	commit := GitCommit
		withTree: tree
		message: 'aMessage'
		author: author
		committer: committer.
	commit parents: OrderedCollection new.
		
	repository
		storeObject: blob;
		storeObject: tree;
		storeObject: commit.
	repository updateRef: repository headFile to: commit signature! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'MaxLeske 8/12/2011 08:58'!
createSomeCommits
	(workingCopy / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'content' ].
	self commit: 'first commit'.
	(workingCopy / 'dir1') ensureDirectory.
	(workingCopy / 'dir1' / 'file2') writeStreamDo: [ :stream | stream nextPutAll: 'more content' ].
	self commit: 'second commit'! !

!FSGitFilesystemUsageTests methodsFor: 'helpers' stamp: 'CamilloBruni 7/15/2011 16:26'!
reset
	workingCopy reset! !

!FSGitFilesystemUsageTests methodsFor: 'running' stamp: 'CamilloBruni 7/13/2011 19:05'!
setUp
	| ref |
	super setUp.
	ref := FSFilesystem inMemory / 'testRepo'.
	basicRepository := GitRepository on: ref.
	fsRepository := FSGitRepository  on: ref.
	workingCopy := fsRepository head! !

!FSGitFilesystemUsageTests methodsFor: 'running' stamp: 'CamilloBruni 6/22/2011 05:09'!
tearDown
	GitFSCK validate: basicRepository.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 17:37'!
testActiveBranch
	self assert: workingCopy branchName = 'master'.
	(workingCopy / 'aFile') writeStreamDo: [ :stream | ].
	self branch: 'branch 1' message: 'branch 1 commit'.
	self assert: workingCopy branchName = 'branch 1'.
	basicRepository updateHeadToBranch: 'master'.
	self assert: workingCopy branchName = 'master'.
	
	(workingCopy / 'anotherFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	basicRepository 
		basicUpdateRef: basicRepository headFile 
		to: basicRepository head signature hexHash.
	self assert: workingCopy branchName isNil! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/9/2011 20:09'!
testAddingEmptySubdir
	| ref filename |
	self assert: workingCopy root children isEmpty.
	
	filename := 'aFile'.
	ref := (workingCopy / 'foo' / 'bar') ensureDirectory / filename.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	
	ref := (workingCopy / 'foo' / 'bar2' / 'emptyDir') ensureDirectory.
	self assert: ref exists.
	
	self commit: 'adding empty subdirs'.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/9/2011 17:03'!
testAddingFile
	| ref filename |
	self assert: workingCopy root children isEmpty.
	
	filename := 'aFile'.
	ref := workingCopy / filename.
	self deny: ref exists.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	self assert: (workingCopy / filename) exists.
	self commit: 'a commit'.
	
	self assert: ref exists.
	self assert: (workingCopy / filename) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/9/2011 17:04'!
testAddingFileSubdir
	| ref filename |
	self assert: workingCopy root children isEmpty.
	
	filename := 'aFile'.
	ref := (workingCopy / 'foo' / 'bar') ensureDirectory / filename.
	self deny: ref exists.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	self assert: (workingCopy / 'foo' / 'bar' / filename) exists.
	self commit: 'a commit'.
	
	self assert: ref exists.
	self assert: (workingCopy / 'foo' / 'bar' / filename) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/1/2011 16:31'!
testAddingMultipleDirectoriesFails
	| dir1 subDir1 dir2 sha1name |
	dir1 := 'aDir'.
	subDir1 := dir1 , '/sub'.
	dir2 := 'dir 2'.
	sha1name := workingCopy name.
	(workingCopy / dir1) createDirectory.
	self assert: (workingCopy / dir1) exists.
	(workingCopy referenceTo: subDir1) createDirectory.
	self assert: (workingCopy referenceTo: subDir1) exists.
	(workingCopy / dir2) createDirectory.
	self assert: (workingCopy / dir2) exists.	
	
	self commit: 'aCommit..'.
	"the commit should not have happened:"
	self assert: sha1name = workingCopy name.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/1/2011 16:31'!
testAddingSingleDirectoryFails
	| ref dirname sha1name |
	dirname := 'aDir'.
	sha1name := workingCopy name.
	ref := workingCopy / dirname.
	ref createDirectory.
	self assert: ref exists.
	
	self commit: 'aCommit'.
	"the commit shouldn't have happened:"
	self assert: sha1name = workingCopy name
! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:54'!
testAddingWithReload
	| ref filename |
	self assert: workingCopy root children isEmpty.
	
	filename := 'aFile'.
	ref := workingCopy / filename.
	self deny: ref exists.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self assert: ref exists.
	
	self commit: 'a commit'.
	workingCopy := fsRepository head.
	ref := workingCopy / filename.
	self assert: ref exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 08:57'!
testBranchMessageAndTag
	self createSomeCommits.
	self assert: workingCopy message = 'second commit'.
	
	(workingCopy / 'aChange') writeStreamDo: [ :stream | stream nextPutAll: 'changed' ].
	workingCopy
		branch: 'new branch'
		message: 'changed one file'
		andTag: 'test fork'.
		
	self assert: workingCopy branchName = 'new branch'.
	self assert: basicRepository branches size = 2.
	self 
		shouldnt: (basicRepository branches at: 'master')
		raise: Error.
	self
		shouldnt: (basicRepository branches at: 'new branch')
		raise: Error.
	self assert: workingCopy parents first message = 'second commit'.
	self assert: workingCopy message = 'changed one file'.
	self assert: workingCopy tagNames size = 1.
	self assert: workingCopy tagNames first = 'test fork'.
	self assert: (fsRepository tagNamed: 'test fork') = workingCopy! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/13/2011 19:07'!
testBranching
	| master branch |
	(workingCopy root / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	master := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/master').
	self assert: master message = 'a commit'.
	self assert: workingCopy branchName = 'master'.
	
	(workingCopy root / 'anotherFile') writeStreamDo: [ :stream | ].
	self branch: 'my fork' message: 'my first branch'.
	branch := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/my fork').
	self assert: branch message = 'my first branch'.
	self assert: workingCopy branchName = 'my fork'.
	
	self assert: branch parents size = 1.
	self assert: branch parents first = master
	! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:21'!
testCommitting
	| revision |
	revision := workingCopy revision.
	self assert: revision isNil.
	
	(workingCopy  / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1'.
	revision := workingCopy revision.
	self assert: (workingCopy  / 'aFile') exists.
	self assert: revision isNil not.
	self assert: revision tree entries size = 1.
	self assert: (revision tree entries includesKey: 'aFile').
	
	(workingCopy  / 'file2') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	revision := workingCopy revision.
	self assert: (workingCopy  / 'file2') exists.
	self assert: (workingCopy / 'aFile') exists.
	self assert: revision isNil not.
	self assert: revision tree entries size = 2.
	self assert: (revision tree entries includesKey: 'aFile').
	self assert: (revision tree entries includesKey: 'file2')! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:53'!
testCommittingCreateAndDelete
	self assert: workingCopy root children isEmpty.
	(workingCopy / 'newFile') writeStreamDo: [ :stream | stream nextPutAll: 'some content' ].
	
	workingCopy delete: (workingCopy / 'newFile').
	
	self commit: 'commit 1'.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:22'!
testCommittingWithReload
	(workingCopy / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1'.
	self assert: (workingCopy / 'aFile') exists.
	
	workingCopy := fsRepository head.
	(workingCopy / 'file2') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	self assert: (workingCopy / 'file2') exists.
	self assert: (workingCopy / 'aFile') exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 8/1/2011 17:13'!
testCompleteUsecase
	|git |
	git := workingCopy workingDirectory.

	workingCopy root children size.

	(git / 'f') writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self commit: 'adding files'.

	(git / 'f') delete.
	self commit: 'deleted files'.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 8/1/2011 18:21'!
testCompleteUsecase2
	|git |
	git := workingCopy workingDirectory.
	
	(git / 'f') writeStreamDo: [ :stream | stream nextPutAll: 'new file' ].
	self commit: 'adding files'.

	(git / 'f') delete.
	self commit: 'deleted files'.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:19'!
testCopying
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := workingCopy / 'copy'.
	self deny: ref exists.
	
	(workingCopy / 'file1') copyTo: ref.
	self commit: 'a commit'.
	self assert: ref exists.
	self assert: (workingCopy / 'file1') exists.
	self assert: (workingCopy / 'copy') exists.
	originalContent := (workingCopy / 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'.
	! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:55'!
testCopyingExistingFile
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := workingCopy / 'copy'.
	self deny: ref exists.
	
	(workingCopy / 'file1') copyTo: ref.
	self assert: ref exists.
	self assert: (workingCopy / 'copy') exists.
	originalContent := (workingCopy / 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:20'!
testCopyingWithReload
	| ref originalContent copyContent |
	self createSomeCommits.
	ref := workingCopy / 'copy'.
	self deny: ref exists.
	
	(workingCopy / 'file1') copyTo: ref.
	self commit: 'a commit'.
	workingCopy := fsRepository head.
	ref := workingCopy / 'copy'.
	self assert: ref exists.
	self assert: (workingCopy / 'file1') exists.
	originalContent := (workingCopy / 'file1') readStream contents.
	copyContent := ref readStream contents.
	self assert: originalContent = copyContent.
	self assert: copyContent asString = 'content'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 08:57'!
testCreateAndDeleteFile
	self assert: workingCopy root children isEmpty.
	(workingCopy / 'newFile') writeStreamDo: [ :stream | stream nextPutAll: 'some content' ].
	self deny: workingCopy root children isEmpty.
	
	workingCopy delete: (workingCopy / 'newFile').
	self assert: workingCopy root children isEmpty.
	self deny: (workingCopy / 'newFile') exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 08:56'!
testCreateAndReadFile
	| filename |
	filename := 'missing File'.
	self deny: (workingCopy / filename) exists.
	self
		shouldnt: [ (workingCopy / filename) writeStreamDo: [ :stream |
			stream nextPutAll: 'initial content' ] ]
		raise: FSFileDoesNotExist.
	self assert: (workingCopy readStreamOn: filename) contents asString = 'initial content'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:56'!
testDeleteExistingFile
	self createSomeCommits.
	self assert: workingCopy root children size = 2.
	self assert: workingCopy root allChildren size = 4.
	
	workingCopy delete: (workingCopy / 'file1').
	self assert: workingCopy root children size = 1.
	self assert: workingCopy root allChildren size = 3.
	self deny: (workingCopy / 'file1') exists.
	self assert: (workingCopy / 'dir1' / 'file2') exists.
	
	self commit: 'deleted file1'.
	self deny: (workingCopy  / 'file1') exists! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 17:06'!
testDeleteModifiedFile
	| ref path |
	path := 'dir1/file2'.
	self createSomeCommits.
	
	ref := workingCopy referenceTo: path.
	ref parent ensureDirectory.
	
	ref writeStreamDo: [ :stream | 
		stream nextPutAll: 'changed it' ].
	
	self assert: ref readStream contents asString = 'changed it'.
	
	ref delete.
	self deny: ref exists.
	self deny: (workingCopy referenceTo: path) exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 09:00'!
testDeleting
	| ref  |
	self createSomeCommits.
	ref := workingCopy / 'myDir' / 'myFile'.
	self deny: ref exists.

	ref parent ensureDirectory.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'content'].
	self assert: ref exists.
	
	self commit: 'added myFile'.
	self assert: ref exists.
	
	ref delete.
	self assert: ref parent exists.
	self deny: ref exists.
	self assert: ref parent children isEmpty.
	
	self commit: 'deleted myFile'.
	"since refs parent is empty we don't store it anymore"
	self deny: ref parent exists.
	self deny: (workingCopy / 'myDir' / 'anotherFile') exists.
	self deny: ref exists.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:54'!
testDeletingDirectory
	| ref |
	self createSomeCommits.
	ref := workingCopy / 'myDir' / 'myFile'.
	self deny: ref exists.
	self deny: ref parent exists.
	
	ref parent ensureDirectory.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'content' ].
	self assert: ref exists.
	self assert: ref parent exists.
	
	self commit: 'added myDir'.
	self assert: ref exists.
	self assert: ref parent exists.
	
	ref parent delete.
	self deny: ref exists.
	self deny: ref parent exists.
	
	self commit: 'removed myDir'.
	self deny: ref parent exists.
	self deny: ref exists.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:54'!
testDeletingMultipleDirectories
	| ref ref2 |
	self createSomeCommits.
	ref := workingCopy referenceTo: '/a/b/c/d1/e/myFile'.
	ref parent ensureDirectory.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'content'].
	self assert: ref exists.
	self assert: ref parent exists.
	
	
	ref2 := workingCopy referenceTo: '/a/b/c/d2/e/myFile'.
	ref2 parent ensureDirectory.
	ref2 writeStreamDo: [ :stream | stream nextPutAll: 'content'].
	self assert: ref2 exists.
	self assert: ref2 parent exists.

	self commit: 'added myFiles'.

	ref2 parent delete.
	self commit: 'deleted /a/b/c/d2/e'.
	self deny: ref2 exists.
	
	(workingCopy referenceTo: '/a/b/c/') delete.
	self commit: 'deleted /a/b/c/'.
	self deny: ref exists.
	self deny: ref2 exists.! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:58'!
testDeletingMultipleWithIntermediateCreate
	| ref ref2 |
	self createSomeCommits.
	ref := workingCopy referenceTo: '/a/b/c/d1/e/myFile'.
	ref parent ensureDirectory.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'content' ].
	
	ref2 := workingCopy referenceTo: '/a/b/c/d2/e/myFile'.
	ref2 parent ensureDirectory.
	ref2 writeStreamDo: [ :stream | stream nextPutAll: 'content' ].
	self commit: 'added myFiles'.
	
	ref2 parent delete.
	(workingCopy referenceTo: '/a/b/c/') deleteAll.
	
	self deny: ref exists.
	self deny: ref2 exists.
	
	self commit: 'deleted /a/b/c/'.
	self deny: ref exists.
	self deny: ref2 exists.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 08:55'!
testDetachingWorkingCopy
	self createSomeCommits.
	self assert: (workingCopy / 'dir1') exists.
	self assert: (workingCopy / 'file1') exists.
	self assert: (workingCopy / 'dir1' / 'file2') exists.
	self deny: workingCopy parents isEmpty.

	workingCopy detachWorkingCopy.
	self deny: (workingCopy / 'dir1') exists.
	self deny: (workingCopy / 'file1') exists.
	self deny: (workingCopy / 'dir1' / 'file2') exists.
	self deny: workingCopy parents isEmpty.
	
	workingCopy := fsRepository head.
	(workingCopy / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'remaining change' ].
	
	workingCopy detachWorkingCopy.
	self deny: (workingCopy / 'dir1') exists.
	self deny: (workingCopy / 'dir1' / 'file2') exists.
	self deny: (workingCopy / 'file1') exists.
	self deny: workingCopy parents isEmpty! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:54'!
testEmptyCommitTimeStamp
	"uncommitted working copy will not have a stamp"
	| commitStamp authorStamp repo fs |
	repo := FSFilesystem inMemory / 'repo.git'.
	fs := (FSGitRepository on: repo) head.
	
	commitStamp := fs timeStampOfCommit.
	authorStamp := fs timeStampOfCreation.
	self assert: commitStamp isNil.
	self assert: authorStamp isNil.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 08:55'!
testEquals
	| head |
	head := fsRepository head.
	self assert: head name isEmpty.
	self assert: workingCopy name isEmpty.
	self assert: head name = workingCopy name.
	self deny: head = workingCopy.
	
	(workingCopy / 'foo') writeStreamDo: [ :stream | stream nextPutAll: 'some content' ].
	self commit: 'a commit'.
	self assert: workingCopy name isEmpty not.
	self deny: head name = workingCopy name.
	self deny: head = workingCopy.
	
	head := workingCopy.
	self assert: head = workingCopy! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:50'!
testGettingCommitTimestamp
	| commitStamp authorStamp repo fs |
	repo := FSFilesystem inMemory / 'repo.git'.
	self createFixedTimeStampCommitIn: repo.
	fs := (FSGitRepository on: repo) head.
	
	commitStamp := fs timeStampOfCommit.
	self assert: TimeStamp equals: commitStamp species.
	self 
		assert: (DateAndTime year: 2011 month: 06 day: 28 hour: 22 minute: 10) asTimeStamp 
		equals: commitStamp.
		
	authorStamp := fs timeStampOfCreation.
	self assert: TimeStamp equals: authorStamp species.
	self 
		assert: (DateAndTime year: 2010 month: 06 day: 27 hour: 21 minute: 11) asTimeStamp 
		equals: authorStamp! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/19/2011 18:51'!
testLoadExistingFiles
	self createSomeCommits.
	
	self assert: (workingCopy root / 'file1') exists.
	self assert: (workingCopy root / 'dir1') exists.
	self assert: (workingCopy root / 'dir1' / 'file2') exists.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/19/2011 18:51'!
testLoadExistingFilesFromHead
	self createSomeCommits.
	self assert: (workingCopy root / 'file1') exists.
	self assert: (workingCopy root / 'dir1') exists.
	self assert: (workingCopy root / 'dir1' / 'file2') exists.
	self deny: (workingCopy root / 'dir1' / 'file1') exists.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/19/2011 18:51'!
testLoadExistingFilesFromRevision
	self createSomeCommits.
	workingCopy := workingCopy parents first.
	
	self assert: (workingCopy root / 'file1') exists.
	self deny: (workingCopy root / 'dir1') exists.
	self deny: (workingCopy root / 'dir1' / 'file2') exists.! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:57'!
testModifyExistingFile
	| ref originalContent |
	self createSomeCommits.
	self assert: workingCopy root children size = 2.
	self assert: workingCopy root allChildren size = 4.
	
	ref := workingCopy / 'dir1' / 'file2'.
	originalContent := ref readStream contents asString.
	ref writeStreamDo: [ :stream | 
		stream 
			nextPutAll: originalContent;
			nextPutAll: '/moremoremore' ].
	self assert: workingCopy root children size = 2.
	self assert: workingCopy root allChildren size = 4.
	ref := workingCopy / 'dir1' / 'file2'.
	self assert: ref exists.
	self assert: ref readStream contents asString = 'more content/moremoremore'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:52'!
testModifying
	| ref |	
	self createSomeCommits.
	ref := workingCopy / 'file1'.
	self assert: ref readStream contents asString = 'content'.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	
	self commit: 'a commit'.
	self assert: ref readStream contents asString = 'changed it'.
	self assert: (fsRepository head / 'file1') readStream contents asString = 'changed it'.
	self assert: (fsRepository head parents first / 'file1') readStream contents asString = 'content'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'MaxLeske 8/12/2011 08:57'!
testModifyingWithReload
	| ref |	
	self createSomeCommits.
	ref := workingCopy / 'file1'.
	self assert: ref readStream contents asString = 'content'.
	ref writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	
	self commit: 'a commit'.
	workingCopy := fsRepository head.
	ref := workingCopy / 'file1'.
	self assert: ref readStream contents asString = 'changed it'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:25'!
testParents
	workingCopy parents isEmpty.
	
	(workingCopy / 'aFile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	self assert: workingCopy parents isEmpty.
	
	(workingCopy / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit number 2'.
	self deny: workingCopy parents isEmpty.
	self assert: workingCopy parents size = 1! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:25'!
testParentsWithReload
	workingCopy parents isEmpty.
	
	(workingCopy / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	workingCopy := fsRepository head.
	self assert: workingCopy parents isEmpty.
	
	(workingCopy / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit number 2'.
	workingCopy := fsRepository head.
	self deny: workingCopy parents isEmpty.
	self assert: workingCopy parents size = 1! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:57'!
testReadExistingFile
	self createSomeCommits.
	self assert: (workingCopy / 'file1') exists.
	self assert: (workingCopy / 'file1') readStream contents asString = 'content'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:57'!
testReadExistingModifiedFile
	self createSomeCommits.
	(workingCopy / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'changed it' ].
	self assert: (workingCopy / 'file1') readStream contents asString = 'changed it'! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 10/18/2010 16:11'!
testReadNonexistentFile
	self 
		should: [ workingCopy readStreamOn: 'missing File' ]
		raise: FSFileDoesNotExist! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:53'!
testReadingBareRepository
	| ref repositoryEntries |
	ref := FSFilesystem inMemory / '.git'.
	fsRepository := FSGitRepository on: ref.
	workingCopy := fsRepository head.
	self createSomeCommits.
	
	self deny: fsRepository head root children isEmpty.
	self assert: fsRepository head root children size = 2.
	
	repositoryEntries := #(
		'HEAD'
		'objects'
		'info'
		'refs').
	self assert: ((ref children collect: [ :each | each basename ]) includesAllOf: repositoryEntries)! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:34'!
testReadingNonBareRepository
	| ref repositoryEntries |
	ref := FSFilesystem inMemory / 'repo'.
	fsRepository := FSGitRepository on: ref.
	workingCopy := fsRepository head.
	self createSomeCommits.
	
	self deny: fsRepository head root children isEmpty.
	self assert: fsRepository head root children size = 2.
	
	self assert: ref children size = 1.
	self assert: ref children first basename = '.git'.
	
	repositoryEntries := #(
		'HEAD'
		'objects'
		'info'
		'refs').
	self assert: ((ref children first children collect: [ :each | each basename ]) includesAllOf: repositoryEntries)! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:00'!
testReset
	self assert: workingCopy root children isEmpty.
	(workingCopy / 'afile') writeStreamDo: [ :stream | stream nextPutAll: 'blah' ].
	(workingCopy / 'aFolder') ensureDirectory.
	(workingCopy root resolve: 'aFolder/afile') writeStreamDo: [ :stream | ].
	
	self reset.
	self assert: workingCopy root children isEmpty! !

!FSGitFilesystemUsageTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:58'!
testResetWithDeletions
	| ref |
	self createSomeCommits.
	ref := workingCopy / 'file1'.
	self assert: ref exists.
	
	ref delete.
	self deny: ref exists.
	
	self reset.
	self assert: ref exists! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:25'!
testRevisionChange
	| oldRevision newRevision |
	oldRevision := workingCopy revision.
	self assert: oldRevision isNil.
	
	(workingCopy / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	oldRevision := workingCopy revision.
	self deny: oldRevision isNil.
	
	(workingCopy / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	newRevision := workingCopy revision.
	self deny: oldRevision = newRevision.
	self assert: (newRevision parents includes: oldRevision)! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 8/1/2011 16:44'!
testRevisionChangeWithReload
	| oldRevision newRevision |
	oldRevision := workingCopy revision.
	self assert: oldRevision isNil.
	
	(workingCopy / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit'.
	oldRevision := workingCopy revision.
	self deny: workingCopy revision isNil.
	
	(workingCopy / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2'.
	newRevision  := workingCopy revision.
	self deny: oldRevision = workingCopy revision.
	self assert: (newRevision parents includes: oldRevision)! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:25'!
testTagging
	| ref |
	(workingCopy / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'a commit' andTag: 'my first tag'.
	ref := basicRepository tagsDir / 'my first tag'.
	self assert: ref exists.
	self assert: ref basename = 'my first tag'.
	self assert: (basicRepository objectNamed: ref readStream contents asString) message = 'a commit'! !

!FSGitFilesystemUsageTests methodsFor: 'commit tests' stamp: 'CamilloBruni 7/15/2011 16:25'!
testTags
	self assert: workingCopy tagNames isEmpty.
	
	(workingCopy / 'afile') writeStreamDo: [ :stream | ].
	self commit: 'commit 1' andTag: 'tag 1'.
	self assert: workingCopy tagNames size = 1.
	self assert: (workingCopy tagNames includes: 'tag 1').
	
	(workingCopy / 'anotherfile') writeStreamDo: [ :stream | ].
	self commit: 'commit 2' andTag: 'tag number two'.
	self assert: workingCopy tagNames size = 2.
	self assert: (workingCopy tagNames includes: 'tag number two').
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag3' 
		to: (GitObjectSignature from: workingCopy name).
	self assert: workingCopy tagNames size = 3.
	self assert: (workingCopy tagNames includesAllOf: #('tag 1' 'tag number two' 'tag3'))! !

GitTestCase subclass: #FSGitLazyLoadingTests
	instanceVariableNames: 'filesystem repository workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitLazyLoadingTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testLazyLoading; add: #testLookup; add: #testBasicCommit; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!FSGitLazyLoadingTests methodsFor: 'helpers' stamp: 'MaxLeske 10/18/2010 16:07'!
commit: message
	[ [ workingCopy commit: message ]
		on: GitStampNameRequest 
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest 
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!FSGitLazyLoadingTests methodsFor: 'running' stamp: 'CamilloBruni 7/15/2011 16:02'!
setUp
	super setUp.
	filesystem := FSFilesystem inMemory.
	repository := FSGitRepository on: filesystem root.
	workingCopy := repository head.
	
	workingCopy / 'file1' writeStreamDo: [ :stream | stream nextPutAll: 'file1' ].
	workingCopy / 'file2' writeStreamDo: [ :stream | stream nextPutAll: 'file2' ].
	workingCopy / 'file3' writeStreamDo: [ :stream | stream nextPutAll: 'file3' ].
	self commit: 'added 3 files to root'.

	(workingCopy / 'dir1') createDirectory.
	(workingCopy / 'dir1' / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir1' ].
	(workingCopy / 'dir1' / 'file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir1' ].
	(workingCopy / 'dir1' / 'file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir1' ].
	self commit: 'added a directory with 3 files to root'.
	
	(workingCopy / 'dir2') createDirectory.
	(workingCopy / 'dir2' / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir2' ].
	(workingCopy / 'dir2' / 'file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir2' ].
	(workingCopy / 'dir2' / 'file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir2' ].
	self commit: 'added 3 files to dir2'.
	
	(workingCopy / 'dir2' / 'dir2.1') createDirectory.
	(workingCopy / 'dir2' / 'dir2.1' / 'file1') writeStreamDo: [ :stream | stream nextPutAll: 'file1 in dir2.1' ].
	(workingCopy / 'dir2' / 'dir2.1' / 'file2') writeStreamDo: [ :stream | stream nextPutAll: 'file2 in dir2.1' ].
	(workingCopy / 'dir2' / 'dir2.1' / 'file3') writeStreamDo: [ :stream | stream nextPutAll: 'file3 in dir2.1' ].
	self commit: 'added a subdir to dir2 and 3 files to the subdir'! !

!FSGitLazyLoadingTests methodsFor: 'running' stamp: 'CamilloBruni 6/22/2011 05:08'!
tearDown
	GitFSCK validate: repository repository.! !

!FSGitLazyLoadingTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:59'!
testBasicCommit		
	self assert: (workingCopy / 'newFile') exists not.
	(workingCopy / 'newFile') writeStreamDo: [ :stream | stream nextPutAll: 'testcontent' ].
	self commit: 'test commit'.
	
	self assert: (repository head / 'newFile') exists! !

!FSGitLazyLoadingTests methodsFor: 'tests' stamp: 'CamilloBruni 8/1/2011 16:16'!
testLazyLoading
	| entries contents |
	entries := #(
		'/'
		'/file1' '/file2' '/file3'
		'/dir1' '/dir1/file1' '/dir1/file2' '/dir1/file3'
		'/dir2' '/dir2/file1' '/dir2/file2' '/dir2/file3'
		'/dir2/dir2.1' '/dir2/dir2.1/file1' '/dir2/dir2.1/file2' '/dir2/dir2.1/file3').
	self assert: ((repository head root allChildren collect: [ :each |
		each fullName ]) includesAllOf: entries).
	contents := (repository head / 'file1')
		readStreamDo: [ :stream | stream contents asString ].
	self assert: contents = 'file1'! !

!FSGitLazyLoadingTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 16:00'!
testLookup
	| files directories ref |
	files := #(
		'/file1' '/file2' '/file3'
		'/dir1/file1' '/dir1/file2' '/dir1/file3'
		'/dir2/file1' '/dir2/file2' '/dir2/file3'
		'/dir2/dir2.1/file1' '/dir2/dir2.1/file2' '/dir2/dir2.1/file3').
	directories := #('/dir1' '/dir2' '/dir2/dir2.1').
	
	files do: [ :name |
		ref := workingCopy root resolve: name.
		self assert: ref exists.
		self assert: ref isFile ].
	directories do: [ :name |
		ref := workingCopy root resolve: name.
		self assert: ref exists.
		self assert: ref isDirectory ].
	
	self assert: workingCopy root children size = 5.
	self assert: workingCopy root entries size = 5.
	self assert: workingCopy root allChildren size = (files size + directories size + 1).
	self assert: workingCopy root allEntries size = (files size + directories size + 1).
	self assert: workingCopy root files size = 3.
	self assert: workingCopy root directories size = 2! !

GitTestCase subclass: #FSGitRepositoryTests
	instanceVariableNames: 'repository basicRepository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!FSGitRepositoryTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new) add: #testLightTags; yourself)); add: (#passed->((Set new) add: #testVersionsOrder; add: #testFullTags; add: #testVersions; add: #testBranches; add: #testTags; add: #testHead; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!FSGitRepositoryTests methodsFor: 'running' stamp: 'CamilloBruni 7/13/2011 19:51'!
setUp
	| repoRef repo commit1 commit2 tree blob |
	super setUp.
	repoRef := FSFilesystem inMemory / 'testRepo'.
	repo := GitRepository on: repoRef.
	blob := GitBlob bytes: 'blob1'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'first blob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature }.
	commit1 := GitCommit 
		withTree: tree
		message: 'first commit'
		author: GitStamp now
		committer: GitStamp now.
	repo
		storeObject: blob;
		storeObject: tree;
		storeObject: commit1;
		updateRef: repo headsDir / 'branch1' to: commit1 signature.
	
	blob := GitBlob bytes: 'blob2'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'second blob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature }.
	commit2 := GitCommit
		withTree: tree
		message: 'second commit'
		author: GitStamp now
		committer: GitStamp now.
	commit2 parents: { commit1 }.
	repo
		storeObject: blob;
		storeObject: tree;
		storeObject: commit2;
		updateRef: repo headsDir / 'master' to: commit2 signature.
		
	basicRepository := repo.
	repository := FSGitRepository on: repoRef! !

!FSGitRepositoryTests methodsFor: 'running' stamp: 'CamilloBruni 6/22/2011 05:22'!
tearDown
	GitFSCK validate: basicRepository.! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/21/2010 20:56'!
testBranches
	| branches |
	branches := repository branches.
	self assert: branches size = 2.
	branches do: [ :aBranch |
		self assert: aBranch species = FSGitFilesystem.
		self assert: ({ FSPath / 'first blob'. FSPath / 'second blob' } 
			includes: aBranch root children first path) ]! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:04'!
testFullTags
	| commit tags |
	commit := basicRepository objectWithSignature: (basicRepository resolveRef: 'refs/heads/branch1').
	[ [ GitTag
		makeFullTag: 'version 4.3alpha'
		for: commit
		message: 'PGP encrypted string'
		in: basicRepository ] 
			on: GitStampNameRequest 
			do: [ :req | req resume: 'Homer Simpson' ] ]
				on: GitStampEmailRequest
				do: [ :req | req resume: 'homer@nuke.com' ].
			
	tags := repository tags.
	self assert: tags size = 1.
	self assert: tags first species = FSGitFilesystem.
	self assert: tags first revision = commit.! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/11/2011 21:32'!
testHead
	| head fsOnHead |
	head := basicRepository objectWithSignature: basicRepository headRef.
	fsOnHead := repository head.
	self assert: fsOnHead species = FSGitFilesystem.
	self assert: head = (fsOnHead  revision)! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/15/2011 15:44'!
testLightTags
	| master branch1 tags |
	master := basicRepository resolveRef: 'refs/heads/master'.
	branch1 := basicRepository resolveRef: 'refs/heads/branch1'.
	basicRepository 
		updateRef: basicRepository tagsDir / 'version 4.3alpha' to: master;
		updateRef: basicRepository tagsDir / 'fail' to: branch1.
		
	tags := repository tags.
	self assert: tags size = 2.
	tags do: [ :each |
		self assert: each species = FSGitFilesystem.
		self assert: ({ master. branch1} includes: (each revision) signature) ]! !

!FSGitRepositoryTests methodsFor: 'commit tests' stamp: 'CamilloBruni 6/22/2011 04:47'!
testTags
	self assert: repository tags isEmpty.
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag1' 
		to: basicRepository head signature.
	self assert: repository tags size = 1.
	self assert: (repository tags includes: repository head).
	
	basicRepository 
		updateRef: basicRepository tagsDir / 'tag2' 
		to: basicRepository head parents first signature.
	self assert: repository tags size = 2.
	self assert: (repository tags includes: repository head).
	self assert: (repository tags includes: repository head parents first)! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/13/2011 17:03'!
testVersions
	| versions versionOld versionNew workingCopy |
	workingCopy := repository head.
	(workingCopy / 'aFile') writeStreamDo: [ :stream | stream nextPutAll: 'third commit' ].
	workingCopy commit: 'commit 3'.
	
	versions := repository versions.
	self assert: versions size = 3.
	versions detect: [ :version| version message = 'commit 3' ].
	versionNew := versions detect: [ :version | version message = 'second commit' ].
	versionOld := versions detect: [ :version | version message = 'first commit' ].
	
	self assert: versionOld root children size = 1.
	self assert: versionOld root children first basename = 'first blob'.
	
	self assert: versionNew root children size = 1.
	self assert: versionNew root children first basename = 'second blob'! !

!FSGitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 2/1/2011 17:56'!
testVersionsOrder
	| versions versionOld versionNew |
	versions := repository versions.
	versionOld := versions detect: [ :version | version message = 'first commit' ].
	versionNew := versions detect: [ :version | version message = 'second commit' ].
	self assert: versionNew = (versions at: 1).
	self assert: versionOld = (versions at: 2)! !

GitTestCase subclass: #GitBlobTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitBlobTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testRetrievingBytes; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitBlobTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:28'!
testRetrievingBytes
	| blob |
	blob := GitBlob bytes: 'testBlob'.
	self assert: blob bytes = ('testBlob' asByteArray).
	
	blob := GitBlob bytes: 3242.
	self assert: blob bytes = (3242 asByteArray).
	
	blob := GitBlob bytes: #[3 42 45 3].
	self assert: blob bytes = #[3 42 45 3]! !

GitTestCase subclass: #GitCommitTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitCommitTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testPrintOn; add: #testWritingSameCommitTwice; add: #testInitializeWithEverything; add: #testRetrievingParents; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 10/25/2010 10:01'!
testInitializeWithEverything
	| commit tree message author committer blob |
	blob := GitBlob bytes: 'aBlob'.
	message := 'a test message'.
	author := (GitStamp new
				email: 'mail@server.ext';
				name: 'Homer Simpson';
				timeStamp: TimeStamp now;
				yourself).
	committer := (GitStamp new
					email: 'homer@simpson.spr';
					name: 'Bart Simpson';
					timeStamp: TimeStamp now;
					yourself).
	tree := GitTree entries: {GitTreeEntry 
								entryName: 'filename'
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	
	commit := GitCommit 
				withTree: tree
				message: message
				author: author
				committer: committer.
	
	self assert: commit tree = tree.
	self assert: commit message = message.
	self assert: commit author = author.
	self assert: commit committer = committer! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:55'!
testPrintOn
	| message commit template |
	message := 'printOn test'.
	commit := GitCommit new.
	commit message: message.
	
	template := 'a GitCommit(''printOn test'')'.
	
	self assert: commit printString = template.! !

!GitCommitTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:34'!
testRetrievingParents
	| commit collection parent1 parent2 parent3 |
	parent1 := GitCommit new.
	parent2 := GitCommit new.
	parent3 := GitCommit new.
	collection := OrderedCollection new.
	collection add: parent1; add: parent2; add: parent3.
	commit := GitCommit new.
	commit parents: collection.
	
	self assert: commit parents size = 3.
	self assert: commit parents first == parent1.
	self assert: commit parents second == parent2.
	self assert: commit parents third == parent3! !

!GitCommitTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:52'!
testWritingSameCommitTwice
	| commit repository tree blob treeEntry |
	blob := GitBlob bytes: 'ablob'.
	treeEntry :=
		GitTreeEntry 
			entryName: 'theblob'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature.
	tree := GitTree entries: { treeEntry }.
	commit := 
		GitCommit 
		withTree: tree 
		message: 'a commit'
		author: self defaultStamp
		committer: self defaultStamp.
	repository := GitRepository on: (FSFilesystem inMemory root).
	repository storeObject: commit.
	repository updateRef: repository headsDir / 'master' to: commit signature.
	
	self shouldnt: [ repository storeObject: commit ] raise: Exception.! !

GitTestCase subclass: #GitObjectLoaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitObjectLoaderTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testFor; add: #testNew; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'CamilloBruni 7/13/2011 19:54'!
testFor
	| repository loader |
	repository := GitRepository on: FSFilesystem inMemory root.
	loader := GitObjectLoader for: repository.
	self assert: (loader instVarNamed: 'repository')= repository! !

!GitObjectLoaderTests methodsFor: 'class-tests' stamp: 'MaxLeske 8/11/2011 22:58'!
testNew
	self should: [ GitObjectLoader new ] raise: Error! !

GitTestCase subclass: #GitObjectSignatureTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitObjectSignatureTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testByteSignatureForBlob; add: #testPrintOn; add: #testEquals; add: #testByteSignatureCreation; add: #testHexHashFromIntegerHash; add: #testBytePadding; add: #testHashingConditions; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:54'!
authorEmail
	^'homer@nuke.com'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 19:55'!
authorName
	^'Homer Simpson'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/25/2011 18:17'!
authorTimeStamp
	^(TimeStamp fromUnixTime: 1274888648) offset: (Duration hours: 2)! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:18'!
blobContents
	^ 'aString' asByteArray! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:28'!
blobSignature
	^ GitObjectSignature for: self createBlob! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:26'!
blobTypeCode
	^ GitBlob typeCode! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:50'!
commitContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream 
		nextPutAll: self treeTypeCode asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self treeSignature hexHash asByteArray;
		nextPut: Character lf asInteger;
		"this commit has no parents"
		nextPutAll: 'author' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'committer' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPut: Character lf asInteger;
		nextPutAll: self commitMessage asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:49'!
commitMessage
	^ 'the message'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:32'!
commitSignature
	^ GitObjectSignature for: self createCommit! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
commitTypeCode
	^ GitCommit typeCode! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:18'!
createBlob
	^ GitBlob bytes: self blobContents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:49'!
createCommit
		^ GitCommit 
			withTree: self createTree
			message: self commitMessage
			author: self defaultStamp
			committer: self defaultStamp.! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/24/2010 17:18'!
createTag
	^ GitTag new
		name: self tagName;
		message: self tagMessage;
		tagger: self defaultStamp;
		object: self createCommit;
		yourself! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:55'!
createTree
	^ GitTree entries: { self createTreeEntry }! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:56'!
createTreeEntry
	^ GitTreeEntry
			entryName: self treeEntryName
			mode: GitTreeEntry fileMode
			objectSignature: self createBlob signature! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:35'!
defaultStamp
	| fixedTime |
	fixedTime := DateAndTime 
		year: 2010
		month: 7 
		day: 22
		hour: 19
		minute: 34
		second: 23
		offset: (Duration hours: 5).
		
	^ [ [ (GitStamp now) timeStamp: fixedTime ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ]! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/24/2010 17:18'!
referenceAuthor
	^ GitStamp new
		name: self authorName;
		email: self authorEmail;
		timeStamp: self authorTimeStamp;
		yourself! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:45'!
referenceBlobHash
	^'e7cbb71a0d5b66bd7116563b395a98a81e50f988'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceBlobSize
	^8! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'CamilloBruni 6/21/2011 08:22'!
referenceCommitHash
	^ '0f02985452ad9beebb2af39649145fbf5d95aa31'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceContent
	^'testfile'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:12'!
referenceFileName
	^'TESTFILE.txt'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/26/2010 18:11'!
referenceMessage
	^'testfile commit'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 6/23/2010 19:59'!
referenceTreeEntryMode
	^8r100644! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 10:52'!
referenceTreeHash
	^'ad8473da921cc7878fe1106d3083b477d6592de4'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 5/27/2010 11:43'!
referenceTreeSize
	^40! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:20'!
tagContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream
		nextPutAll: 'object' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self commitSignature hexHash asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'type' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: GitCommit typeCode asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'tag' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self tagName asByteArray;
		nextPut: Character lf asInteger;
		nextPutAll: 'tagger' asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self defaultStamp printString asByteArray;
		nextPut: Character lf asInteger;
		nextPut: Character lf asInteger;
		nextPutAll: self tagMessage asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:13'!
tagMessage
	^ 'Maybe a PGP encrypted message'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:12'!
tagName
	^ 'some releaseRC1'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:16'!
tagSignature
	^ GitObjectSignature for: self createTag! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
tagTypeCode
	^ GitTag typeCode! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 7/31/2010 17:50'!
testBytePadding
	| signature testHash savedHash stream |
	testHash := '00a350a77157772e1e35bb27b0582a8c51094fac'.
	self assert: testHash size = 40.
	signature := GitObjectSignature from: testHash.
	self assert: signature hexHash = testHash.
	
	stream := GitStream new.
	signature saveTo: stream.
	savedHash := stream contents hex.
	self assert: savedHash size = 40.
	self assert: (savedHash beginsWith: '00a35').
	self deny: (savedHash beginsWith: 'a35').
	self assert: savedHash = testHash! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:52'!
testByteSignatureCreation
	"reference commit created with GitX 0.7.1 on git 1.7.0"
	| fileRef stream testCommit tree blob |	
	fileRef := FSFilesystem inMemory /self referenceFileName.
	fileRef writeStreamDo: [:writer | writer nextPutAll: self referenceContent].
	stream := fileRef readStream.
	self assert:  stream contents size = self referenceBlobSize.	
	blob := GitBlob bytes: stream contents.
	
	tree := GitTree entries: {GitTreeEntry
								entryName: self referenceFileName
								mode: GitTreeEntry fileMode
								objectSignature: blob signature}.
	testCommit := GitCommit 
					withTree: tree
					message: self referenceMessage
					author: self referenceAuthor
					committer: self referenceAuthor.
	
	self assert: blob signature hexHash = self referenceBlobHash.
	self assert: GitTreeEntry fileMode  = self referenceTreeEntryMode.
	self assert: tree size = self referenceTreeSize.
	self assert: tree signature hexHash = self referenceTreeHash.
	self assert: testCommit signature hexHash = self referenceCommitHash.! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/9/2010 11:42'!
testByteSignatureForBlob
	| stream bytes |
	stream := ReadStream on: (GitObjectSignature new byteSignatureFor: (GitBlob bytes: 'test')).
	
	bytes := stream upTo: Character space asInteger.
	self assert: bytes asString = 'blob'.
	
	bytes := stream upTo: 0.
	self assert: bytes asString asInteger = 4.
	
	bytes := stream upToEnd.
	self assert: bytes asString = 'test'! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/24/2010 12:33'!
testEquals
	| signature1 signature2 |
	signature1 := GitObjectSignature for: (GitBlob bytes: 'test1').
	signature2 := GitObjectSignature for: (GitBlob bytes: 'test2').
	self deny: signature1 = signature2! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 7/22/2010 20:30'!
testHashingConditions
	| objectSignature objectContent objectTypeCode templateStream objectNames |
	objectNames := #('blob' 'commit' 'tree' 'tag').
	
	objectNames do:  [ :objectName |
		templateStream := WriteStream on: (ByteArray new).
		objectTypeCode := self perform: (objectName , 'TypeCode') asSymbol.
		objectContent := self perform: (objectName , 'Contents') asSymbol.
		objectSignature := self perform: (objectName , 'Signature') asSymbol.
		
		templateStream 
			nextPutAll: objectTypeCode asByteArray;
			nextPut: Character space asInteger;
			nextPutAll: objectContent size asString asByteArray;
			nextPut: 0;
			nextPutAll: objectContent.
		
		self assert: templateStream contents = objectSignature bytes ]! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 5/25/2011 17:11'!
testHexHashFromIntegerHash
	| hash |
	hash := SecureHashAlgorithm new hashMessage: 11.
	self assert: hash printStringHex size = 39.
	hash := GitObjectSignature new hexHashFrom: hash.
	self assert: hash size = 40.
	self assert: hash asLowercase = hash.! !

!GitObjectSignatureTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 19:48'!
testPrintOn
	| signature template |
	signature := GitObjectSignature from: 'bf8b4530d8d246dd74ac53a13471bba17941dff7'.
	template := 'a GitObjectSignature(bf8b4530d8d246dd74ac53a13471bba17941dff7)'.
	
	self assert: signature printString = template! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 20:02'!
treeContents
	| stream |
	stream := WriteStream on: (ByteArray new).
	stream
		nextPutAll: (GitTreeEntry fileMode printStringBase: 8) asByteArray;
		nextPut: Character space asInteger;
		nextPutAll: self treeEntryName asByteArray;
		nextPut: 0;
		nextPutAll: self createBlob signature integerHash asByteArray.
		
	^ stream contents! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:55'!
treeEntryName
	^ 'a filename'! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:42'!
treeSignature
	^ GitObjectSignature for: self createTree! !

!GitObjectSignatureTests methodsFor: 'helpers' stamp: 'MaxLeske 7/22/2010 19:27'!
treeTypeCode
	^ GitTree typeCode! !

GitTestCase subclass: #GitRepositoryTests
	instanceVariableNames: 'repository reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitRepositoryTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testStoreObjectCommitNoAuthor; add: #testStoreObjectTree; add: #testBranches; add: #testGitInitializeOnReferenceWithSubDirs; add: #testDirExistsButNotInitialized; add: #testNew; add: #testDirDoesntExist; add: #testEquals; add: #testResolveRef; add: #testCommit; add: #testObjectWithSignature; add: #testStoreObjectCommit; add: #testTags; add: #testStoreObjectBlob; add: #testLooseObjectBytesAt; add: #testInitializeOnDiskFilesystem; add: #testInitializeOnMemoryFilesystem; add: #testSetupDirectoryStructure; add: #testCreateReferencedDirectory; add: #testIsInitialized; add: #testStoreObjectCommitNoTree; add: #testBaseDir; add: #testOnClass; add: #testLooseObjectNames; add: #testDelete; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitRepositoryTests methodsFor: 'running' stamp: 'CamilloBruni 7/13/2011 19:51'!
setUp
	| commit1 commit2  blob1 blob2 tree1 tree2 stamp |
	super setUp.
	stamp := GitStamp new
		name: 'Homer Simpson';
		email: 'homer@nuke.com';
		yourself.
	reference := (FSFilesystem inMemory / 'test.git').
	repository := GitRepository on: reference.

	blob1 := GitBlob bytes: 'testBlob'.
	blob2 := GitBlob bytes: 'second test Blob'.
	tree1 := GitTree entries: {
		GitTreeEntry
			entryName: 'blob1'
			mode: GitTreeEntry fileMode
			objectSignature: blob1 signature}.
	tree2 := GitTree entries: {
		GitTreeEntry
			entryName: 'blob2'
			mode: GitTreeEntry fileMode
			objectSignature: blob2 signature}.
			
	commit1 := GitCommit 
		withTree: tree1
		message: 'message1'
		author: stamp 
		committer: stamp.
	commit2 := GitCommit
		withTree: tree2
		message: 'message2'
		author: stamp
		committer: stamp.
	commit2 parents: { commit1 }.
	
	repository
		storeObject: blob1;
		storeObject: blob2;
		storeObject: tree1;
		storeObject: tree2;
		storeObject: commit1;
		storeObject: commit2;
		updateRef: repository headsDir / 'branch1' to: commit1 signature;
		updateRef: repository headsDir / 'master' to: commit2 signature;
		updateRef: repository headFile to: commit2 signature! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:07'!
testBaseDir
	self assert: (reference basename endsWith: '.git').
	self assert: repository baseDir = reference.
	
	reference := reference resolve: 'noExtension'.
	repository := GitRepository on: reference.
	self deny: repository baseDir = reference.
	self assert: repository baseDir = (reference resolve: '.git')! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/11/2010 13:05'!
testBranches
	| branches |
	branches := repository branches.
	self assert: branches size = 2.
	branches do: [ :each |
		self assert: (each isKindOf: GitCommit).
		self assert: (#('message1' 'message2') includes: each message)]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:05'!
testCommit
	| blob tree commit tag lightTag |
	blob := GitBlob bytes: 'hello'.
	repository storeObject: blob.
	tree := GitTree
		entries: {(GitTreeEntry fileName: 'testFile' mode: GitTreeEntry fileMode objectSignature: blob signature)}.
	repository storeObject: tree.
	commit := (GitCommit new)
		author: self defaultStamp;
		committer: self defaultStamp;
		tree: tree;
		message: 'initial commit';
		yourself.
	repository storeObject: commit.
	tag := GitTag
		makeFullTag: 'tagtest'
		for: commit
		message: 'First tag'
		tagger: self defaultStamp
		in: repository.
	lightTag := GitTag makeLightWeightTag: 'light tag' for: commit in: repository.
	repository updateRef: 'HEAD' to: commit signature! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 20:06'!
testCreateReferencedDirectory
	| filesystem |
	filesystem := FSFilesystem inMemory.
	reference := filesystem root resolve: 'alpha/beta/gamma/repo.git'.
	repository := GitRepository on: reference.
	
	filesystem root allChildren do: [:each | self assert: each exists].
	
	filesystem := FSFilesystem inMemory.
	reference := filesystem root resolve: '/alpha/beta/repo.git'.
	(filesystem root / 'alpha') createDirectory.
	repository := GitRepository on: reference.
	
	self 
		shouldnt: [repository gitInitialize]
		raise: FSDirectoryExists
		description: 'The code should check if one of parent directories exists before trying to create it'.
	self assert: (filesystem root / 'alpha') exists.
	self assert: (filesystem root / 'alpha' / 'beta') exists.
	self assert: (filesystem root / 'alpha' / 'beta' / 'repo.git') exists.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/24/2010 15:17'!
testDelete
	self assert: repository exists description: 'Need a repository to delete.'.
	[ repository delete ]
		on: GitRepositoryDeleteRequest 
		do: [ :req | req resume: false].
	self assert: repository exists.
	
	[ repository delete ]
		on: GitRepositoryDeleteRequest 
		do: [ :req | req resume: true].
	self deny: repository exists description: 'Deletion of repository failed.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:00'!
testDirDoesntExist
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: (reference resolve: 'newRepo').
	self deny: repository exists.
	repository gitInitialize.
	self assert: repository exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 7/23/2010 10:02'!
testDirExistsButNotInitialized
	reference := reference resolve: 'newRepo'.
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: reference.
	reference createDirectory.
	self deny: repository exists.
	repository gitInitialize.
	self assert: repository exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:56'!
testEquals
	| repo2 |
	repo2 := GitRepository on: FSFilesystem inMemory workingDirectory .
	self deny: repository = repo2.
	
	repo2 := GitRepository on: repository baseDir.
	self assert: repository = repo2! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 20:08'!
testGitInitializeOnReferenceWithSubDirs
	reference := FSFilesystem inMemory referenceTo: 'alpha/beta/testRepo.git'.
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: reference.
	self 
		shouldnt: [repository gitInitialize]
		raise: FSDirectoryDoesNotExist
		description: 'The reference of the repository points to a directory'
			, 'inside of a parent directory that does not exist.'
			, 'Create the parent directory first.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/11/2011 23:08'!
testHeadFile
	| headContents |
	self assert: (repository baseDir / 'HEAD') exists.
	self assert: repository headFile exists.
	self assert: (repository baseDir allFiles select: [ : file |
		file basename = 'HEAD' ]) size = 1.
	self shouldnt: [ repository validate ] raise: GitInvalidRepositoryError.
	
	repository headFile delete.
	self deny: repository headFile exists.
	self should: [ repository validate ] raise: GitInvalidRepositoryError	
	! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 20:10'!
testInitializeOnDiskFilesystem
	reference := FSFilesystem onDisk referenceTo: 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository baseDir filesystem isKindOf: FSFilesystem).
	self assert: repository baseDir = (reference / '.git').
	reference deleteAll.
	self deny: reference exists! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:54'!
testInitializeOnMemoryFilesystem
	reference := FSFilesystem inMemory / 'newRepo'.
	repository := GitRepository basicNew initializeOn: reference.
	self assert: (repository baseDir filesystem isKindOf: FSFilesystem).
	self assert: repository baseDir = (reference / '.git')! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:54'!
testIsInitialized
	repository := GitRepository basicNew.
	repository instVarNamed: 'reference' put: (FSFilesystem inMemory / 'test.git').
	self deny: repository isInitialized.
	
	repository gitInitialize.
	self assert: repository isInitialized! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 21:45'!
testLooseObjectBytesAt
	| blob bytes |
	blob := repository storeObject: (GitBlob bytes: 'testBlob').
	bytes := repository looseObjectBytesAt: blob signature.

	self assert: bytes = (blob signature bytes)! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:53'!
testLooseObjectNames
	| blob names |
	reference := (FSFilesystem inMemory / 'test.git').
	repository := GitRepository on: reference.
	blob := repository storeObject: (GitBlob bytes: 'test').
	names := repository looseObjectNames.
	self assert: names size = 1.
	self assert: names first species = (blob signature species).
	self assert: names first = (blob signature).
	"blob is unreferenced, hence we create a new empty repos"
	repository baseDir delete.! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 8/11/2011 22:58'!
testNew
	self should: [ GitRepository new ] raise: Error! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:38'!
testObjectWithSignature
	| initialBlob loadedBlob |
	initialBlob := GitBlob bytes: 'test'.
	repository storeObject: initialBlob.
	loadedBlob := repository objectWithSignature: initialBlob signature.
	self assert: initialBlob = loadedBlob.
	"blob is unreferenced, hence we create a new empty repos"
	repository baseDir delete.! !

!GitRepositoryTests methodsFor: 'class-tests' stamp: 'MaxLeske 8/11/2011 23:00'!
testOnClass
	self should: [ GitRepository on: FSFilesystem onDisk ]
		raise: Error
		description: 'Please provide an instance of a filesystem, not a class.'.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 15:27'!
testResolveRef
	| blob resolved |
	blob := repository storeObject: (GitBlob bytes: 'test').
	repository updateRef: 'HEAD' to: blob signature.
	
	resolved := repository resolveRef: 'HEAD'.
	self assert: blob signature = resolved! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 6/30/2010 08:20'!
testSetupDirectoryStructure
	| dirs |
	dirs := #(
		'HEAD'
		'info'
		'objects'
		'objects/pack'
		'objects/info'
		'refs'
		'refs/heads'
		'refs/tags').
	dirs do: [ :dir | self assert: (repository baseDir resolveString: dir) exists ]! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:39'!
testStoreObjectBlob
	| blob storedObject signature hash |
	blob := GitBlob bytes: 'test'.
	signature := blob signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: blob.
	self assert: storedObject = blob.
	"blob is unreferenced, hence we create a new empty repos"
	repository baseDir delete.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:44'!
testStoreObjectCommit
	| tree commit signature hash storedObject |
	tree := GitTree new.
	commit := GitCommit new.
	commit
		author: self defaultStamp;
		committer: self defaultStamp;
		tree: tree;
		message: 'testCommit'.
	signature := commit signature.
	hash := SecureHashAlgorithm new hashMessage: signature bytes.
	hash := hash hex asLowercase forceTo: 40 paddingStartWith: $0.
	storedObject := repository storeObject: commit.
	self assert: storedObject = commit.
	"tree is unreferenced, hence we create a new empty repos"
	repository baseDir delete.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/11/2011 22:59'!
testStoreObjectCommitNoAuthor
	| commit |
	commit := GitCommit new.
	commit tree: GitTree new.
	self 
		should: [ repository storeObject: commit ] 
		raise: GitInvalidObjectError
		whoseDescriptionIncludes: 'Author identity is required.'
		description: 'Author needs to be set before storing the commit.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/11/2011 22:59'!
testStoreObjectCommitNoTree
	| commit |
	commit := GitCommit new.
	self 
		should: [ repository storeObject: commit ] 
		raise: GitInvalidObjectError
		whoseDescriptionIncludes: 'Cannot have a commit without a tree.'
		description: 'Every commit needs a tree. Make sure this is checked for.'! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:40'!
testStoreObjectTree
	| tree storedObject |
	tree := GitTree new.
	storedObject := repository storeObject: tree.
	self assert: storedObject = tree.
	"tree is unreferenced, hence we create a new empty repos"
	repository baseDir delete.! !

!GitRepositoryTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:05'!
testTags
	| firstCommit tag tags |
	firstCommit := repository head parents first.
	tag := GitTag
		makeLightWeightTag: 'tag1'
		for: firstCommit
		in: repository.
	
	tags := repository tags.
	self assert: tags size = 1.
	self assert: tags first name = 'tag1'.
	self assert: tags first object = firstCommit! !

GitTestCase subclass: #GitStampTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitStampTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testPrintOn; add: #testStampReading; add: #testNow; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitStampTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:34'!
testNow
	| stamp now |
	stamp := self defaultStamp.
	now := TimeStamp now.
	
	self assert: stamp timeStamp <= now.	
	self assert: stamp name = GitStamp defaultName.
	self assert: stamp email = GitStamp defaultEmail.! !

!GitStampTests methodsFor: 'tests' stamp: 'CamilloBruni 6/22/2011 05:35'!
testPrintOn
	| stamp template now |
	now := TimeStamp now.
	stamp := self defaultStamp.
	stamp timeStamp: now.
	
	template := 
		GitStamp defaultName, ' <',GitStamp defaultEmail,'> ', 
		now asUnixTime asString, 
		' ',
		(now offset positive ifTrue: ['+'] ifFalse: ['-']),
		(now offset hours printString forceTo: 2 paddingStartWith: $0),
		(now offset minutes printString forceTo: 2 paddingStartWith: $0).
	
	self assert: stamp printString = template! !

!GitStampTests methodsFor: 'tests' stamp: 'MaxLeske 7/22/2010 18:48'!
testStampReading
	| str stamp |
	str := 'Tony Garnock-Jones <tonygarnockjones@gmail.com> 1266614121 +1300'.
	stamp := GitStamp fromString: str.
	self assert: str = stamp printString.! !

GitTestCase subclass: #GitStorableObjectTests
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitStorableObjectTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testTypeCode; add: #testForType; add: #testWithSignature; add: #testLoadFrom; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitStorableObjectTests class methodsFor: 'package selection' stamp: 'MaxLeske 7/24/2010 17:26'!
packageNamesUnderTest
	^#('FS-Git-Plumbing')! !

!GitStorableObjectTests methodsFor: 'helpers' stamp: 'MaxLeske 8/12/2011 09:04'!
buildMockFor: aGitObjectClass
	| tree fakeName |
	fakeName := GitObjectSignature for: (GitBlob bytes: 'test').
	tree := GitTree entries: {
		(GitTreeEntry 
			entryName: 'testFile' 
			mode: GitTreeEntry fileMode 
			objectSignature: fakeName)}.
	aGitObjectClass = GitBlob
		ifTrue: [ ^ GitBlob bytes: 'test' ].
	aGitObjectClass = GitCommit
		ifTrue: [ 
			^ (GitCommit new)
				author: self defaultStamp;
				committer: self defaultStamp;
				tree: tree;
				message: 'initial commit';
				yourself ].
	aGitObjectClass = GitTag
		ifTrue: [ 
			^ GitTag
				makeFullTag: 'testTag'
				for: (repository storeObject: (self buildMockFor: GitCommit))
				message: 'aMessage'
				tagger: self defaultStamp
				in: repository ].
	aGitObjectClass = GitTree
		ifTrue: [ ^ GitTree entries: {(GitTreeEntry entryName: 'testFile' mode: GitTreeEntry fileMode objectSignature: fakeName)} ].
	self error: 'the class you provided needs to be instantiated to test #loadFrom:. Add a new entry to #buildMockFor:'! !

!GitStorableObjectTests methodsFor: 'running' stamp: 'CamilloBruni 7/13/2011 19:52'!
setUp
	super setUp.
	repository := GitRepository on: (FSFilesystem inMemory / 'repo')! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 8/11/2011 23:02'!
testForType
	GitStorableObject allSubclasses do: [ :each | each typeCode isNil 
		ifFalse: [ self assert: (GitStorableObject forType: each typeCode) = each ]
		ifTrue: [ self should: [ GitStorableObject forType: each typeCode ] raise: GitInvalidObjectError ] ]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 7/26/2010 09:00'!
testLoadFrom
	| stream object loadedObject clazzes |
	clazzes := GitStorableObject allSubclasses reject: [:each | each typeCode isNil].
	clazzes do:
		[:each | object := self buildMockFor: each.
		repository storeObject: object.
		stream := repository looseObjectStreamAt: object signature.
		loadedObject := GitStorableObject loadFrom: stream from: repository. 
		self assert: loadedObject isNil not.
		self assert: loadedObject species = (object species).
		self assert: loadedObject = object]! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/24/2010 14:46'!
testTypeCode
	self assert: GitStorableObject typeCode isNil! !

!GitStorableObjectTests methodsFor: 'class-tests' stamp: 'MaxLeske 6/26/2010 22:11'!
testWithSignature
	| instance signature |
	signature := GitObjectSignature from: 234.
	instance := GitStorableObject withSignature: signature.
	self assert: instance signature = signature.
	self assert: instance signature == signature! !

GitTestCase subclass: #GitTagTests
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTagTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testMakeLightTag; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitTagTests methodsFor: 'running' stamp: 'CamilloBruni 7/13/2011 19:52'!
setUp
	super setUp.
	repository := GitRepository on: (FSFilesystem inMemory / 'testRepo')! !

!GitTagTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:05'!
testMakeLightTag
	| commit tree blob |
	blob := GitBlob bytes: 'test'.
	tree := GitTree entries: {
		GitTreeEntry
			entryName: 'testFile'
			mode: GitTreeEntry fileMode
			objectSignature: blob signature}.
	commit := [ [ GitCommit withDefaultStamps ]
		on: GitStampNameRequest
		do: [ :req | req resume: 'Homer Simpson' ] ]
			on: GitStampEmailRequest
			do: [ :req | req resume: 'homer@nuke.com' ].
	commit
		tree: tree;
		message: 'myCommit'.
	
	GitTag makeLightWeightTag: 'testTag' for: commit in: repository.	
	self assert: (repository tagsDir / 'testTag') exists! !

GitTestCase subclass: #GitTaglikeObjectTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTaglikeObjectTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testRetrievingProperties; add: #testLoadingAssociations; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'CamilloBruni 7/11/2011 17:35'!
testLoadingAssociations
	| taglikeObject |
	taglikeObject := GitTaglikeObject new.
	self assert: taglikeObject properties isEmpty.
	
	taglikeObject loadKey: 'keyname' value: 'keyobject'.
	self assert: (taglikeObject properties includesKey: 'keyname').
	self assert: (taglikeObject properties at: 'keyname') == 'keyobject'
	
	! !

!GitTaglikeObjectTests methodsFor: 'tests' stamp: 'MaxLeske 6/27/2010 20:44'!
testRetrievingProperties
	| object collection |
	self assert: GitTaglikeObject new properties isEmpty.
	
	collection := OrderedCollection new.
	collection add: Object new.
	object := GitTaglikeObject new.
	object properties: collection.
	
	self assert: object properties = collection.
	self assert: object properties isEmpty not! !

!GitTestCase methodsFor: 'helpers' stamp: 'MaxLeske 8/12/2011 09:58'!
defaultStamp
	^ GitStamp now! !

!GitTestCase methodsFor: 'running' stamp: 'MaxLeske 8/12/2011 10:03'!
setUp
	"Setting these variables for the tests ensures that no FillInTheBlankMorph will
	stall the building of the image and that tests don't require user interaction."
	GitStamp ensureDefaultValues! !

!GitTestCase methodsFor: 'running' stamp: 'MaxLeske 8/12/2011 10:03'!
tearDown
	GitStamp
		defaultName: nil;
		defaultEmail: nil! !

GitTestCase subclass: #GitTreeEntryTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTreeEntryTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testPrintMode; add: #testSize; add: #testLessThanOrEqual; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitTreeEntryTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:46'!
testLessThanOrEqual
	| entry1 entry2 entry3 entry4 entry5 |
	entry1 := GitTreeEntry 
		entryName: 'Connencton.doc'
		mode: nil
		objectSignature: nil.
	entry2 := GitTreeEntry 
		entryName: 'GitFS note.txt'
		mode: nil
		objectSignature: nil.
	entry3 := GitTreeEntry
		entryName: 'dummy.docx'
		mode: nil
		objectSignature: nil.
	entry4 := GitTreeEntry
		entryName: 'aFolder'
		mode: GitTreeEntry dirMode
		objectSignature: nil.
	entry5 := GitTreeEntry
		entryName: 'aFolder'
		mode: GitTreeEntry fileMode
		objectSignature: nil.
	
	self assert: entry1 <= entry2.
	self assert: entry1 <= entry3.
	self assert: entry2 <= entry3. "asciiValue of $G is smaller than that of $d"
	self assert: entry5 <= entry4 "entries for tree objects are compared with a trailing slash"! !

!GitTreeEntryTests methodsFor: 'class-tests' stamp: 'MaxLeske 7/10/2010 11:18'!
testPrintMode
	self assert: (GitTreeEntry printMode: GitTreeEntry fileMode) = '100644'.
	self assert: (GitTreeEntry printMode: GitTreeEntry dirMode) = '40000'! !

!GitTreeEntryTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testSize
	| treeEntry fileName mode signature sizeToBe |
	fileName := 'testFile'.
	mode := GitTreeEntry printMode: GitTreeEntry fileMode.
	signature := (GitBlob bytes: 'fooBlob') signature.
	treeEntry := GitTreeEntry 
					entryName: fileName 
					mode: GitTreeEntry fileMode
					objectSignature: signature.
					
	sizeToBe := 
		(mode size) +
		(Character space asInteger asByteArray size) +
		(fileName size) +
		(0 decimalDigitLength) +
		(signature size).
	
	self assert: signature size = 20.
	self assert: treeEntry size = sizeToBe.! !

GitTestCase subclass: #GitTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing-Tests'!

!GitTreeTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testAt; add: #testNoEntries; add: #testSettingEntries; add: #testRemoveEntry; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!GitTreeTests methodsFor: 'tests' stamp: 'CamilloBruni 7/13/2011 19:52'!
testAt
	| tree entry blob repo |
	blob := GitBlob bytes: 'testblob'.
	repo := GitRepository on: (FSFilesystem inMemory / 'testRepo').
	repo storeObject: blob.
	entry := GitTreeEntry
		entryName: 'testFile'
		mode: GitTreeEntry fileMode
		objectSignature: blob signature.
	tree := GitTree in: repo.
	tree entries: {entry}.
	
	self assert: (tree at: entry entryName) = blob! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 6/26/2010 22:30'!
testNoEntries
	self assert: (GitTree new entries) species = Dictionary.
	self assert: (GitTree new entries) isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testRemoveEntry
	| tree collection entry1 entry2 entry3 |
	entry1 := GitTreeEntry
		entryName: 'entry 1'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 1).
	entry2 := GitTreeEntry
		entryName: 'entry 2'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 2).
	entry3 := GitTreeEntry
		entryName: 'entry 3'
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 3).
	collection := OrderedCollection new add: entry1; add: entry2; add: entry3; yourself.
	tree := GitTree entries: collection.
	
	self assert: tree entries size = 3.
	self assert: (tree entries includesKey: entry1 entryName).
	self assert: (tree entries includesKey: entry2 entryName).
	self assert: (tree entries includesKey: entry3 entryName).
	
	tree removeEntryAt: entry2 entryName.
	self assert: tree entries size = 2.
	self deny: (tree entries includesKey: entry2 entryName).
	
	tree removeEntryAt: entry3 entryName.
	self assert: tree entries size = 1.
	self deny: (tree entries includesKey: entry3 entryName).
	
	tree removeEntryAt: entry1 entryName.
	self assert: tree entries isEmpty! !

!GitTreeTests methodsFor: 'tests' stamp: 'MaxLeske 7/10/2010 10:39'!
testSettingEntries
	| collection tree entry1 entry2 |
	collection := OrderedCollection new.
	entry1 := GitTreeEntry 
		entryName: 'object1' 
		mode: GitTreeEntry fileMode
		objectSignature: (GitObjectSignature from: 234).
	entry2 := GitTreeEntry 
		entryName: 'object2' 
		mode: GitTreeEntry dirMode
		objectSignature: (GitObjectSignature from: 564).
	collection add: entry1; add: entry2.
	tree := GitTree entries: collection.
	
	self assert: (tree entries at: entry1 entryName)= entry1.
	self assert: (tree entries at: entry2 entryName) = entry2.
	self deny: entry1 = entry2! !

TestCase subclass: #MCGitWriterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello-Tests'!

!MCGitWriterTest class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new) add: #testMethodPathForClass; add: #testMethodPathForMetaClass; yourself)); add: (#failures->((Set new))); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!MCGitWriterTest methodsFor: 'tests' stamp: 'StefanMarr 5/18/2011 23:27'!
testMethodPathForClass
	"Most basic test, just making sure that it does not raise errors"
	| writer |
	writer := MCGitWriter new.
	writer stream: (FSLocator imageDirectory).
	self shouldnt: [ writer methodPathFor: (Integer>>#even) methodReference ] raise: Error.  ! !

!MCGitWriterTest methodsFor: 'tests' stamp: 'StefanMarr 5/18/2011 23:27'!
testMethodPathForMetaClass
	"Most basic test, just making sure that it does not raise errors"
	| writer |
	writer := MCGitWriter new.
	writer stream: (FSLocator imageDirectory).
	self shouldnt: [ writer methodPathFor: (Integer class>>#new) methodReference ] raise: Error.  ! !

TestCase subclass: #PackfileTests
	instanceVariableNames: 'packReader filesystem repository packFile indexFile'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Tests'!

!PackfileTests class methodsFor: 'history' stamp: 'CamilloBruni 8/1/2011 16:49'!
lastStoredRun
	^ ((Dictionary new) add: (#errors->((Set new))); add: (#passed->((Set new))); add: (#failures->((Set new) add: #testVersion2PackfileStructure; yourself)); add: (#timeStamp->(DateAndTime basicNew instVarAt: 1 put: 60559; instVarAt: 2 put: (Duration seconds: 7200 nanoSeconds: 0); instVarAt: 3 put: 2455775; instVarAt: 4 put: 0; yourself)); yourself)! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 20:30'!
calculateChecksum
	| position packData signature |
	position := packReader position.
	self assert: (packReader skip: 20; atEnd).
	
	packReader position: 0.
	packData := packReader next: position.
	packReader position: position.
	
	"GitObjectSignature already has all the code for dealing with this. No point in copying it here."
	signature := GitObjectSignature new.
	
	^ signature hexHashFrom: (signature integerHashFrom:  packData)! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/1/2011 20:20'!
compressedDataSize 
	"Expand the data for two reasons:
		1: to find next header we need to know the length
			of the compressed data.
		2: verify compressed data"
	| position compressedSize |
	position := packReader position.
	compressedSize := self compressedDataSize: packReader upToEnd.
	packReader position: position.
	
	^ compressedSize! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/1/2011 20:20'!
compressedDataSize: aByteArray
	| zlibReader |
	zlibReader := ZLibReadStream on: aByteArray.
	^ zlibReader 
		contents;
		sourcePosition! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 21:07'!
getEntryHeadersAt: position
"The most significant bit of every header byte states if the current byte is followed
by another header byte or by data.
First bit = 1: another header byte follows.
First bit = 0: data follows.
Thus, the n header bytes of an entry consist of n-1 bytes who's most significant
bit is 1 and 1 byte who's most significant bit is 0."
	| bytes oldPosition |
	oldPosition := packReader position.
	packReader position: position.
	bytes := ByteArray streamContents: [ :stream |			
		[ (packReader peek bitAnd: 2r10000000) isZero ] whileFalse:
			[ stream nextPut: packReader next ].
		stream nextPut: packReader next ].
	
	packReader position: oldPosition.
	^ bytes readStream ! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/1/2011 19:37'!
getSizeFromHeaders: aStream
"aStream contains the n header bytes of a pack entry.
The bytes are in network byte order (last  byte is most significant).
The last four bits of the first byte are the four least significant
bits of the bits describing the size of the following data. 
The number of bits describing the size of the data is (n - 1) * 7 + 4.
Note that the most significant bit of each byte encodes if another header
byte follows or not and thus has to be stripped (#bitAnd:) from the bits
that describe the size.
The four bits from the first byte do not need to be shifted. The following
bits need to be shifted to the left so that they make up the most significant
part of the bits already present."
	| size shift byte |
	byte := aStream next.
	size := byte bitAnd: 2r00001111.
	shift := 4.
	[ (byte bitAnd: 2r10000000) isZero ] whileFalse: [
		byte := aStream next.
		size := size bitOr: ((byte bitAnd: 2r01111111) bitShift: shift).
		shift := shift + 7 ].
	
	^ size! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:06'!
hasBlobSignature
	^ (packReader peek bitAnd: 2r0111000) = 2r00110000! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 15:01'!
hasCommitData
"If the current entry (first byte of data) is really commit data then the first four bytes will spell 'tree'"
	| compressedSize stream position |
	position := packReader position.
	self skipEntryHeader.	
	compressedSize := self compressedDataSize.
	stream := (ZLibReadStream on: (packReader next: compressedSize)) contents readStream.
	packReader position: position.
	
	^ (stream next: 4) asString = GitTree typeCode! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 14:44'!
hasCommitSignature
	^ (packReader peek bitAnd: 2r01110000) = 2r00010000! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 16:57'!
hasOfsDeltaSignature 
	^ (packReader peek bitAnd: 2r01110000) = 2r01100000! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:09'!
hasRefDeltaSignature
	^ (packReader peek bitAnd: 2r01110000) = 2r01110000! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:03'!
hasTreeData
"If the current entry (first byte of data) is really tree data then the beginning of
the data will either be GitTreeEntry class>>dirMode of GitTreeEntry class>>fileMode."
	| compressedSize stream position mode |
	position := packReader position.
	self skipEntryHeader.
	compressedSize := self compressedDataSize.
	stream := (ZLibReadStream on: (packReader next: compressedSize)) contents readStream.
	packReader position: position.
	
	mode := (stream next: 6) asString.
	^ (mode = (GitTreeEntry printMode: GitTreeEntry dirMode)) | 
		(mode = (GitTreeEntry printMode: GitTreeEntry fileMode))! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 14:44'!
hasTreeSignature
	^ (packReader peek bitAnd: 2r01110000) = 2r00100000! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:07'!
isEntryBlob
	^ self hasBlobSignature! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 14:29'!
isEntryCommit
	^ self hasCommitSignature & self hasCommitData
	! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 19:42'!
isEntryOfsDelta
	^ self hasOfsDeltaSignature and: [
		self verifyNewFormatDeltification ]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 19:40'!
isEntryRefDelta
	^ self hasRefDeltaSignature and: [
		self verifyOldFormatDeltification ]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 14:42'!
isEntryTree
	^ self hasTreeSignature & self hasTreeData
	! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 7/31/2011 19:23'!
newPackBytes
	^  #[80 65 67 75 0 0 0 2 0 0 0 9 150 13 120 156 149 139 91 10 194 48 16 0 255 115 138 252 11 178 121 108 55 1 17 15 160 135 216 108 55 182 216 90 169 17 122 124 235 17 252 27 102 152 182 170 90 101 116 142 144 130 71 32 207 133 69 188 171 66 49 11 1 64 236 92 95 36 33 153 23 175 250 108 118 143 93 14 89 107 140 61 86 238 37 84 12 20 0 74 244 26 139 79 41 139 68 135 134 63 109 88 86 123 227 205 94 245 253 80 123 154 121 155 126 116 185 207 60 78 71 89 230 179 117 1 82 206 72 169 179 7 240 0 102 183 243 216 154 254 127 154 50 241 16 204 23 77 115 62 194 150 13 120 156 149 139 91 10 194 48 16 69 255 179 138 252 11 146 231 204 4 68 92 128 46 98 154 76 109 177 177 18 35 116 249 214 37 248 113 15 135 3 183 55 17 157 40 66 16 15 37 0 3 185 209 122 65 11 3 140 81 172 47 41 228 125 96 7 84 47 110 242 236 26 209 56 199 62 27 95 8 101 180 129 176 56 114 92 40 83 142 152 130 13 176 3 21 127 250 180 54 125 227 77 95 229 253 16 125 170 188 45 63 187 220 43 207 203 49 175 245 172 173 55 148 82 4 31 245 193 56 99 212 94 235 220 187 252 255 84 195 194 147 83 95 224 204 61 250 149 10 120 156 149 203 81 10 194 48 12 128 225 247 158 34 239 130 164 235 82 82 16 241 0 122 136 164 75 221 176 101 176 85 216 241 213 35 248 246 243 193 223 55 51 48 95 226 100 69 67 33 13 22 39 38 239 35 101 206 129 179 103 149 113 80 138 99 82 39 239 62 175 27 60 228 128 187 237 47 131 75 147 163 254 234 246 108 178 212 115 94 219 21 124 64 78 137 136 17 78 56 32 186 175 182 165 119 251 255 116 90 101 118 31 148 230 49 222 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 48 103 100 92 116 181 218 225 112 237 202 45 218 25 125 137 175 167 245 79 177 3 0 173 195 12 141 177 147 14 120 156 237 205 193 109 4 33 12 5 208 123 186 68 17 72 120 124 136 132 118 74 163 129 105 108 205 214 144 67 164 60 14 8 99 255 231 215 235 215 206 87 174 24 249 236 182 102 100 31 43 102 244 145 45 86 21 209 114 157 251 188 171 219 214 179 79 245 236 232 121 30 53 89 193 103 175 140 209 86 253 213 228 8 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 196 127 34 94 87 86 224 202 18 143 52 231 247 108 247 79 235 71 248 56 49 239 103 103 155 81 153 82 238 90 153 171 125 166 79 166 122 243 44 45 117 246 196 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 238 15 112 111 165 188 86 95 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 184 177 191 38 44 194 47 78 171 54 240 83 221 202 119 135 115 14 101 70 23 1 0 193 127 13 180 111 129 20 120 156 219 248 136 241 233 67 198 141 31 88 44 55 7 88 62 52 7 0 65 214 7 142 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 208 72 72 217 216 126 75 228 238 170 77 130 207 194 103 41 197 139 79 104 126 14 0 184 12 12 230 104 71 120 156 123 250 144 113 69 241 134 21 150 0 22 10 4 116 128 203 29 149 43 4 45 128 160 238 7 34 175 90 90 243 30 93 116 212]! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/1/2011 17:33'!
newPackIndexBytes
	^ #[255 116 79 99 0 0 0 2 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 9 0 0 0 9 33 246 147 158 244 77 95 173 195 245 55 48 11 66 228 178 136 156 196 21 40 96 100 177 135 218 20 221 170 178 17 230 87 154 34 95 23 144 131 231 55 1 1 162 213 123 64 195 125 169 180 43 104 142 97 235 150 143 148 62 119 2 42 60 3 216 126 241 72 125 40 42 216 200 197 121 65 70 148 23 152 86 78 54 212 106 104 47 19 231 22 182 245 225 61 148 205 148 97 183 216 191 124 86 88 78 94 42 125 81 242 126 169 238 195 108 194 105 91 114 225 246 222 251 63 91 62 109 133 17 101 200 195 140 24 186 66 181 100 155 234 81 23 87 50 80 114 171 172 194 31 199 73 199 0 4 97 219 200 87 254 162 26 126 14 145 12 242 79 60 69 92 198 224 234 134 213 137 160 219 183 188 38 35 6 167 206 57 209 7 171 227 64 117 68 253 170 20 123 41 107 122 21 131 94 156 124 250 196 26 250 52 48 37 249 109 0 0 0 161 0 0 3 52 0 0 1 217 0 0 1 54 0 0 2 193 0 0 2 237 0 0 3 8 0 0 1 173 0 0 0 12 128 203 29 149 43 4 45 128 160 238 7 34 175 90 90 243 30 93 116 212 128 204 239 142 92 55 155 120 143 201 242 76 66 220 203 250 82 34 159 48]! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 7/31/2011 19:22'!
oldPackBytes
	^ #[80 65 67 75 0 0 0 2 0 0 0 9 150 13 120 156 149 139 91 10 194 48 16 0 255 115 138 252 11 178 121 108 55 1 17 15 160 135 216 108 55 182 216 90 169 17 122 124 235 17 252 27 102 152 182 170 90 101 116 142 144 130 71 32 207 133 69 188 171 66 49 11 1 64 236 92 95 36 33 153 23 175 250 108 118 143 93 14 89 107 140 61 86 238 37 84 12 20 0 74 244 26 139 79 41 139 68 135 134 63 109 88 86 123 227 205 94 245 253 80 123 154 121 155 126 116 185 207 60 78 71 89 230 179 117 1 82 206 72 169 179 7 240 0 102 183 243 216 154 254 127 154 50 241 16 204 23 77 115 62 194 150 13 120 156 149 139 91 10 194 48 16 69 255 179 138 252 11 146 231 204 4 68 92 128 46 98 154 76 109 177 177 18 35 116 249 214 37 248 113 15 135 3 183 55 17 157 40 66 16 15 37 0 3 185 209 122 65 11 3 140 81 172 47 41 228 125 96 7 84 47 110 242 236 26 209 56 199 62 27 95 8 101 180 129 176 56 114 92 40 83 142 152 130 13 176 3 21 127 250 180 54 125 227 77 95 229 253 16 125 170 188 45 63 187 220 43 207 203 49 175 245 172 173 55 148 82 4 31 245 193 56 99 212 94 235 220 187 252 255 84 195 194 147 83 95 224 204 61 250 149 10 120 156 149 203 81 10 194 48 12 128 225 247 158 34 239 130 164 235 82 82 16 241 0 122 136 164 75 221 176 101 176 85 216 241 213 35 248 246 243 193 223 55 51 48 95 226 100 69 67 33 13 22 39 38 239 35 101 206 129 179 103 149 113 80 138 99 82 39 239 62 175 27 60 228 128 187 237 47 131 75 147 163 254 234 246 108 178 212 115 94 219 21 124 64 78 137 136 17 78 56 32 186 175 182 165 119 251 255 116 90 101 118 31 148 230 49 222 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 48 103 100 92 116 181 218 225 112 237 202 45 218 25 125 137 175 167 245 79 177 3 0 173 195 12 141 177 147 14 120 156 237 205 193 109 4 33 12 5 208 123 186 68 17 72 120 124 136 132 118 74 163 129 105 108 205 214 144 67 164 60 14 8 99 255 231 215 235 215 206 87 174 24 249 236 182 102 100 31 43 102 244 145 45 86 21 209 114 157 251 188 171 219 214 179 79 245 236 232 121 30 53 89 193 103 175 140 209 86 253 213 228 8 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 68 34 145 72 36 18 137 196 127 34 94 87 86 224 202 18 143 52 231 247 108 247 79 235 71 248 56 49 239 103 103 155 81 153 82 238 90 153 171 125 166 79 166 122 243 44 45 117 246 196 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 14 135 195 225 112 56 28 238 15 112 111 165 188 86 95 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 184 177 191 38 44 194 47 78 171 54 240 83 221 202 119 135 115 14 101 70 23 1 0 193 127 13 180 127 55 1 1 162 213 123 64 195 125 169 180 43 104 142 97 235 150 143 148 62 120 156 219 248 136 241 233 67 198 141 31 88 44 55 7 88 62 52 7 0 65 214 7 142 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 208 72 72 217 216 126 75 228 238 170 77 130 207 194 103 41 197 139 79 104 126 14 0 184 12 12 230 120 216 191 124 86 88 78 94 42 125 81 242 126 169 238 195 108 194 105 91 114 120 156 123 250 144 113 69 241 134 21 150 0 22 10 4 116 186 55 135 207 98 139 116 127 42 115 209 155 183 207 21 210 63 246 127 155]! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/1/2011 17:34'!
oldPackIndexBytes
	^ #[255 116 79 99 0 0 0 2 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 5 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 6 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 7 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 8 0 0 0 9 0 0 0 9 33 246 147 158 244 77 95 173 195 245 55 48 11 66 228 178 136 156 196 21 40 96 100 177 135 218 20 221 170 178 17 230 87 154 34 95 23 144 131 231 55 1 1 162 213 123 64 195 125 169 180 43 104 142 97 235 150 143 148 62 119 2 42 60 3 216 126 241 72 125 40 42 216 200 197 121 65 70 148 23 152 86 78 54 212 106 104 47 19 231 22 182 245 225 61 148 205 148 97 183 216 191 124 86 88 78 94 42 125 81 242 126 169 238 195 108 194 105 91 114 225 246 222 251 63 91 62 109 133 17 101 200 195 140 24 186 66 181 100 155 234 81 23 87 50 80 114 171 172 194 31 199 73 199 0 4 97 219 200 87 254 162 26 126 14 145 12 242 79 60 69 92 198 224 234 134 213 137 160 219 183 188 38 35 205 236 7 179 209 7 171 227 64 117 68 253 170 20 123 41 129 10 32 104 94 156 124 250 196 26 250 52 48 37 249 109 0 0 0 161 0 0 3 70 0 0 1 217 0 0 1 54 0 0 2 193 0 0 2 237 0 0 3 26 0 0 1 173 0 0 0 12 186 55 135 207 98 139 116 127 42 115 209 155 183 207 21 210 63 246 127 155 32 129 183 53 182 202 246 131 141 151 241 229 42 129 80 88 101 205 117 166]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/3/2011 11:59'!
packOffsetOf: sha1
	| dummyFS pack |
	"GitPack already contains the necessary code."
	dummyFS := FSFilesystem inMemory.
	(dummyFS root / 'pack.idx') writeStreamDo: [ :stream | 
		stream nextPutAll: self oldPackIndexBytes ].
	(dummyFS root / 'pack.pack') writeStreamDo: [ :stream |
		stream nextPutAll: self oldPackBytes ].
	 
	pack := GitPack new.
	pack basePath: dummyFS root / 'pack'.
	^ pack offsetAt: (pack tableIndexForSha: sha1 )! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/3/2011 11:59'!
setUp
	| packsDir |
	filesystem := FSFilesystem inMemory.
	repository := FSGitRepository on: (filesystem root / 'repo').
	repository repository headFile writeStreamDo: [ :stream | 
		stream nextPutAll: '21f6939ef44d5fadc3f537300b42e4b2889cc415' ] .
	
	packsDir := filesystem root resolve: 'repo/.git/objects/pack'.
	packsDir ensureDirectory.	
	packFile := packsDir / 'pack-bbb272fda736770dc087758f6fc7c4b6324ea146.pack'.
	indexFile := packsDir / 'pack-bbb272fda736770dc087758f6fc7c4b6324ea146.idx'! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/1/2011 18:46'!
skipBaseObjectOffset
"The base object offset has a special encoding (basically it is the
same encoding used for encoding the size of object data).
See http://git.rsbx.net/Documents/Git_Data_Formats.txt.
This is the Smalltalk code that would decode the offset:

| byte offset |
	byte := packReader next.
	offset := byte bitAnd: 2r01111111.
	[ (byte bitAnd: 2r10000000) isZero ] whileFalse: [
		byte := packReader next.
		offset := ((offset + 1) bitShift: 7) bitOr: (byte bitAnd: 2r01111111) ]"
	
	self skipEntryHeader "same code"! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:22'!
skipBaseObjectReference
	packReader skip: 20! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 16:24'!
skipChecksum
	packReader skip: 20! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/30/2011 14:18'!
skipEntryHeader
	"First bit of 1: another header byte follows.
	First bit of 0: no more header bytes. Next byte is data."
	[ (packReader peek bitAnd: 2r10000000) isZero ] whileFalse: [ packReader next ].
	"skip last header byte"
	packReader next! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/12/2011 09:10'!
tagPackBytes
	^  #[80 65 67 75 0 0 0 2 0 0 0 4 146 10 120 156 149 139 81 10 194 48 16 5 255 115 138 253 23 228 165 219 198 44 136 120 0 61 196 166 217 104 177 161 80 35 244 248 218 35 200 252 12 3 211 86 51 210 28 56 13 81 78 204 162 210 13 60 50 163 196 94 44 100 69 73 57 0 18 75 114 250 105 207 101 165 187 110 116 179 247 203 232 92 117 155 119 187 62 170 78 243 113 92 234 133 60 239 0 189 167 3 58 192 253 106 157 90 179 255 79 231 221 23 154 125 47 176 203 8 120 156 21 139 65 14 194 32 20 5 247 156 226 239 77 204 135 210 138 137 49 30 64 15 241 129 23 82 45 210 20 98 234 237 149 204 102 102 49 197 63 17 26 153 160 29 199 96 217 69 48 155 56 156 135 41 88 136 7 28 91 241 39 227 252 52 106 175 218 119 5 133 146 243 220 84 147 68 31 108 117 46 111 221 35 97 163 135 236 116 71 125 129 46 89 246 165 219 45 101 153 151 227 127 186 146 30 58 108 236 72 7 54 204 74 9 101 212 42 9 234 7 238 138 42 233 161 2 120 156 51 52 48 48 51 49 81 72 203 204 73 53 100 216 238 173 116 47 176 209 187 45 45 70 225 81 196 174 206 219 11 207 229 159 2 0 183 237 13 168 191 7 120 156 203 201 33 15 112 101 83 3 112 1 0 64 254 52 140 80 105 116 186 20 195 221 191 143 108 193 129 108 96 159 8 250 223 160 142]! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/12/2011 09:11'!
tagPackIndexBytes
	^   #[255 116 79 99 0 0 0 2 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 1 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 2 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 3 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 0 0 0 4 44 24 13 196 8 222 0 45 57 54 196 234 190 232 4 171 114 139 101 27 62 196 115 13 64 167 114 88 184 119 2 165 22 243 81 161 158 87 189 20 173 99 181 137 115 57 169 37 60 51 15 132 158 109 160 251 214 0 152 251 183 75 34 222 81 129 75 134 102 92 32 226 88 186 137 219 161 206 111 202 170 215 26 180 124 224 168 84 18 202 209 46 50 184 88 172 0 0 0 12 0 0 0 128 0 0 1 0 0 0 1 44 80 105 116 186 20 195 221 191 143 108 193 129 108 96 159 8 250 223 160 142 173 216 13 96 82 36 81 4 106 0 139 82 104 33 165 209 30 237 80 225]! !

!PackfileTests methodsFor: 'tests' stamp: 'MaxLeske 7/31/2011 19:24'!
testPackfileStructureWithNewDeltas
"see pack-format.txt in the 'technical' directory of the git sources.
Another good resource is this document: http://git.rsbx.net/Documents/Git_Data_Formats.txt.
Also, read comments in #verifyPackfileHeader and
#verifyPackfileEntries"
	packReader := self newPackBytes readStream.
		
	self 
		verifyPackfileHeader;
		verifyPackfileEntries! !

!PackfileTests methodsFor: 'tests' stamp: 'MaxLeske 7/31/2011 20:30'!
testPackfileStructureWithOldDeltas
"see pack-format.txt in the 'technical' directory of the git sources.
Another good resource is this document: http://git.rsbx.net/Documents/Git_Data_Formats.txt.
Also, read comments in #verifyPackfileHeader and
#verifyPackfileEntries"
	packReader := self oldPackBytes readStream.
		
	self 
		verifyPackfileHeader;
		verifyPackfileEntries! !

!PackfileTests methodsFor: 'tests' stamp: 'MaxLeske 8/1/2011 18:40'!
testReadingNewPackfile
	self 
		useNewPack;
		verifyReadingPackfile! !

!PackfileTests methodsFor: 'tests' stamp: 'MaxLeske 8/1/2011 18:40'!
testReadingOldPackfile
	self 
		useOldPack;
		verifyReadingPackfile! !

!PackfileTests methodsFor: 'tests' stamp: 'MaxLeske 8/12/2011 09:36'!
testReadingPackedTag
	| loadedObject |
	self useTagPack.
	
	self 
		shouldnt: [ loadedObject := repository repository objectNamed: 
			'3ec4730d40a77258b87702a516f351a19e57bd14' ]
		raise: Error.
	self assert: GitTag equals: loadedObject species.
	
	self assert: 1 equals: repository repository tags size! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 16:52'!
tryNewDeltaFormatForOffsets
	"the positions are given as additions because git uses zero based
	indices while Smalltalk uses 1 based indices. So the byte at position
	12 in the file is at position 12 + 1 in the ByteArray.
	
	The first header byte of each entry has the form
	xyyyzzzz
	x: 	if 1then another header byte follows
		if 0 then data follows
	y: 	001 - OBJ_COMMIT
		010 - OBJ_TREE
		011 - OBJ_BLOB
		100 - OBJ_TAG
		110 - OBJ_OFS_DELT (new packfile format)
		111 - OBJ_REF_DELTA (old packfile format)
	z: four least significant bits of bits describing size (see #getSizeFromHeaders:)"
	| content |
	content := packReader contents.
	self assert: 2r00010000 equals: ((content at: 12+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00010000 equals: ((content at: 161+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00010000 equals: ((content at: 310+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00100000 equals: ((content at: 429+1) bitAnd: 2r01110000). "tree"
	self assert: 2r00110000 equals: ((content at: 473+1) bitAnd: 2r01110000). "blob"
	self assert: 2r00100000 equals: ((content at: 705+1) bitAnd: 2r01110000). "tree"
	self assert: 2r01100000 equals: ((content at: 749+1) bitAnd: 2r01110000). "ofs_delta (blob)"
	self assert: 2r00100000 equals: ((content at: 776+1) bitAnd: 2r01110000). "tree"
	self assert: 2r01100000 equals: ((content at: 820+1) bitAnd: 2r01110000) "ofs_delta (blob)"! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 20:02'!
tryNewDeltaFormatForSizes
	self 
		assert: 214 
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 12)).
	self
		assert: 214
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 161)).
	self
		assert: 165
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 310)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 429)).
	self
		assert: 28977
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 473)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 705)).
	self
		assert: 15
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 749)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 776)).
	self
		assert: 8
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 820))! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 16:52'!
tryOldDeltaFormatForOffsets
	"the positions are given as additions because git uses zero based
	indices while Smalltalk uses 1 based indices. So the byte at position
	12 in the file is at position 12 + 1 in the ByteArray.
	
	The first header byte of each entry has the form
	xyyyzzzz
	x: 	if 1then another header byte follows
		if 0 then data follows
	y: 	001 - OBJ_COMMIT
		010 - OBJ_TREE
		011 - OBJ_BLOB
		100 - OBJ_TAG
		110 - OBJ_OFS_DELT (new packfile format)
		111 - OBJ_REF_DELTA (old packfile format)
	z: four least significant bits of bits describing size (see #getSizeFromHeaders:)"
	| content |
	content := packReader contents.
	self assert: 2r00010000 equals: ((content at: 12+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00010000 equals: ((content at: 161+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00010000 equals: ((content at: 310+1) bitAnd: 2r01110000). "commit"
	self assert: 2r00100000 equals: ((content at: 429+1) bitAnd: 2r01110000). "tree"
	self assert: 2r00110000 equals: ((content at: 473+1) bitAnd: 2r01110000). "blob"
	self assert: 2r00100000 equals: ((content at: 705+1) bitAnd: 2r01110000). "tree"
	self assert: 2r01110000 equals: ((content at: 749+1) bitAnd: 2r01110000). "ref_delta (blob)"
	self assert: 2r00100000 equals: ((content at: 794+1) bitAnd: 2r01110000). "tree"
	self assert: 2r01110000 equals: ((content at: 838+1) bitAnd: 2r01110000) "ref_delta (blob)"! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 19:33'!
tryOldDeltaFormatForSizes
	self 
		assert: 214 
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 12)).
	self
		assert: 214
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 161)).
	self
		assert: 165
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 310)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 429)).
	self
		assert: 28977
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 473)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 705)).
	self
		assert: 15
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 749)).
	self
		assert: 33
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 794)).
	self
		assert: 8
		equals: (self getSizeFromHeaders: (self getEntryHeadersAt: 838))! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/1/2011 17:33'!
useNewPack
	packFile writeStreamDo: [ :stream |
		stream nextPutAll: self newPackBytes ].
	indexFile writeStreamDo: [ :stream |
		stream nextPutAll: self newPackIndexBytes ] ! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/1/2011 19:01'!
useOldPack
	packFile writeStreamDo: [ :stream |
		stream nextPutAll: self oldPackBytes ].
	indexFile writeStreamDo: [ :stream |
		stream nextPutAll: self oldPackIndexBytes ] ! !

!PackfileTests methodsFor: 'running' stamp: 'MaxLeske 8/12/2011 09:47'!
useTagPack
	packFile writeStreamDo: [ :stream |
		stream nextPutAll: self tagPackBytes ].
	indexFile writeStreamDo: [ :stream |
		stream nextPutAll: self tagPackIndexBytes ].
	(repository repository tagsDir / 'version1') writeStreamDo: [ :stream |
		stream nextPutAll: '3ec4730d40a77258b87702a516f351a19e57bd14' ]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/3/2011 11:59'!
verifyDeltificationWithBaseAt: baseOffset
	| interpreter baseSize finalSize finalObjectBytes baseObjectBytes result dummyFS pack position unpackedDelta |
	"GitDeltaInterpreter GitPack and GitPackedObject already contain the necessary code."
	interpreter := GitDeltaInterpreter new.
	position := packReader position.
	unpackedDelta := (ZLibReadStream on: packReader upToEnd) contents.
	packReader position: position.
	interpreter 
		instVarNamed: 'commands' put: unpackedDelta readStream;
		instVarNamed: 'instructions' put: unpackedDelta. 

	dummyFS := FSFilesystem inMemory.
	(dummyFS root / 'pack.idx') writeStreamDo: [ :stream | 
		stream nextPutAll: self oldPackIndexBytes ].
	(dummyFS root / 'pack.pack') writeStreamDo: [ :stream |
		stream nextPutAll: self oldPackBytes ].
	 
	pack := GitPack new.
	pack basePath: dummyFS root / 'pack'.

	baseSize := interpreter readBaseObjectSize.
	finalSize := interpreter readFinalObjectSize.
	baseObjectBytes := (pack packedObjectAtOffset: baseOffset) unpackedBytes.
	finalObjectBytes := interpreter runAgainst: baseObjectBytes.
	
	result := baseObjectBytes size = baseSize.
	^ result & (finalObjectBytes size = finalSize)! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 16:52'!
verifyEntryOffsets
	[ self tryNewDeltaFormatForOffsets ]
		on: TestFailure
		do: [ self tryOldDeltaFormatForOffsets ]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 16:51'!
verifyEntrySizes
	[ self tryNewDeltaFormatForSizes ]
		on: TestFailure
		do: [ self tryOldDeltaFormatForSizes ]! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 22:10'!
verifyNewFormatDeltification
	| packedObject position result |
	"GitPackedObject already contains the necessary code."
	position := packReader position.
	packedObject := GitPackedObject new.
	packedObject instVarNamed: 'file' put: packReader.

	self skipEntryHeader.
	result := self verifyDeltificationWithBaseAt: position - packedObject readBaseObjectOffset.
	packReader position: position.
	^ result! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 19:57'!
verifyOldFormatDeltification
	| baseObjectName position result |
	position := packReader position.
	self skipEntryHeader.
	baseObjectName := (packReader next: 20) hex.
	result := self verifyDeltificationWithBaseAt: (self packOffsetOf: baseObjectName).
	packReader position: position.
	^ result! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/2/2011 19:50'!
verifyPackfileEntries
"see pack-format.txt in the 'technical' directory of the git sources.
Also, take a look at the comments in #verifyEntryOffsets, #verifyEntrySizes
and #expandAndVerify:."

	| sha1 checksum calculatedChecksum |
	self 
		verifyEntryOffsets;
		verifyEntrySizes.

	self assert: self isEntryCommit.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	self assert: self isEntryCommit.	
	self skipEntryHeader.
	packReader skip: self compressedDataSize.

	self assert: self isEntryCommit.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	self assert: self isEntryTree.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	self assert: self isEntryBlob.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	self assert: self isEntryTree.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	[ self assert: self isEntryRefDelta.
	self skipEntryHeader.
	"deltified objects of type ref_delta reference their base object via SHA-1 name:"
	self skipBaseObjectReference.
	packReader skip: self compressedDataSize ]
		on: TestFailure
		do: [ self assert: self isEntryOfsDelta.
			self skipEntryHeader.
			"deltified objects of type ofs_delta reference their base object via offset:"
			self skipBaseObjectOffset.
			packReader skip: self compressedDataSize ].
	
	self assert: self isEntryTree.
	self skipEntryHeader.
	packReader skip: self compressedDataSize.
	
	[ self assert: self isEntryRefDelta.
	self skipEntryHeader.
	"deltified objects of type ref_delta reference their base object via SHA-1 name:"
	self skipBaseObjectReference.
	packReader skip: self compressedDataSize ]
		on: TestFailure
		do: [ self assert: self isEntryOfsDelta.
			self skipEntryHeader.
			"deltified objects of type ofs_delta reference their base object via offset:"
			self skipBaseObjectOffset.
			packReader skip: self compressedDataSize ].

	calculatedChecksum := self calculateChecksum.
	checksum := packReader next: 20.
	self 
		assert: checksum hex
		equals: calculatedChecksum.
	
	self assert: packReader atEnd! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 7/31/2011 19:32'!
verifyPackfileHeader
"see pack-format.txt in the 'technical' directory of the git doc'"
	| content |
	"4-byte signature: 'P', 'A', 'C', 'K'"
	content := packReader next: 4.
	self assert: 'PACK' equals: content asString.
	
	"4-byte version number (network byte order): 2 or 3 "
	content := packReader next: 4.
	self assert: #[0 0 0 2] equals: content.  "version 2"
	
	"4-byte number of objects contained in the pack (network byte order)"
	content := packReader next: 4.
	self assert: #[0 0 0 9] equals: content  "9 objects" ! !

!PackfileTests methodsFor: 'helpers' stamp: 'MaxLeske 8/12/2011 09:30'!
verifyReadingPackfile
	| workingCopy|
	workingCopy := repository head.
	self assert: 'blah2' equals: workingCopy message withBlanksTrimmed.
	
	self assert: 1 equals: workingCopy parents size.
	workingCopy := workingCopy parents first.
	self assert: 'blah' equals: workingCopy message withBlanksTrimmed.
	
	"I reset HEAD to the second to last commit. So the last commit is not reachable"
	workingCopy := repository commitNamed: 'fea21a7e0e910cf24f3c455cc6e0ea86d589a0db'.
	self assert: 'blah3' equals: workingCopy message withBlanksTrimmed.
	
	"see if all object types are accessable"
	self 
		shouldnt: [ repository repository objectNamed: '98564e36d46a682f13e716b6f5e13d94cd9461b7' ] "tree"
		raise: Error.
	self 
		shouldnt: [ repository repository objectNamed: 'e1f6defb3f5b3e6d851165c8c38c18ba42b5649b' ] "tree"
		raise: Error.
	self 
		shouldnt: [ repository repository objectNamed: 'ea511757325072abacc21fc749c7000461dbc857' ] "tree"
		raise: Error.
	self 
		shouldnt: [ repository repository objectNamed: '370101a2d57b40c37da9b42b688e61eb968f943e' ] "original blob"
		raise: Error.
	self 
		shouldnt: [ repository repository objectNamed: 'd8bf7c56584e5e2a7d51f27ea9eec36cc2695b72' ] "blob in second commit (delta)"
		raise: Error.
	self 
		shouldnt: [ repository repository objectNamed: '286064b187da14ddaab211e6579a225f179083e7' ] "blob in third commit (delta)"
		raise: Error! !

FSFilesystem subclass: #FSGitFilesystem
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitFilesystem commentStamp: 'CamilloBruni 8/1/2011 18:30' prior: 0!
I am wrapper for accessing the files of a specific revision from a git repository. I am almost never used directly but create by a FSGitRepository.

	ref := FSFilesystem onDisnk / 'home' / 'MyUser' / 'foo.git'.
	basicRepository := GitRepository on: ref.
	fsRepository := FSGitRepository  on: ref.
	workingCopy := fsRepository head. "this returns an FSGitFilesystem for head"!

!FSGitFilesystem methodsFor: 'comparing' stamp: 'CamilloBruni 7/15/2011 15:59'!
= other
	(self name isEmptyOrNil and: [ other name isEmptyOrNil ])
		ifFalse: [ ^ self name = other name ]
		ifTrue: [ ^ self == other ]! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 17:37'!
branch: branchName message: aMessage
	store branch: branchName message: aMessage! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 17:37'!
branch: branchName message: message andTag: tagName
	store branch: branchName message: message andTag: tagName! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/13/2011 20:21'!
branchName
	^ store branchName! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:26'!
branchNames
	^ repository branchNames! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:26'!
branches
	^ repository branches! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/13/2011 20:18'!
commit: aMessageString
	^ store commit: aMessageString! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:36'!
commit: aMessageString andTag: tagName
	^ store commit: aMessageString andTag: tagName! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 18:03'!
detachWorkingCopy
	store detachWorkingCopy.! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/13/2011 20:15'!
message
	^ store message! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 8/1/2011 16:17'!
name
"Answers the sha1 name of the currently active commit (the checked out version of the repository)."

	^ self store name! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 17:18'!
parents
	^ store parents collect: [ :commit | 
		repository filesystemOn: commit ]! !

!FSGitFilesystem methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 15:26'!
repository
	^ repository! !

!FSGitFilesystem methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 15:26'!
repository: anFSGitRepository
	repository := anFSGitRepository! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/13/2011 19:46'!
reset
	store reset! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/13/2011 20:21'!
revision
	^ store revision! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:27'!
tagNames
	^ repository tagNames! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:27'!
tags
	^ repository tags! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:33'!
timeStampOfCommit
	^ store timeStampOfCommit! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 7/15/2011 15:33'!
timeStampOfCreation
	^ store timeStampOfCreation! !

!FSGitFilesystem methodsFor: 'delegating' stamp: 'CamilloBruni 8/9/2011 16:57'!
validate
	^ repository validate! !

Object subclass: #FSGitModificationManager
	instanceVariableNames: 'createdPaths deletedPaths repository cache treeMap storedObjects hasChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitModificationManager commentStamp: 'CamilloBruni 8/1/2011 18:31' prior: 0!
I track the changes made to a FSGitFilesystem. On a commit all the changes are saved to "disk" and I am reset.!

!FSGitModificationManager class methodsFor: 'initialize-release' stamp: 'MaxLeske 10/18/2010 17:34'!
initializeWith: aRepository with: aCache for: aRevision
	^ self new 
		initializeWith: aRepository 
		with: aCache 
		for: aRevision! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 18:03'!
cache: aFilesystem
	cache := aFilesystem! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 18:05'!
clearRootTree
	cache workingDirectory children do: [ :reference|
		reference deleteAll].
	
	self 
		initializeWith: repository
		with: cache
		for: nil! !

!FSGitModificationManager methodsFor: 'enumerating' stamp: 'CamilloBruni 7/5/2011 13:17'!
createdPathsDo: aBlock	
	| tree |
	createdPaths associations do: [ :assoc | 
		tree := self treeAt: assoc key. 
		assoc value do: [ :path |
			aBlock 
				value: tree
				value: path ] ]! !

!FSGitModificationManager methodsFor: 'testing' stamp: 'CamilloBruni 7/15/2011 20:58'!
hasModifications
	^ hasChanges! !

!FSGitModificationManager methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 20:58'!
initializeWith: aRepository with: aCache for: aRevision
	super initialize.
	hasChanges := false.
	repository := aRepository.
	cache := aCache.
	treeMap := Dictionary new.
	createdPaths := Dictionary new.
	deletedPaths := Dictionary new.
	treeMap
		at: FSPath root
		put: (aRevision
			ifNotNil: [ self newTreeFrom: aRevision tree ]
			ifNil: [ self newTreeAt: FSPath root ])! !

!FSGitModificationManager methodsFor: 'testing' stamp: 'CamilloBruni 7/5/2011 17:05'!
isDeleted: aPath
	deletedPaths
			at: aPath parent
			ifPresent: [ :set | ^ set includes: aPath ].
	deletedPaths keysAndValuesDo: [ :parent :set|
			(parent contains: aPath)
				ifTrue: [
					(set anySatisfy: [ :path| path contains: aPath])
						ifTrue: [ ^ true ]
				].
		].
	^ false! !

!FSGitModificationManager methodsFor: 'private' stamp: 'CamilloBruni 8/9/2011 19:28'!
lookupTreeAt: aPath
	|tree|
	tree := self rootTree.
	"walk down the tree from the root directory by directory"
	aPath do: [ :segment |
		tree := tree
			entryAt: segment
			ifPresent: [ :entry | 
				"let's try to load the entry, might still be that it has been freshly added by a previous processCreation send"
				^ repository objectWithSignature: entry objectSignature ifAbsent: [ nil]]
			ifAbsent: [ ^ nil ] ].! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 3/6/2011 13:33'!
mapAll: aKeyedCollection
	aKeyedCollection associations do: [ :assoc | 
		treeMap 
			at: assoc key
			ifAbsentPut: [ assoc value ] ]! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 19:00'!
markCreated: aPath 
	^ self markCreated: aPath 
		inTree: (self treeAt: aPath parent)! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 8/1/2011 16:31'!
markCreated: aPath inTree: aTree
	| newTree |
	aPath = FSPath root ifTrue: [ ^ self ].

	treeMap
		at: aPath parent
		ifAbsentPut: [ self newTreeFrom: aTree ].
	
	(createdPaths
		at: aPath parent
		ifAbsentPut: [ FSGitSetWrapper new]) 
			add: aPath.! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 7/5/2011 16:23'!
markDeleted: aPath 
	^ self markDeleted: aPath 
		inTree: (self treeAt: aPath parent)! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 7/20/2011 15:12'!
markDeleted: aPath inTree: aTree
	| duplicateDeletions |
	aPath isRoot ifTrue: [self halt].
	hasChanges := true.
	
	"remove any children of the deleted path"
	duplicateDeletions := OrderedCollection new.
	deletedPaths keysAndValuesDo: [ :key :paths|
		(paths includesSubPath: aPath)
			ifTrue: [ duplicateDeletions add: key]].
	duplicateDeletions do: [:aKey | deletedPaths removeKey: aKey].
		
	"collect a list of deleted paths"
	(deletedPaths
		at: aPath parent
		ifAbsentPut: [ FSGitSetWrapper new]) 
			add: aPath.! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 8/1/2011 16:30'!
markDirectoryCreated: aPath 
	^ self markCreated: aPath! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 8/1/2011 16:30'!
markFileCreated: aPath 
	"only files trigger changes"
	hasChanges := true.
	^ self markCreated: aPath! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 11/12/2010 21:56'!
nameOf: aPath
	| tree |
	tree := treeMap at: aPath parent.
	^ (tree entryAt: aPath basename) objectSignature hexHash! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'MaxLeske 10/17/2010 19:45'!
newTreeAt: aPath
	^ GitTree new! !

!FSGitModificationManager methodsFor: 'private' stamp: 'MaxLeske 10/18/2010 20:25'!
newTreeFrom: aTree
	| tree |
	tree := GitTree new.
	tree entries: aTree entries.
	^ tree! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/9/2011 19:14'!
processBlobAt: path in: tree
	| reference data blob entry |
	reference := cache referenceTo: path.
	
	reference isFile ifFalse: [ ^ self].
	
	data := reference readStreamDo: [ :stream | stream contents ].
	blob := GitBlob bytes: data.
	repository storeObject: blob .
	
	entry := GitTreeEntry
		entryName: path basename
		mode: GitTreeEntry fileMode
		objectSignature: blob signature.
	tree putEntry: entry! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/9/2011 19:14'!
processBlobs
	"only save new files here and put new git tree entries into their parent"
	
	| reference data blob entry |
	self createdPathsDo: [ :tree :path |
		self processBlobAt: path in: tree]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 7/15/2011 19:34'!
processChanges
	self processBlobs.
	self processCreations.
	self processDeletions.
	self storeTrees.! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/9/2011 20:16'!
processCreationOf: entries from: entryPath
	|tree childTree|
	entries
		select: [ :path | (cache referenceTo: path) isDirectory ]
		thenDo: [ :directory |
			childTree := (self treeAt: directory).
			"don't process empty trees for now"
			childTree isEmpty ifFalse: [
				tree := (self treeAt: entryPath) 
					putEntry: (GitTreeEntry
						entryName: directory basename
						mode: GitTreeEntry dirMode
						objectSignature: childTree signature).
				self flag: 'TODO: update refs only once after processing all deletes and creates'.
				self updateParentSignaturesOfChanged: tree at: entryPath.]]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/9/2011 19:10'!
processCreations
	"create the deepest trees first to avoid regenration of signatures of the parent trees"
	| paths|
	paths := (createdPaths associations sorted: [ :assoc1 :assoc2 |
		assoc2 value maxDepth <= assoc1 value maxDepth ]).
	paths do: [ :assoc |
		self processCreationOf: assoc value from: assoc key]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/1/2011 16:59'!
processDeletionOf: entries from: entryPath
	"remove all the deleted files/paths from its parent (the current tree)"
	|  target tree|
	tree := self treeAt: entryPath.
	entries do: [ :path |
		tree removeEntryAt: path basename ifAbsent: []].
	entryPath isRoot ifTrue: [ ^ self].
	"update the entry in the repository with the new tree"
	self flag: 'TODO: update refs only once after processing all deletes and creates'.
	self updateParentSignaturesOfChanged: tree at: entryPath.! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/1/2011 16:59'!
processDeletions
	| entryPath target |
	deletedPaths copy keysAndValuesDo: [ :path :deletedEntries |
		self processDeletionOf: deletedEntries from: path]! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/17/2010 15:01'!
repository: aRepository
	repository := aRepository ! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'MaxLeske 10/18/2010 20:21'!
rootTree
	^ treeMap at: FSPath root! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 7/5/2011 13:20'!
storeCreatedTrees
	createdPaths keysDo: [ :path | self storeTreeAt: path].
	"(createdPaths associations sorted: [ :assoc1 :assoc2 |
		assoc2 value maxSize <= assoc1 value maxSize ]) do: [ :assoc |
			self storeTreeAt: assoc key]."! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 7/5/2011 13:19'!
storeDeletedTrees
	"store the changed trees"
	deletedPaths keysDo: [ :path | self storeTreeAt: path].! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/9/2011 20:19'!
storeTree: aTree 
	(storedObjects includes: aTree) ifTrue: [ ^ self ].
	storedObjects add: aTree.
	"don't store empty trees"
	aTree isEmpty ifFalse: [ repository storeObject: aTree ].! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/1/2011 17:10'!
storeTreeAt: aPath
	aPath isRoot ifTrue: [ ^ self ].
	"bubble up the tree hierarchy and save all of them"
	self storeTreeAt: aPath parent.
	self storeTree: (self treeAt: aPath).! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 8/1/2011 18:21'!
storeTrees
	storedObjects := Set new.
	self storeCreatedTrees.
	self storeDeletedTrees.
	repository storeObject: self rootTree! !

!FSGitModificationManager methodsFor: 'private' stamp: 'CamilloBruni 8/9/2011 19:28'!
treeAt: aPath
	| tree |
	treeMap
		at: aPath
		ifPresent: [ :aTree| ^ aTree].
	tree := (self lookupTreeAt: aPath)	
				ifNil: [ self newTreeAt: aPath ].
	treeMap at: aPath put: tree.
	^ tree ! !

!FSGitModificationManager methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 21:10'!
updateCacheAt: aPath with: aGitTree.
	treeMap 
		at: aPath
		ifAbsentPut: [ aGitTree]! !

!FSGitModificationManager methodsFor: 'operations' stamp: 'CamilloBruni 7/20/2011 15:30'!
updateParentSignaturesOfChanged: target at: aPath 
	"recursively update the refs of the target and subsequently remove empty directories"
	|parent|
	self flag: 'TODO: update refs only once after processing all deletes and creates'.
	aPath isRoot ifTrue: [ ^ self ].
	parent := self treeAt: aPath parent.
	target isEmpty 
		ifTrue: [
			parent removeEntryAt: aPath basename.]
		ifFalse: [
			parent putEntry: 
				(GitTreeEntry
					entryName: aPath basename
					mode: GitTreeEntry dirMode
					objectSignature: (GitObjectSignature for: target))].
	self updateParentSignaturesOfChanged: parent at: aPath parent. ! !

Object subclass: #FSGitRepository
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitRepository commentStamp: '<historical>' prior: 0!
I represent a git repository. Every message sent to me will return an FSGitFilesystem object or a collection of such objects.!

!FSGitRepository class methodsFor: 'instance creation' stamp: 'MaxLeske 7/9/2010 15:10'!
on: anFSReference
	^ self new initializeOn: anFSReference! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 17:25'!
branchNamed: aString
	^ self filesystemOn: (repository branchNamed: aString)! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 17:23'!
branchNames
	^ repository branchNames! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 15:18'!
branches
	^ repository branches collect: [ :commit | 
		self filesystemOn: commit ]! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 09:29'!
commitNamed: hexHash
	^ self filesystemOn: (repository objectNamed: hexHash)! !

!FSGitRepository methodsFor: 'private' stamp: 'CamilloBruni 7/15/2011 15:27'!
filesystemOn: aGitCommit
	^ (FSGitFilesystem store: (self gitStoreOn: aGitCommit))
			repository: self;
			yourself! !

!FSGitRepository methodsFor: 'private' stamp: 'CamilloBruni 7/15/2011 18:41'!
gitStoreOn: aGitCommit
	^ FSGitStore inMemory
		repository: repository;
		revision: aGitCommit;
		yourself! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 17:25'!
head
	^ self filesystemOn: repository head! !

!FSGitRepository methodsFor: 'initialize-release' stamp: 'MaxLeske 7/23/2010 09:59'!
initializeOn: anFSReference
	repository := GitRepository on: anFSReference! !

!FSGitRepository methodsFor: 'private' stamp: 'MaxLeske 8/13/2011 17:02'!
recursivelyCollectCommits: aCommit into: aCollection
	aCommit isLoaded ifFalse: [ aCommit loadLazily ].
	aCollection add: aCommit.
	aCommit parents do: [ :parent | 
		self 
			recursivelyCollectCommits: parent
			into: aCollection ]! !

!FSGitRepository methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/22/2011 05:08'!
repository
	^ repository! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 17:25'!
tagNamed: aString
	^ self filesystemOn: (repository tagNamed: aString)! !

!FSGitRepository methodsFor: 'accessing' stamp: 'MaxLeske 11/8/2010 17:24'!
tagNames
	^ repository tagNames! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 17:26'!
tags
	^ repository tags collect: [ :tag | 
		self filesystemOn: tag object ]! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 16:58'!
validate
	^ 	repository validate! !

!FSGitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 19:56'!
versions
	| commits |
	commits := OrderedCollection new.
	self 
		recursivelyCollectCommits: repository head
		into: commits.
	
	^ commits collect: [ :each |
		self filesystemOn: each ] ! !

Object subclass: #FSGitSetWrapper
	instanceVariableNames: 'set maxDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git'!

!FSGitSetWrapper commentStamp: 'CamilloBruni 8/1/2011 18:32' prior: 0!
I am used to track changes made to a certain directory in a git filesystem. Each change is recorded by the FSGitModificationManager which stores the modified paths for each parent directory in a FSGitSetWrapper.!

!FSGitSetWrapper methodsFor: 'testing' stamp: 'MaxLeske 10/18/2010 12:14'!
= aSet
	^ set = aSet! !

!FSGitSetWrapper methodsFor: 'adding' stamp: 'CamilloBruni 7/15/2011 19:43'!
add: newObject
	maxDepth := maxDepth max: newObject size.
	set add: newObject! !

!FSGitSetWrapper methodsFor: 'testing' stamp: 'CamilloBruni 7/5/2011 16:34'!
allSatisfy: aBlock
	^ set allSatisfy: aBlock! !

!FSGitSetWrapper methodsFor: 'accessing' stamp: 'MaxLeske 10/19/2010 09:38'!
anyOne
	^ set anyOne! !

!FSGitSetWrapper methodsFor: 'testing' stamp: 'CamilloBruni 7/5/2011 17:07'!
anySatisfy: aBlock
	^ set anySatisfy: aBlock! !

!FSGitSetWrapper methodsFor: 'enumerating' stamp: 'MaxLeske 10/18/2010 12:13'!
collect: aBlock
	^ set collect: aBlock! !

!FSGitSetWrapper methodsFor: 'enumerating' stamp: 'MaxLeske 10/18/2010 12:13'!
do: aBlock
	set do: aBlock! !

!FSGitSetWrapper methodsFor: 'testing' stamp: 'MaxLeske 10/18/2010 12:15'!
includes: anObject
	^ set includes: anObject! !

!FSGitSetWrapper methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2011 08:00'!
includesSubPath: aFSPath 
	^ set anySatisfy: [ :el| aFSPath contains: el]! !

!FSGitSetWrapper methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/15/2011 19:43'!
initialize
	super initialize.
	set := Set new.
	maxDepth := 0! !

!FSGitSetWrapper methodsFor: 'accessing' stamp: 'CamilloBruni 7/15/2011 19:44'!
maxDepth
	^ maxDepth! !

!FSGitSetWrapper methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/1/2011 17:05'!
printOn: aStream
	aStream << self class name.
	aStream << '('.
	set do: [ :path| aStream << path basename]
		separatedBy: [ aStream << ', '].
	aStream << ')'.! !

!FSGitSetWrapper methodsFor: 'enumerating' stamp: 'MaxLeske 10/18/2010 12:13'!
select: aBlock
	^ set select: aBlock! !

!FSGitSetWrapper methodsFor: 'enumerating' stamp: 'MaxLeske 10/19/2010 09:07'!
select: selectBlock thenDo: doBlock
	set
		select: selectBlock
		thenDo: doBlock! !

!FSGitSetWrapper methodsFor: 'enumerating' stamp: 'MaxLeske 10/18/2010 22:31'!
sorted: aBlock
	^ set sorted: aBlock! !

Object subclass: #GitDeltaInterpreter
	instanceVariableNames: 'source instructions output commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions
	^ instructions! !

!GitDeltaInterpreter methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 12:01'!
instructions: anObject
	instructions := anObject! !

!GitDeltaInterpreter methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:13'!
invalidBytes: aString
	GitInvalidBytesError signal: aString! !

!GitDeltaInterpreter methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:13'!
invalidPackfile: aString
	GitInvalidPackfileError signal: aString! !

!GitDeltaInterpreter methodsFor: 'private' stamp: 'MaxLeske 8/1/2011 19:17'!
readBaseObjectSize
	^ self readVariableInteger! !

!GitDeltaInterpreter methodsFor: 'private' stamp: 'MaxLeske 8/1/2011 19:17'!
readFinalObjectSize
	^ self readVariableInteger! !

!GitDeltaInterpreter methodsFor: 'private' stamp: 'MaxLeske 8/12/2011 08:31'!
readVariableInteger
"modified little endian encoding.
Most significant bit of each byte indicates if that byte is the last
byte encoding for the value (bit = 0) or if another byte follows (bit = 1)"
	| value shift byte |
	value := 0.
	shift := 0.
	[ byte := commands next.
	value := value bitOr: ((byte bitAnd: 127) << shift).
	shift := shift + 7 ] doWhileFalse: [ (byte bitAnd: 128) isZero ].
	
	^ value! !

!GitDeltaInterpreter methodsFor: 'executing' stamp: 'tonyg 2/23/2010 12:20'!
run
	[commands atEnd] whileFalse: [self runNextCommand]! !

!GitDeltaInterpreter methodsFor: 'executing' stamp: 'MaxLeske 8/11/2011 21:55'!
runAgainst: aByteArray
	source := aByteArray.
	commands := ReadStream on: instructions.
	source size = self readBaseObjectSize ifFalse: [ 
		self invalidPackfile: 'Actual size of delta base object is not the same as announced by packfile.' ].
	output := WriteStream on: (ByteArray new: self readFinalObjectSize).
	self run.

	^ output contents! !

!GitDeltaInterpreter methodsFor: 'executing' stamp: 'MaxLeske 8/12/2011 08:34'!
runNextCommand
	| cmd |
	cmd := commands next.
	cmd = 0 ifTrue: [ self invalidBytes: 'opcode 0 is a reserved byte. This delta is invalid.' ].
	cmd < 128
		ifTrue: [output nextPutAll: (commands next: cmd)]
		ifFalse: [ | offset size |
			offset := 0.
			(cmd bitAnd: 2r00000001) isZero ifFalse: [ offset := commands next].
			(cmd bitAnd: 2r00000010) isZero ifFalse: [ offset := offset bitOr: commands next << 8 ].
			(cmd bitAnd: 2r00000100) isZero ifFalse: [ offset := offset bitOr: commands next << 16 ].
			(cmd bitAnd: 2r00001000) isZero ifFalse: [ offset := offset bitOr: commands next << 24 ].

			size := 0.
			(cmd bitAnd: 2r00010000) isZero ifFalse: [size := commands next].
			(cmd bitAnd: 2r00100000) isZero ifFalse: [size := size bitOr: commands next << 8 ].
			(cmd bitAnd: 2r01000000) isZero ifFalse: [size := size bitOr: commands next << 16 ].
			size = 0 ifTrue: [ size := 16r10000 ].
			
			output nextPutAll: (source copyFrom: offset + 1 to: offset + size) ]! !

Object subclass: #GitFSCK
	instanceVariableNames: 'all repository reachable unreachable errors referenceContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitFSCK class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 6/21/2011 06:05'!
validate: aGitRepository
	^ self new 
		repository: aGitRepository; 
		validate! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:34'!
accept: anObject
	(reachable includes: anObject) ifTrue: [ ^ self].
	self mark: anObject.
	^ anObject accept: self.! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/22/2011 03:56'!
acceptCollection: aCollection 
	aCollection do: [ :each|
		self accept: each ].! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:55'!
collectError: error for: anObject
	errors at: anObject ifAbsentPut: OrderedCollection new.
	(errors at: anObject) add: (
		(error respondsTo: #copyUpTo:)
			ifTrue: [ error copyUpTo: referenceContext ]	
			ifFalse: [ error ])! !

!GitFSCK methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 6/22/2011 05:01'!
collectObjects
	all := repository looseObjects.
	self flag: 'TODO: collect packed objects'.
	all := all reject: [ :each| each species == GitTag]! !

!GitFSCK methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 04:19'!
collectUnreachableErrors	
	unreachable do: [ :each|
		self collectError: 'unreachable object' for: each].! !

!GitFSCK methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:13'!
invalidRepository: aString
	GitInvalidRepositoryError signal: aString! !

!GitFSCK methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/21/2011 15:05'!
mark: anObject
	unreachable remove: anObject ifAbsent: [].
	reachable add: anObject.! !

!GitFSCK methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 03:45'!
repository
	^ repository! !

!GitFSCK methodsFor: 'accessing' stamp: 'CamilloBruni 6/21/2011 06:06'!
repository: aGitRepository
	repository := aGitRepository! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 21:57'!
validate
	self collectObjects.
	referenceContext := thisContext.
	errors := Dictionary new.
	unreachable := all copy.
	reachable := Set new.
	self validateHead.
	self validateBranches.
	self validateTags.
	
	self validateUnreachableObjects.
	
	errors isEmpty ifFalse: [ 
		self invalidRepository: 'GitFSCK found errors in the validated repository.' ].! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:57'!
validate: anObject
	[ anObject validateIn: repository ]
		on: GitError
		do: [ :error | self collectError: error for: anObject ]! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:57'!
validateBranches
	[ repository branches do: [ :head| self accept: head ] ]
		on: GitError
		do: [ :error | 
			self halt.
			self collectError: error for: repository ]! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:57'!
validateHead
	self validateHeadFile.
	[ repository head ifNotNilDo: [ :head | self accept: head ] ]
		on: GitError
		do: [ :error | 
			self collectError: error for: repository ]! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:58'!
validateHeadFile
	[ repository headFile exists  ifFalse: [ self invalidRepository: 'Could not find ''HEAD''' ] ]
		on: GitError
		do: [ :error |
			self collectError: error for: repository ]! !

!GitFSCK methodsFor: 'validation' stamp: 'CamilloBruni 6/21/2011 06:27'!
validateObjects
	self halt.
	self flag: 'Validate all object files'! !

!GitFSCK methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:58'!
validateTags
	[ repository tags do: [ :tag | self accept: tag ] ]
		on: GitError
		do: [ :error | 
			self collectError: error for: repository ]! !

!GitFSCK methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 04:28'!
validateUnreachableObjects
	| unreachableCopy reachableCopy |
	unreachable ifEmpty: [ ^ self ].
	
	self collectUnreachableErrors.
	unreachableCopy := unreachable copy.
	reachableCopy := reachable copy.
	self acceptCollection: unreachable.
	reachable := reachableCopy.
	unreachable := unreachableCopy.! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/22/2011 04:15'!
visitBlob: aBlob
	self validate: aBlob.! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/22/2011 04:15'!
visitCommit: aCommit
	self validate: aCommit.
	self accept: aCommit tree.
	self acceptCollection: aCommit parents.! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/22/2011 04:59'!
visitTag: aTag
	self validate: aTag.
	self accept: aTag object! !

!GitFSCK methodsFor: 'visiting' stamp: 'CamilloBruni 6/22/2011 04:14'!
visitTree: aTree
	self validate: aTree.
	self acceptCollection: aTree objects.! !

Object subclass: #GitObject
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObject commentStamp: '<historical>' prior: 0!
I am an abstract object that defines messages to which every object (in git speak) responds.

Instance Variables:
	repository <GitRepository>
		A private variable that a GitObject uses to lazily load itself!

!GitObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/30/2010 14:12'!
in: aGitRepository
	^self new initializeOn: aGitRepository; yourself! !

!GitObject methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:13'!
accept: aVisitor
	self subclassResponsibility! !

!GitObject methodsFor: 'initialize-release' stamp: 'MaxLeske 7/24/2010 17:21'!
initializeOn: aGitRepository
	repository := aGitRepository! !

!GitObject methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:22'!
invalid: aString
	GitInvalidObjectError signal: aString! !

!GitObject methodsFor: 'testing' stamp: 'CamilloBruni 7/15/2011 19:22'!
isTree
	^ false! !

!GitObject methodsFor: 'initialize-release' stamp: 'MaxLeske 7/23/2010 14:52'!
loadFrom: aStream
"This method implements the loading of a GitObject from a stored byte sequence."

	self subclassResponsibility! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:24'!
repository
	^repository! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:24'!
repository: aGitRepository
	repository := aGitRepository! !

!GitObject methodsFor: 'saving' stamp: 'MaxLeske 7/23/2010 14:50'!
saveTo: aStream
"This method defines the byte sequence for the signature calculation (GitObjectSignature) and also the content of a particular object that is stored in the repository."

	self subclassResponsibility! !

!GitObject methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 14:47'!
size
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream size! !

!GitObject methodsFor: 'validation' stamp: 'CamilloBruni 6/21/2011 06:16'!
validate! !

GitObject subclass: #GitObjectSignature
	instanceVariableNames: 'signature integerHash'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObjectSignature commentStamp: '<historical>' prior: 0!
I know how to calculate a signature for each GitStorableObject and I handle the hash representation of that signature. My equivalent in git speak is the sha1 name which is a 40 character hash (SHA1 encryption of header and contents of each object; see http://git.rsbx.net/Documents/Git_Data_Formats.txt and http://book.git-scm.com/1_the_git_object_model.html) stored as a string.

Instance Variables:
	signature	<ByteArray>
		The byte sequence representing the signature for an object that is used to calculate it sha1 hash.
	integerHash	<Integer>
		The hash calculated from the signature byte sequence in integer representation.!

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 4/29/2010 18:05'!
for: aGitObject
	^self new computeSignatureFor: aGitObject! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/9/2010 13:13'!
from: aHexHash
	^self new from: aHexHash! !

!GitObjectSignature class methodsFor: 'initialize-release' stamp: 'MaxLeske 5/25/2011 18:09'!
loadFrom: aStream
	| hash |
	hash := (aStream next: 20).
	^self from: hash hex! !

!GitObjectSignature methodsFor: 'comparing' stamp: 'MaxLeske 5/9/2010 15:15'!
= other
	^self integerHash = other integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'CamilloBruni 5/19/2011 21:19'!
byteSignatureFor: aGitObject
	| stream data |
	stream := GitStream new.
	data := aGitObject byteData.
	"format for object hash (see manual -> 'Object storage format'):
	<ascii type without space> + <space> + <ascii decimal size> + <byte\0> + <binary object data>"
	stream
		nextPutAll: aGitObject typeCode;
		space;
		nextPutAll: data size asString; "ASCII value for $0 is not 0 but 48!!"
		nullByte;
		nextPutAll: data.
	^stream contents! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:53'!
bytes
	"Answers the byte sequence that was used to calculate the sha1 name of the associated object."
	
	^signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/8/2010 20:41'!
computeSignatureFor: aGitObject
	signature := self byteSignatureFor: aGitObject.
	integerHash := self integerHashFrom: signature! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 13:32'!
from: aHexHash
	integerHash := Integer readFrom: aHexHash asString asUppercase base: 16! !

!GitObjectSignature methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:14'!
hash
	^ self integerHash! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:53'!
hexHash
	"Answers the sha1 name of the associated object."
	
	^self hexHashFrom: integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 5/9/2010 14:38'!
hexHashFrom: aByteString
	"#hex will omit a leading 0. If that's the case insert leading 0."
	^aByteString asByteArray hex asLowercase forceTo: 40 paddingStartWith: $0! !

!GitObjectSignature methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 14:54'!
integerHash
"Answers the sha1 hash of the associated object in integer representation."

	^integerHash! !

!GitObjectSignature methodsFor: 'private' stamp: 'MaxLeske 7/30/2011 16:29'!
integerHashFrom: byteArray
	^ SecureHashAlgorithm new hashMessage: byteArray! !

!GitObjectSignature methodsFor: 'printing' stamp: 'MaxLeske 5/8/2010 20:42'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self hexHash; nextPut: $).! !

!GitObjectSignature methodsFor: 'saving' stamp: 'MaxLeske 7/31/2010 17:51'!
saveTo: aStream
	aStream nextPutBytes: (self integerHash asByteArray forceTo: 20 paddingStartWith: 0)! !

GitObject subclass: #GitStorableObject
	instanceVariableNames: 'signature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStorableObject commentStamp: '<historical>' prior: 0!
I am an abstract class defining common messages to which all object (in git speak) respond that can actually be stored, i.e. commits, trees, tags, blobs.

Instance Variables:
	signature	<GitObjectSignature>
		Lazily loaded signature of the object. See class comment for GitObjectSignature.!

GitStorableObject subclass: #GitBlob
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitBlob commentStamp: '<historical>' prior: 0!
I am the git object responsible for storing content. I do not know or care what data I hold.

Instance Variables:
	bytes	<ByteArray>
		The contents I hold, stored in a ByteArray.!

!GitBlob class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 16:02'!
bytes: aByteArray
	^ self new initializeBytes: aByteArray asByteArray! !

!GitBlob class methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:19'!
typeCode
	^ 'blob'! !

!GitBlob methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:13'!
accept: aVisitor
	^ aVisitor visitBlob: self.! !

!GitBlob methodsFor: 'saving' stamp: 'MaxLeske 7/23/2010 14:56'!
bytes
"Answers the contents of a blob object."

	^ bytes! !

!GitBlob methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 13:31'!
initializeBytes: aByteArray
	bytes := aByteArray! !

!GitBlob methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:15'!
loadFrom: aStream
	bytes := aStream contents.
	^self! !

!GitBlob methodsFor: 'saving' stamp: 'tonyg 2/23/2010 15:36'!
saveTo: aStream
	aStream nextPutAll: bytes.! !

!GitBlob methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 04:17'!
validateIn: aGitRepository
	self validate! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 8/11/2011 23:02'!
forType: aString  
 ^ self allSubclasses
       detect: [ :class | aString isNil not and: [ class typeCode = aString ] ]
       ifNone: [ GitInvalidObjectError signal: 
		'Could not load object. Invalid type code: ', aString printString ]! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:45'!
loadFrom: aStream from: aGitRepository
	| code length limitedStream object |
	code := (aStream upTo: Character space asInteger) asString.
	length := (aStream upTo: 0) asString asInteger.
	limitedStream := ReadStream on: (aStream next: length).
	object := (self forType: code) in: aGitRepository.
	^object loadFrom: limitedStream! !

!GitStorableObject class methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 12:24'!
typeCode
	"Override in subclasses, returning non-nil where you want them to be constructible from a git object store."
	^ nil! !

!GitStorableObject class methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 17:07'!
withSignature: aGitSignature
	^self new withSignature: aGitSignature! !

!GitStorableObject methodsFor: 'comparing' stamp: 'CamilloBruni 8/3/2011 13:17'!
= other
	(other isKindOf: self class)
		ifFalse: [ ^ false ].
		
	^self signature = other signature! !

!GitStorableObject methodsFor: 'private' stamp: 'Max Leske 4/25/2010 13:05'!
byteData
	| stream |
	stream := GitStream new.
	self saveTo: stream.
	^stream contents! !

!GitStorableObject methodsFor: 'comparing' stamp: 'CamilloBruni 6/22/2011 03:50'!
hash
	^ self signature hash bitXor: self class name hash! !

!GitStorableObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:37'!
signature
	^signature ifNil: [ signature := GitObjectSignature for: self ]! !

!GitStorableObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 08:40'!
signature: aSignature
	signature := aSignature! !

!GitStorableObject methodsFor: 'saving' stamp: 'MaxLeske 6/30/2010 14:28'!
store
	self subclassResponsibility! !

!GitStorableObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 15:47'!
typeCode
	^ self class typeCode! !

!GitStorableObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 17:07'!
withSignature: aGitObjectSignature
	signature := aGitObjectSignature! !

GitStorableObject subclass: #GitTaglikeObject
	instanceVariableNames: 'message properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTaglikeObject commentStamp: '<historical>' prior: 0!
I am an abstract class defining common protocol for git objects that behave like tags.

Instance Variables:
	message	<String>
		Holds the message of the tag like object, if used.
	properties	<(Collection of: (Association of: String -> ByteArray))>
		Holds arbitrary key value pairs not specified by git that can be used by applications to store additional data.!

GitTaglikeObject subclass: #GitCommit
	instanceVariableNames: 'tree parents author committer isLoaded'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitCommit commentStamp: '<historical>' prior: 0!
I represent the state of a git repository in time. It is mandatory to provide me with a message describing the changes.

Instance Variables:
	tree	<GitTree>
		Holds a tree representing the root of the repository.
	parents	<Collection of: GitCommit>
		Holds a collection of parent commits. A parent commit represents 
		the state the repository was in when the changes occurred that led to the current commit.
		In case of a merge it is possible for a commit to have more than one parent.
	author	<GitStamp>
		Denotes the author of the content committed.
	committer	<GitStamp>
		Denotes the committer of the commit. The committer might be a different
		person than the author.!

!GitCommit class methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 22:21'!
typeCode
	^ 'commit'! !

!GitCommit class methodsFor: 'instance creation' stamp: 'MaxLeske 7/24/2010 17:18'!
withDefaultStamps
	| s |
	s := GitStamp now.
	^ self new 
		author: s;
		committer: s;
		yourself! !

!GitCommit class methodsFor: 'instance creation' stamp: 'MaxLeske 5/25/2010 19:59'!
withTree: aGitTree message: aString author: aGitStampForAuthor committer: aGitStampForCommitter
	^self new
		tree: aGitTree;
		message: aString;
		author: aGitStampForAuthor;
		committer: aGitStampForCommitter;
		yourself
		! !

!GitCommit methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:14'!
accept: aVisitor
	^ aVisitor visitCommit: self.! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 04:34'!
author
	^ author ifNil: [ 
		self loadLazily. 
		author ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:52'!
author: aGitStamp
	author := aGitStamp! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'MaxLeske 7/26/2010 08:46'!
beLoaded: aBoolean
	isLoaded := aBoolean! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 04:34'!
committer
	^ committer ifNil: [ 
		self loadLazily. 
		committer ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:51'!
committer: aGitStamp
	committer := aGitStamp! !

!GitCommit methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/11/2011 15:27'!
initialize
	super initialize.
	isLoaded := false.
	parents  := OrderedCollection new.
	message := ''.
	properties := OrderedCollection new.
	parents := OrderedCollection new.! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'MaxLeske 7/26/2010 08:46'!
isLoaded
	^ isLoaded! !

!GitCommit methodsFor: 'loading' stamp: 'CamilloBruni 7/11/2011 15:26'!
loadKey: k value: v
	k = 'tree' ifTrue: [tree := repository objectNamed: v. ^ self].
	k = 'parent' ifTrue: [
		parents add: ((GitCommit in: repository) signature: (GitObjectSignature from: v)).
		^ self].
	k = 'author' ifTrue: [author := GitStamp fromString: v. ^ self].
	k = 'committer' ifTrue: [committer := GitStamp fromString: v. ^ self].
	"Otherwise it's treated generically."
	^ super loadKey: k value: v! !

!GitCommit methodsFor: 'lazy-loading' stamp: 'CamilloBruni 7/11/2011 15:26'!
loadLazily
	| stored |
	self isLoaded ifTrue: [ ^ self ].
	self beLoaded: true.
	repository ifNil: [ 
		^ self ].
	
	stored := repository objectWithSignature: self signature.
	stored beLoaded: true.
	
	self 
		message: (stored message ifNil: [ '' ]);
		properties: (stored properties ifNil: [ OrderedCollection new ]);
		tree: stored tree;
		author: stored author;
		committer: stored committer;
		parents: (stored parents ifNil: [ OrderedCollection new ])! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 04:34'!
message
	^ message ifNil: [ 
		self loadLazily. 
		message ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 7/29/2010 10:32'!
parents
	^ parents ifNil: [ 
		self isLoaded 
			ifTrue: [ parents := OrderedCollection new ]
			ifFalse: [ self loadLazily. parents ] ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 20:04'!
parents: aGitCommitCollection
	^ parents := aGitCommitCollection! !

!GitCommit methodsFor: 'printing' stamp: 'CamilloBruni 6/22/2011 04:33'!
printOn: aStream
	super printOn: aStream.
	self message
		ifNotNilDo: [ :msg|
			aStream
				nextPutAll: '(''';
				nextPutAll: msg;
				nextPutAll: ''')' ]! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 04:35'!
properties
	^ properties ifNil: [ 
		self loadLazily. 
		properties ]! !

!GitCommit methodsFor: 'saving' stamp: 'MaxLeske 7/22/2010 19:40'!
saveFixedKeysTo: aStream
	"Looking at builtin-commit-tree.c, the order of properties is fixed:
	tree, parents, author, committer, everything else, message."
	self saveKey: GitTree typeCode value: tree signature hexHash to: aStream.
	self parents do: [:each | self saveKey: 'parent' value: each signature hexHash to: aStream].
	self saveKey: 'author' value: author printString to: aStream.
	self saveKey: 'committer' value: committer printString to: aStream.! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 20:38'!
tags
	^ self tagsFor: repository! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 20:39'!
tagsFor: aGitRepos
	^ aGitRepos tags select: [ :tag| tag object = self]! !

!GitCommit methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 04:35'!
tree
	^ tree ifNil: [ 
		self loadLazily. 
		tree ]! !

!GitCommit methodsFor: 'accessing' stamp: 'MaxLeske 5/26/2010 19:49'!
tree: aGitTree
	tree := aGitTree! !

!GitCommit methodsFor: 'validating' stamp: 'MaxLeske 8/11/2011 21:42'!
validate
	tree ifNil: [ self invalid: 'Cannot have a commit without a tree.' ].
	tree species = GitTree ifFalse: [ self invalid: 'The tree of this commit object is not of class GitTree.' ].
	author ifNil: [ self invalid: 'Author identity is required.' ].
	committer ifNil: [ self invalid: 'Committer identity is required.' ].
	super validate.! !

!GitCommit methodsFor: 'validating' stamp: 'CamilloBruni 6/22/2011 04:36'!
validateIn: aGitRepository
	self loadLazily.
	self validate.
	aGitRepository objectWithSignature: self tree signature! !

GitTaglikeObject subclass: #GitTag
	instanceVariableNames: 'tagger object name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTag commentStamp: '<historical>' prior: 0!
I have the responsibility of associating a human readable tag (a string) with a commit (on the filesystem level I do this by simply referencing the sha1 name of a commit), e.g. 'version 0.5' -> '2341f8c0615bbcc465ac4686025e880786430697'.

It is important to know that there are light tags and real tag objects in git. Light tags are represented through a file with the tag name that holds a reference to a commit. Alternatively that file can hold a reference to a tag object which in turn references a commit. Full tags have the advantage of knowing the tagger (and the date of tagging) and can hold a message (e.g. a PGP signature).
I incorporate both possibilities.

Instance Variables:
	tagger	<GitStamp>
		Who created the tag.
	object	<GitCommit>
		The commit referenced by the tag name.
	name	<String>
		The human readable tag (or tag name) that describes the referenced commit.!

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 8/12/2011 09:04'!
makeFullTag: tagName for: aGitCommit message: message in: aGitRepository
	^ self
		makeFullTag: tagName
		for: aGitCommit
		message: message
		tagger: GitStamp now
		in: aGitRepository! !

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 8/12/2011 09:04'!
makeFullTag: tagName for: aGitCommit message: message tagger: aGitStamp in: aGitRepository
	| tag |
	tag := self new
		name: tagName;
		object: aGitCommit;
		message: message;
		tagger: aGitStamp;
		repository: aGitRepository;
		yourself.
	tag store.
	^tag! !

!GitTag class methodsFor: 'instance creation' stamp: 'MaxLeske 8/12/2011 09:05'!
makeLightWeightTag: aString for: aGitCommit in: aGitRepository
	| tag |
	self assert: aGitCommit species = GitCommit.
	tag := self new
		name: aString;
		object: aGitCommit;
		repository: aGitRepository;
		yourself.
	tag store.
	^tag
		! !

!GitTag class methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:16'!
typeCode
	^ 'tag'! !

!GitTag methodsFor: 'comparing' stamp: 'MaxLeske 8/12/2011 08:26'!
= other
	(other isKindOf: self class)
		ifFalse: [ ^ false ].
	
	self isLightWeightTag ~= other isLightWeightTag
		ifTrue: [ ^ false ].
	
	self isLightWeightTag
		ifTrue: [ ^ self name = other name].
		
	^self signature = other signature! !

!GitTag methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:15'!
accept: aVisitor
	^ aVisitor visitTag: self.! !

!GitTag methodsFor: 'accessing' stamp: 'CamilloBruni 7/21/2011 15:14'!
hash
	^ self name hash bitXor: self class name hash! !

!GitTag methodsFor: 'testing' stamp: 'MaxLeske 8/12/2011 08:26'!
isLightWeightTag
	^self message isNil and: [ self tagger isNil ]! !

!GitTag methodsFor: 'loading' stamp: 'MaxLeske 8/12/2011 08:28'!
loadKey: key value: value
	key = 'object' ifTrue: [ object := repository objectNamed: value. ^ self ].
	key = 'type' ifTrue: [ self assert: object typeCode = value. ^ self ].
	key = 'tag' ifTrue: [ name := value. ^ self ].
	key = 'tagger' ifTrue: [ tagger := GitStamp fromString: value. ^ self ].
	"Otherwise it's treated generically."
	^ super loadKey: key value: value! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 15:11'!
name
	^ name! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 13:59'!
name: aString
	name := aString! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 08:29'!
object
	^ object! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:01'!
object: aGitCommit
	self assert: aGitCommit species = GitCommit.
	object := aGitCommit! !

!GitTag methodsFor: 'printing' stamp: 'MaxLeske 8/12/2011 08:43'!
printOn: aStream
	super printOn: aStream.
	aStream
		<< '("';
		<< self name;
		<< '")'! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 6/30/2010 15:13'!
saveFixedKeysTo: aStream
	"From builtin-mktag.c:
	A signature file has a very simple fixed format: four lines
	of ""object <sha1>"" + ""type <typename>"" + ""tag <tagname>"" +
	""tagger <committer>"", followed by a blank line, a free-form tag
	message and a signature block that git itself doesn't care about,
	but that can be verified with gpg or similar."
	self saveKey: 'object' value: self object signature hexHash to: aStream.
	self saveKey: 'type' value: self object typeCode to: aStream.
	self saveKey: 'tag' value: self name to: aStream.
	self saveKey: 'tagger' value: self tagger printString to: aStream.! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 8/12/2011 09:03'!
store
	self isLightWeightTag
		ifTrue: [self storeLightWeightTag]
		ifFalse: [self storeFullTag]
	! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 7/24/2010 15:25'!
storeFullTag
	repository 
		updateRef: repository tagsDir / self name 
		to: (repository storeObject: self) signature! !

!GitTag methodsFor: 'saving' stamp: 'MaxLeske 8/12/2011 09:03'!
storeLightWeightTag
	repository 
		updateRef: repository tagsDir / self name 
		to: self object signature! !

!GitTag methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:30'!
tagger
	^ tagger! !

!GitTag methodsFor: 'accessing' stamp: 'MaxLeske 6/30/2010 14:00'!
tagger: aGitStamp
	self assert: aGitStamp species = GitStamp.
	tagger := aGitStamp! !

!GitTag methodsFor: 'validating' stamp: 'MaxLeske 8/12/2011 08:26'!
validate
	self name ifNil: [ self invalid: 'Tag name is required.' ].
	self isLightWeightTag ifTrue: [ ^ self ].
	super validate.
	self object ifNil: [ self invalid: 'Cannot have a full tag without a tagged object.' ].
	self tagger ifNil: [ self invalid: 'Tagger identity is required for full tags.' ].! !

!GitTag methodsFor: 'validating' stamp: 'CamilloBruni 6/22/2011 04:11'!
validateIn: aGitRepository
	self validate.
	aGitRepository objectWithSignature: self object signature.! !

!GitTaglikeObject methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 21:53'!
loadFrom: aStream
	| key value |
	[aStream peek = Character lf asInteger] whileFalse: [
		key := (aStream upTo: Character space asInteger) asString.
		value := (aStream upTo: Character lf asInteger) asString.
		self loadKey: key value: value].
	aStream next.
	message := aStream upToEnd asString.
	^self
! !

!GitTaglikeObject methodsFor: 'loading' stamp: 'CamilloBruni 7/11/2011 17:22'!
loadKey: key value: value
	self properties at: key put: value.! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:22'!
message
	^ message! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:22'!
message: aString
	message := aString! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'CamilloBruni 7/11/2011 17:22'!
properties
	^ properties ifNil: [properties := Dictionary new]! !

!GitTaglikeObject methodsFor: 'accessing' stamp: 'tonyg 2/25/2010 16:23'!
properties: anObject
	^ properties := anObject! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'tonyg 2/25/2010 16:27'!
saveFixedKeysTo: aStream
	self subclassResponsibility! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'MaxLeske 4/23/2010 13:50'!
saveKey: key value: value to: aStream
	aStream 
		nextPutAll: key; space; 
		nextPutAll: value; lf.! !

!GitTaglikeObject methodsFor: 'saving' stamp: 'CamilloBruni 7/11/2011 17:23'!
saveTo: aStream
	self validate.
	self saveFixedKeysTo: aStream.
	properties ifNotNil: [properties keysAndValuesDo:[:key :value |
			self saveKey: key value: value to: aStream]].
	aStream lf.
	aStream nextPutAll: message.! !

!GitTaglikeObject methodsFor: 'validating' stamp: 'MaxLeske 8/11/2011 22:28'!
validate
	"Override in subclasses to provide pre-save validation checks.
	Don't forget to call super validate, though!!"
	
	message ifNil: [ GitValidationError signal: 'Message is required for validation.' ].! !

GitStorableObject subclass: #GitTree
	instanceVariableNames: 'entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTree commentStamp: '<historical>' prior: 0!
I represent a node in a git repository that can contain child nodes (mostly I simply represent a directory).

Instance Variables:
	entries	<aCollection of: GitTreeEntry>
		Holds references to all the child nodes together with some 
		additional information (see the class comment for GitTreeEntry).!

!GitTree class methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	^self new entries: aCollection! !

!GitTree class methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 07:17'!
typeCode
	^ 'tree'! !

!GitTree methodsFor: 'visiting' stamp: 'CamilloBruni 6/21/2011 06:15'!
accept: aVisitor
	^ aVisitor visitTree: self.! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:38'!
at: aName
	^ repository objectWithSignature: (self entryAt: aName) objectSignature! !

!GitTree methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 6/22/2011 05:48'!
byteData
	self entries == 0
		ifTrue: [ self halt].
	^ super byteData! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:28'!
entries
	^ entries ifNil: [Dictionary new]! !

!GitTree methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 22:42'!
entries: aCollection
	aCollection do: [:each | self putEntry: each].! !

!GitTree methodsFor: 'private' stamp: 'tonyg 2/23/2010 19:10'!
entryAt: aName
	^ entries at: aName! !

!GitTree methodsFor: 'private' stamp: 'MaxLeske 10/11/2010 23:14'!
entryAt: aName ifAbsent: aBlock
	^ entries 
		at: aName
		ifAbsent: aBlock! !

!GitTree methodsFor: 'private' stamp: 'MaxLeske 10/18/2010 09:46'!
entryAt: aName ifPresent: presentBlock ifAbsent: absentBlock
	^ entries 
		at: aName
		ifPresent: presentBlock
		ifAbsent: absentBlock! !

!GitTree methodsFor: 'initialize-release' stamp: 'tonyg 2/23/2010 20:18'!
initialize
	super initialize.
	entries := Dictionary new.! !

!GitTree methodsFor: 'testing' stamp: 'CamilloBruni 5/19/2011 19:28'!
isEmpty
	^ self size == 0! !

!GitTree methodsFor: 'testing' stamp: 'CamilloBruni 7/15/2011 19:22'!
isTree
	^ true! !

!GitTree methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:35'!
loadFrom: aStream
	| entry |
	[aStream atEnd] 
		whileFalse: [entry := (GitTreeEntry in: repository) loadFrom: aStream.
			self putEntry: entry].
	^self! !

!GitTree methodsFor: 'accessing' stamp: 'CamilloBruni 7/19/2011 18:23'!
objects
	^ entries collect: [ :entry| entry object]! !

!GitTree methodsFor: 'printing' stamp: 'CamilloBruni 7/5/2011 16:54'!
printOn: aStream
	super printOn: aStream.
	aStream << '( '.
	entries keys 
		do: [ :name| aStream << name]
		separatedBy: [ aStream << ', '].
	aStream << ' )'! !

!GitTree methodsFor: 'private' stamp: 'MaxLeske 7/10/2010 10:17'!
putEntry: anEntry
	entries at: anEntry entryName put: anEntry! !

!GitTree methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 19:14'!
removeEntryAt: aFileName
	entries removeKey: aFileName! !

!GitTree methodsFor: 'accessing' stamp: 'CamilloBruni 5/17/2011 18:42'!
removeEntryAt: aFileName ifAbsent: aBlock
	entries removeKey: aFileName ifAbsent: aBlock! !

!GitTree methodsFor: 'saving' stamp: 'CamilloBruni 6/22/2011 05:47'!
saveTo: aStream
	entries values sorted do: [:e | e saveTo: aStream].! !

!GitTree methodsFor: 'accessing' stamp: 'CamilloBruni 7/5/2011 13:13'!
signature
	^ GitObjectSignature for: self! !

!GitTree methodsFor: 'testing' stamp: 'MaxLeske 6/8/2010 22:36'!
size
	| total |
	total := 0.
	self entries do: [:each | total := total + (each size)].
	^total! !

!GitTree methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 03:52'!
validate
	super validate.
	self entries do: [:entry|
		entry validate].! !

!GitTree methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 04:12'!
validateIn: aGitRepository
	self validate.
	aGitRepository objectWithSignature: self signature.
	entries do: [ :item|
		item validateIn: aGitRepository ].! !

GitObject subclass: #GitTreeEntry
	instanceVariableNames: 'mode objectSignature entryName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitTreeEntry commentStamp: '<historical>' prior: 0!
I represent an entry for a child node of a GitTree. I provide additionally information about the object (either another tree or a blob) I reference.

Instance Variables:
	mode	<Integer>
		Octal representation of the type of entry ('chmod' bit settings).
		The mode for newly created objects is either that of 'self fileMode' or 'self dirMode'.
		Loaded entries may have a varying bit sequence because git only pays attention
		to the executable bit.
	objectSignature	<GitObjectSignature>
		The signature of the object referenced.
	entryName	<String>
		'Physical' name of the entry. On a filesystem this would be the name of the
		file or directory.!

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 6/23/2010 19:10'!
dirMode
	^8r040000! !

!GitTreeEntry class methodsFor: 'instance creation' stamp: 'MaxLeske 7/10/2010 10:39'!
entryName: aString mode: anInteger objectSignature: aGitObjectSignature
	^ self new initializeFileName: aString mode: anInteger signature: aGitObjectSignature! !

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 6/23/2010 19:13'!
fileMode
	^8r100644! !

!GitTreeEntry class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 5/16/2010 16:09'!
fileName: aString mode: anInteger objectSignature: aGitObjectSignature
	^ self new initializeFileName: aString mode: anInteger signature: aGitObjectSignature! !

!GitTreeEntry class methodsFor: 'public' stamp: 'MaxLeske 7/21/2010 18:59'!
modeFor: aReference
	^ aReference isDirectory 
		ifTrue: [ self dirMode ]
		ifFalse: [ self fileMode ]! !

!GitTreeEntry class methodsFor: 'printing' stamp: 'MaxLeske 7/10/2010 11:10'!
printMode: octalMode
	^octalMode printStringBase: 8! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 7/24/2010 17:04'!
<= other
	"For sorting reasons, directories are compared
	with a virtual slash appended."
	| myTail otherTail |
	myTail := otherTail := ''.
	self mode = (self class dirMode)
		ifTrue: [ myTail := '/' ].
	other mode = (self class dirMode)
		ifTrue: [otherTail := '/' ].
	
	^ self entryName, myTail <= (other entryName, otherTail)! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 8/5/2010 17:15'!
= other
	^ other species = self species and: [
		other objectSignature = self objectSignature ]! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 7/10/2010 10:16'!
entryName
	^ entryName! !

!GitTreeEntry methodsFor: 'testing' stamp: 'MaxLeske 10/12/2010 19:37'!
hasDirectoryMode
	^ self hasFileMode not! !

!GitTreeEntry methodsFor: 'testing' stamp: 'MaxLeske 10/12/2010 19:36'!
hasFileMode
	^ self mode = (self species fileMode)! !

!GitTreeEntry methodsFor: 'comparing' stamp: 'MaxLeske 10/12/2010 22:16'!
hash
	^ self species hash bitXor: (self mode bitXor: entryName hash)! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/21/2011 10:03'!
initializeFileName: aString mode: anInteger signature: aGitObjectSignature
	entryName := aString.
	mode := anInteger.
	objectSignature := aGitObjectSignature.! !

!GitTreeEntry methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/11/2011 17:28'!
loadFrom: aStream
	mode := Integer readFrom: (aStream upTo: Character space asInteger) asString base: 8.
	entryName := (aStream upTo: 0) asString.
	objectSignature := GitObjectSignature loadFrom: aStream.
	^self! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 08:08'!
mode
	^ mode! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/19/2011 18:23'!
object
	^ repository objectWithSignature: self objectSignature! !

!GitTreeEntry methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 13:03'!
objectSignature
	^ objectSignature! !

!GitTreeEntry methodsFor: 'printing' stamp: 'MaxLeske 6/23/2010 19:17'!
printMode
	^self class printMode: self mode! !

!GitTreeEntry methodsFor: 'printing' stamp: 'CamilloBruni 7/11/2011 17:27'!
printOn: aStream
	super printOn: aStream.
	self entryName ifNil: [ ^ self ].
	aStream
		<< '(''';
		nextPutAll: self entryName;
		<< ''')'.! !

!GitTreeEntry methodsFor: 'saving' stamp: 'MaxLeske 7/10/2010 10:16'!
saveTo: aStream.
	aStream 
		nextPutAll: self printMode;
		space;
		nextPutAll: entryName;
		nullByte.
	objectSignature saveTo: aStream.! !

!GitTreeEntry methodsFor: 'validation' stamp: 'MaxLeske 8/11/2011 22:30'!
validate
	mode ifNil: [ self invalid: 'Filemode is required for tree entries.' ].
	objectSignature ifNil: [ self invalid: 'A valid signature is required for the tree entry to point to.' ].
	entryName ifNil: [ self invalid: 'A tree entry requires a name.' ].
	entryName ifEmpty: [ self invalid: 'Cannot use empty name for a tree entry.' ].! !

!GitTreeEntry methodsFor: 'validation' stamp: 'CamilloBruni 6/22/2011 03:55'!
validateIn: aGitRepository
	self validate.
	aGitRepository objectWithSignature: self objectSignature! !

Object subclass: #GitObjectLoader
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitObjectLoader commentStamp: '<historical>' prior: 0!
I am a helper class that manages loading of objects. I was created to relieve repository objects and GitObjects from having to know about loading protocol.

Instance Variables:
	repository	<GitRepository>
		A reference to the repository an object should be loaded from.!

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:03'!
for: aGitRepository
	^self basicNew initializeOn: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 16:04'!
new
	self error: 'please use the constructor methods.'! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 12:11'!
objectFrom: aStream from: aGitRepository
	^self basicNew objectFrom: aStream from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'CamilloBruni 7/6/2011 16:03'!
objectFrom: aStream from: aGitRepository ifAbsent: aBlock
	^self basicNew objectFrom: aStream from: aGitRepository ifAbsent: aBlock! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 11:28'!
objectNamed: aGitObjectSignature from: aGitRepository
	^self basicNew objectNamed: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'CamilloBruni 7/6/2011 16:02'!
objectNamed: aGitObjectSignature from: aGitRepository ifAbsent: aBlock
	^self basicNew objectNamed: aGitObjectSignature from: aGitRepository ifAbsent: aBlock! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'MaxLeske 6/26/2010 13:16'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	^self basicNew objectWithSignature: aGitObjectSignature from: aGitRepository! !

!GitObjectLoader class methodsFor: 'instance creation' stamp: 'CamilloBruni 7/6/2011 16:02'!
objectWithSignature: aGitObjectSignature from: aGitRepository ifAbsent: aBlock
	^self basicNew objectWithSignature: aGitObjectSignature from: aGitRepository ifAbsent: aBlock! !

!GitObjectLoader methodsFor: 'initialize-release' stamp: 'MaxLeske 6/26/2010 16:04'!
initializeOn: aGitRepository
	repository := aGitRepository! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 22:00'!
objectFrom: aStream
	repository ifNil: [ ^ self error: 'Instance variable #repository needs to be initialized.' ].
	^ GitStorableObject loadFrom: aStream from: repository! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 16:57'!
objectFrom: aStream from: aGitRepository
	repository := aGitRepository.
	^self objectFrom: aStream! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:05'!
objectNamed: aHexHash
	^self objectWithSignature: (GitObjectSignature from: aHexHash)! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:47'!
objectNamed: aHexHash from: aGitRepository
	repository := aGitRepository.
	^self objectNamed: aHexHash! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:03'!
objectNamed: aHexHash from: aGitRepository ifAbsent: aBlock
	repository := aGitRepository.
	^self objectNamed: aHexHash ifAbsent: aBlock! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:05'!
objectNamed: aHexHash ifAbsent: aBlock
	^self objectWithSignature: (GitObjectSignature from: aHexHash) ifAbsent: aBlock! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 22:07'!
objectWithSignature: aGitObjectSignature
	^  self 
		objectWithSignature: aGitObjectSignature 
		ifAbsent: [ 
			GitObjectNotFoundError 
				signal: 'Object not found.'
				with: aGitObjectSignature asString]! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'MaxLeske 6/26/2010 12:44'!
objectWithSignature: aGitObjectSignature from: aGitRepository
	repository := aGitRepository.
	^self objectWithSignature: aGitObjectSignature! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:04'!
objectWithSignature: aGitObjectSignature from: aGitRepository ifAbsent: aBlock
	repository := aGitRepository.
	^self objectWithSignature: aGitObjectSignature ifAbsent: aBlock! !

!GitObjectLoader methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:25'!
objectWithSignature: aGitObjectSignature ifAbsent: aBlock
	^ [ self objectFrom: (repository looseObjectStreamAt: aGitObjectSignature) ] 
		on: FSFileDoesNotExist
		do: [ :ex | ^ (repository packedObjectWithSignature: aGitObjectSignature ifAbsent: [^ aBlock value]) object ]! !

Object subclass: #GitPack
	instanceVariableNames: 'idx basePath fanoutTable objectCount packData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 15:42'!
basePath
	^ basePath! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 22:09'!
basePath: aReference
	basePath := aReference.
	(basePath , 'idx') readStreamDo: [ :stream | idx := stream contents readStream ].
	((idx next: 8) = #[255 116 79 99 0 0 0 2])
		ifFalse: [ self InvalidPackfile: 'Bad pack index header.' ].
	fanoutTable := idx next: 1024. "256 4-byte big-endian entries."
	objectCount := self fanoutTableAt: 255.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:43'!
crcAt: index
	"1024 fanout, 8 header, minus 4 for smalltalk's 1-based indexing"
	idx position: 1028 + (objectCount * 20) + (index * 4).
	^ (idx next: 4) asInteger.! !

!GitPack methodsFor: 'accessing' stamp: 'tonyg 2/22/2010 22:31'!
fanoutTableAt: fanoutIndex
	^ fanoutTable unsignedLongAt: (fanoutIndex * 4) + 1 bigEndian: true.! !

!GitPack methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:13'!
invalidPackfile:  aString
	GitInvalidPackfileError signal: aString! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 8/2/2011 19:04'!
objectAt: sha1String
	| packedObject |
	packedObject := self packedObjectAt: sha1String.
	^ packedObject 
		ifNotNil: [ packedObject object ]
		ifNil: [ nil ]! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:47'!
offsetAt: index
	| o |
	"1024 fanout, 8 header. We're still using zero-based indexing here."
	idx position: 1032 + (objectCount * 24) + (index * 4).
	o := Integer readFrom: (idx next: 4) hex base: 16.
	(o >= 16r80000000)
		ifFalse: [^ o]
		ifTrue: [
			"the table offset is zero-based here because it comes from git, not smalltalk"
			idx position: 1032 + (objectCount * 28) + ((o bitAnd: 16r7FFFFFFF) * 8).
			^ (idx next: 8) asInteger]! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 8/2/2011 19:59'!
packedObjectAt: sha1String
	| index packOffset |
	index := self tableIndexForSha: sha1String.
	index ifNil: [^ nil].
	packOffset := self offsetAt: index.
	^ self packedObjectAtOffset: packOffset! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 8/2/2011 20:42'!
packedObjectAtOffset: offset 
	| packStream packedObject |
	packData ifNil: [ 
		packData := (basePath , 'pack') readStreamDo: [ :stream | stream contents ] ].
	
	packStream := packData readStream.
	packStream position: offset.
	packedObject := GitPackedObject new pack: self.
	packedObject loadFrom: packStream.
		
	^ packedObject! !

!GitPack methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 08:41'!
tableIndexForSha: sha1
	| fanoutIndex sha1Integer index low high test |
	sha1Integer := Integer readFrom: sha1 asUppercase readStream base: 16.
	fanoutIndex := sha1Integer >> 152. "160 bit hash. We want the upper 8 bits."

	"We are searching using a zero-based index, since position: on the file is zero-based."
	low := (fanoutIndex > 0) 
		ifTrue: [ self fanoutTableAt: fanoutIndex - 1 ] 
		ifFalse: [ 0 ].
	high := (self fanoutTableAt: fanoutIndex) - 1. "binary-search wants an inclusive range"

	[ index := high + low // 2. low > high] whileFalse: [
		idx position: index * 20 + 1032. "256 * 4 for the fanoutTable, plus 8 for the header"
		test := Integer readFrom: (idx next: 20) hex base: 16.
		test = sha1Integer 
			ifTrue: [ ^ index ]
			ifFalse: [ sha1Integer > test
				ifTrue: [ low := index + 1 ]
				ifFalse: [ high := index - 1 ] ] ].
	^ nil! !

Object subclass: #GitPackedObject
	instanceVariableNames: 'pack file decodeClass length headerPos unpackedBytes repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
applyDelta: delta to: base
	decodeClass := base decodeClass.
	unpackedBytes := delta runAgainst: base unpackedBytes.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
decodeClass
	^ decodeClass! !

!GitPackedObject methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:14'!
invalidBytes: aString
	GitInvalidBytesError signal: aString! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:51'!
isBlob: byte
	^ byte = 2r00110000! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:51'!
isCommit: byte
	^ byte = 2r00010000! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:55'!
isOfsDelta: byte
	^ byte = 2r01100000! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:55'!
isRefDelta: byte
	^ byte = 2r01110000! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:52'!
isTag: byte
	^ byte = 2r01000000! !

!GitPackedObject methodsFor: 'testing' stamp: 'MaxLeske 8/1/2011 19:51'!
isTree: byte
	^ byte = 2r00100000! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/1/2011 19:25'!
loadFrom: aStream
	| byte type shift |
	file := aStream.
	headerPos := file position.
	type := self readObjectType.
	file back. "the same byte contains both type and size information"
	length := self readUnpackedDataSize.
	self unpackContentType: type.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 09:25'!
object
	^ (decodeClass in: repository) loadFrom: unpackedBytes readStream! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack
	^ pack! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 11:05'!
pack: anObject
	pack := anObject! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 08:42'!
readBaseObjectOffset 
"modified big endian encoding.
Most significant bit of each byte indicates if that byte is the last
byte encoding for the value (bit = 0) or if another byte follows (bit = 1)"
	| byte offset |
	byte := file next.
	offset := byte bitAnd: 2r01111111.
	[ (byte bitAnd: 2r10000000) isZero ] whileFalse: [
		byte := file next.
		offset := ((offset + 1) << 7) bitOr: (byte bitAnd: 2r01111111) ].
	
	^ offset! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:07'!
readDelta
	^ GitDeltaInterpreter new instructions: (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/1/2011 19:23'!
readObjectType
	"001 - OBJ_COMMIT
	010 - OBJ_TREE
	011 - OBJ_BLOB
	100 - OBJ_TAG
	110 - OBJ_OFS_DELT (new packfile format)
	111 - OBJ_REF_DELTA (old packfile format)"
	^ file next bitAnd: 2r01110000.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 08:42'!
readUnpackedDataSize
	| size shift byte |
	byte := file next.
	size := byte bitAnd: 2r00001111.
	shift := 4.
	[ (byte bitAnd: 2r10000000) isZero ] whileFalse: [
		byte := file next.
		size := size bitOr: ((byte bitAnd: 127) << shift).
		shift := shift + 7].
	
	^ size! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 7/26/2010 12:54'!
repository: aRepository
	repository := aRepository! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 09:06'!
unpackContentType: type
	(self isCommit: type) ifTrue: [ decodeClass := GitCommit. ^ self unpackSimpleContent ].
	(self isTree: type) ifTrue: [ decodeClass := GitTree. ^ self unpackSimpleContent ].
	(self isBlob: type) ifTrue: [ decodeClass := GitBlob. ^ self unpackSimpleContent ].
	(self isTag: type) ifTrue: [ decodeClass := GitTag. ^ self unpackSimpleContent ].
	(self isOfsDelta: type) ifTrue: [ ^ self unpackOffsetDeltaContent ].
	(self isRefDelta: type) ifTrue: [ ^ self unpackRefDeltaContent ].
	self invalidBytes: 'Invalid pack object type ', type printString! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 8/1/2011 19:15'!
unpackOffsetDeltaContent
	| baseOffset base delta |
	baseOffset := self readBaseObjectOffset.
	delta := self readDelta.
	base := pack packedObjectAtOffset: headerPos - baseOffset.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'MaxLeske 6/25/2011 19:09'!
unpackRefDeltaContent
	| s delta base |
	s := (GitObjectSignature loadFrom: file) hexHash.
	delta := self readDelta.
	base := pack packedObjectAt: s.
	self applyDelta: delta to: base.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:05'!
unpackSimpleContent 
	unpackedBytes := (ZLibReadStream on: file) contents.! !

!GitPackedObject methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 20:09'!
unpackedBytes
	^ unpackedBytes! !

!GitPackedObject methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:12'!
unsupportedOperation: aString
	GitUnsupportedOperationError signal: aString! !

Object subclass: #GitRepository
	instanceVariableNames: 'reference packs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitRepository commentStamp: '<historical>' prior: 0!
I am responsible for managing low level operations on a repository. I keep a reference to the location of the physical repository.

Instance Variables
	reference:		<FSReference>
		Holds a reference to the physical location of the repository.
		
!

!GitRepository class methodsFor: 'instance creation' stamp: 'MaxLeske 8/11/2011 22:15'!
new
	self error: 'This class should be instantiated with a Filesystem.'! !

!GitRepository class methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/13/2011 20:05'!
on: aReference
	^self basicNew initializeOn: aReference asReference
	! !

!GitRepository methodsFor: 'comparing' stamp: 'CamilloBruni 7/20/2011 13:07'!
= other
	(other isKindOf: self class)
		ifFalse: [ ^ false ].
	^ self baseDir = (other baseDir)! !

!GitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 19:34'!
activeBranchName
"Answers the name of the currently active branch which is the branch referenced in HEAD. This method can answer nil."

	| headContents |
	headContents := self headFile readStreamDo: [ :stream | stream contents asString withBlanksTrimmed ].
	(headContents beginsWith: 'ref:')
		ifTrue: [ ^ (headContents subStrings: '/') last ]
		ifFalse: [ ^ nil ]! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 7/15/2011 15:40'!
baseDir
"Answers the reference to the actual repository location. If e.g. the repository was initialized with a reference to '/foo' but the repository were actually at '/foo/.git' then this method would answer a reference to '/foo/.git'".

	(reference isRoot not and: [reference basename endsWith: '.git'])
		ifTrue: [^reference]
		ifFalse: [^reference resolve: '.git']! !

!GitRepository methodsFor: 'private' stamp: 'CamilloBruni 8/9/2011 17:47'!
basicUpdateRef: refPath to: aString
	| target pathReference |
	target := [(self baseDir resolve: refPath) readStreamDo: [ :stream | stream contents ]]
		on: FSFileDoesNotExist do: [:ex | ex return: nil].

	(target isNil not and: [target beginsWith: 'ref:']) ifTrue: [
		^ self updateRef: (target allButFirst: 4) withBlanksTrimmed to: aString ].
	
	self flag: 'rather make sure we never get strings in here...'.
	pathReference := refPath isString 
		ifTrue: [self baseDir resolve: refPath]
		ifFalse: [ refPath ].
		
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ :stream | 
		stream 
			nextPutAll: aString; 
			nextPut: Character lf ]! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 10/29/2010 22:23'!
branchNamed: aString
	^ self objectWithSignature: (self resolveRef: 'refs/heads/', aString)! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 10/29/2010 22:17'!
branchNames
	| branchRefs branchNames |
	branchRefs := self headsDir children.
	branchRefs isEmpty
		ifFalse: [ ^ branchRefs collect: [ :ref | ref basename ] ]
		ifTrue: [ | packedRefs |
			packedRefs := self packedRefs.
			packedRefs ifNil: [ ^ OrderedCollection new ].
			branchNames := OrderedCollection new.
			packedRefs keysAndValuesDo: [ :refName :refSignature | 
				(refName includesSubString: 'heads') ifTrue: [
					branchNames add: (refName subStrings: '/') last ] ].
			^ branchNames ]! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 6/22/2011 05:05'!
branches
"Answers a dictionary of all GitCommit objects referenced in '/refs/heads'."

	| dictionary heads |
	dictionary := Dictionary new.
	self headsDir exists ifFalse: [ ^ dictionary ].
	heads := self headsDir children.
	heads isEmpty
		ifFalse: [ heads do: [ :each |
			dictionary at: each basename put: 
				(each readStreamDo: [ :stream |
					self objectNamed: stream contents asString ]) ] ]
		ifTrue: [ | packedRefs | 
			packedRefs := self packedRefs .
			packedRefs ifNil: [ ^ dictionary ].
			packedRefs keysAndValuesDo: [ :refName :refSignature | 
				(refName includesSubString: 'heads') ifTrue: [ 
					dictionary 
						at: refName
						put: (self objectWithSignature: refSignature) ] ] ] .
		
	^ dictionary! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 8/12/2011 09:34'!
collectTagsFromLooseRefs
	| tags |
	tags := Dictionary new.
	
	self tagsDir children do: [ :ref |
		ref readStreamDo: [ :stream |
			tags 
				at: ref basename
				put: (self objectNamed: stream contents asString) ] ].
		
	^ tags! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 8/12/2011 09:34'!
collectTagsFromPackedRefs
	| tags packedRefs |
	tags := Dictionary new.
	
	packedRefs := self packedRefs. 
	packedRefs ifNotNil: [
		packedRefs keysAndValuesDo: [ :refName :refSignature | 
			(refName includesSubString: 'tags') ifTrue: [
				tags
					at: refName
					put: (self objectWithSignature: refSignature) ] ] ].
	
	^ tags! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/24/2010 15:13'!
delete
	"Deletes the repository irrevocably."
	
	GitRepositoryDeleteRequest signal
		ifTrue: [ self baseDir delete ]! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 6/30/2010 08:15'!
exists
	^self baseDir exists and: [self isInitialized]! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 7/11/2010 12:32'!
gitDirectories
	^#('objects' 
		'objects/pack' 
		'objects/info' 
		'refs'
		'refs/heads' 
		'refs/tags' 
		'info')! !

!GitRepository methodsFor: 'private' stamp: 'CamilloBruni 7/13/2011 18:37'!
gitInitialize
	self exists
		ifTrue: [ ^ self ].
	self baseDir ensureDirectory.
	self gitDirectories do: [ :dir | (self baseDir resolve: dir) createDirectory ].
	self baseDir / 'HEAD'
		writeStreamDo: [ :stream | 
			stream
				nextPutAll: 'ref: refs/heads/master';
				nextPut: Character lf ]! !

!GitRepository methodsFor: 'comparing' stamp: 'CamilloBruni 7/15/2011 18:14'!
hash
	^ reference basename hash bitXor: self class name hash! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 8/11/2011 21:22'!
head
"Answers the GitCommit object referenced in / via 'HEAD'."

	| signature |
	signature := self headRef.
	signature 
		ifNil: [ ^ nil ].
	^ self objectWithSignature: signature.! !

!GitRepository methodsFor: 'locations' stamp: 'MaxLeske 7/23/2010 18:02'!
headFile
	^ self baseDir resolve: 'HEAD'! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 8/11/2011 21:23'!
headRef
	^ self resolveRef: 'HEAD'! !

!GitRepository methodsFor: 'locations' stamp: 'CamilloBruni 8/9/2011 17:41'!
headsDir
	^ self baseDir resolve: 'refs/heads'! !

!GitRepository methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/13/2011 20:03'!
initializeOn: anFSReference
	self assert: anFSReference class = FSReference.
	reference := anFSReference.
	self gitInitialize! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 7/23/2010 18:13'!
isInitialized
	| isInitialized foundDirectories |
	foundDirectories := self gitDirectories collect: [:each | (self baseDir resolve: each) exists].
	isInitialized := foundDirectories size = (self gitDirectories size).
	isInitialized := self headFile exists.
	^isInitialized! !

!GitRepository methodsFor: 'testing' stamp: 'MaxLeske 5/26/2010 22:25'!
isObjectDirectory: aReference
	^aReference isDirectory and: [aReference basename size = 2]! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 6/27/2010 21:35'!
looseObjectBytesAt: aGitObjectSignature
	^ (self looseObjectStreamAt: aGitObjectSignature) contents! !

!GitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/6/2011 16:23'!
looseObjectBytesAt: aGitObjectSignature ifAbsent: aBlock
	^ (self looseObjectStreamAt: aGitObjectSignature ifAbsent: [ ^ aBlock value ]) contents! !

!GitRepository methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 10:55'!
looseObjectNames
	| r |
	r := OrderedCollection new.
	self looseObjectNamesDo: [ :n | r add: n ].
	^ r! !

!GitRepository methodsFor: 'enumerating' stamp: 'CamilloBruni 7/13/2011 19:55'!
looseObjectNamesDo: aBlock 
	| root hash |
	root := self baseDir resolve: 'objects'.
	root exists ifFalse: [ ^ self ].
	root directories do: [ :dirReference | 
		(self isObjectDirectory: dirReference)
			ifTrue: [ dirReference files do: [ :fileReference | 
				hash := dirReference basename , fileReference basename.
				aBlock value: (GitObjectSignature from: hash) ] ] ]! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 6/27/2010 21:33'!
looseObjectPathFor: aGitObjectSignature
	| hexHash |
	hexHash := aGitObjectSignature hexHash.
	^ 'objects/' , (hexHash first: 2) , '/' , (hexHash allButFirst: 2)! !

!GitRepository methodsFor: 'private' stamp: 'MaxLeske 8/3/2011 12:06'!
looseObjectStreamAt: aGitObjectSignature
	^ self 
		looseObjectStreamAt: aGitObjectSignature 
		ifAbsent: [ FSFileDoesNotExist signalWith: 
			(self baseDir resolve: (self looseObjectPathFor: aGitObjectSignature)) ]! !

!GitRepository methodsFor: 'private' stamp: 'CamilloBruni 7/6/2011 16:22'!
looseObjectStreamAt: aGitObjectSignature ifAbsent: aBlock
	^ (self baseDir resolve: (self looseObjectPathFor: aGitObjectSignature)) 
		readStreamDo: [ :stream | ReadStream on: (ZLibReadStream on: stream contents) contents ]
		ifAbsent: aBlock! !

!GitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 6/21/2011 05:37'!
looseObjects
	| r |
	r := OrderedCollection new.
	self looseObjectsDo: [ :n | r add: n ].
	^ r! !

!GitRepository methodsFor: 'enumerating' stamp: 'CamilloBruni 6/21/2011 05:38'!
looseObjectsDo: aBlock
	self looseObjectNamesDo: [ :signature|
		aBlock value: (self objectWithSignature: signature)]! !

!GitRepository methodsFor: 'loading' stamp: 'CamilloBruni 7/6/2011 16:00'!
objectNamed: aHexHash
	^ GitObjectLoader objectNamed: aHexHash from: self! !

!GitRepository methodsFor: 'loading' stamp: 'CamilloBruni 7/6/2011 16:00'!
objectWithSignature: aGitObjectSignature
	^ GitObjectLoader objectWithSignature: aGitObjectSignature from: self! !

!GitRepository methodsFor: 'loading' stamp: 'CamilloBruni 7/6/2011 16:01'!
objectWithSignature: aGitObjectSignature ifAbsent: aBlock
	^ GitObjectLoader objectWithSignature: aGitObjectSignature from: self ifAbsent: aBlock! !

!GitRepository methodsFor: 'accessing' stamp: 'MaxLeske 5/9/2010 11:49'!
packedObjectAt: objectName
	| object |
	self packsDo: [ :each |
					object := each objectAt: objectName.
					^object ifNotNil: [^ object] ].
	^ nil! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 8/11/2011 22:17'!
packedObjectWithSignature: aGitObjectSignature
	^ self packedObjectWithSignature: aGitObjectSignature
			ifAbsent: [ 
				GitObjectNotFoundError 
					signal: 'Could not find packed object in pack index file.'
					with: aGitObjectSignature hexHash asString ].! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 7/6/2011 16:26'!
packedObjectWithSignature: aGitObjectSignature ifAbsent: aBlock
	| object |
	self packsDo: [ :each |
		object := each packedObjectAt: aGitObjectSignature hexHash.
		object ifNotNil: [^ object repository: self] ].
	^ aBlock value! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 10/29/2010 07:36'!
packedRefs
	| packedRefs dictionary hexHash refName stream |
	packedRefs := self baseDir resolve: 'packed-refs'.
	packedRefs exists ifFalse: [^ nil].
	
	dictionary := Dictionary new.
	stream := ReadStream on: 
		(packedRefs readStreamDo: [ :str | str contents asString ]).
	[ stream atEnd ] whileFalse: [
		stream peek = $#
			ifTrue: [ stream upTo: Character lf ]
			ifFalse: [
				hexHash := stream upTo: Character space.
				refName := stream upTo: Character lf.
				dictionary at: refName put: (GitObjectSignature from: hexHash) ] ].
	^ dictionary! !

!GitRepository methodsFor: 'enumerating' stamp: 'MaxLeske 8/3/2011 12:06'!
packsDo: aBlock
	| dir shortName |
	packs ifNil: [
		packs := OrderedCollection new.
		dir := self baseDir resolve: 'objects/pack'.
		dir files 
			do: [ :each | (each basename endsWith: '.idx')
				ifTrue: [ shortName := each basename allButLast: 4.
					packs add: (GitPack new basePath: (dir resolve: shortName)) ] ] ].
		
	packs do: [ :each | aBlock value: each ]! !

!GitRepository methodsFor: 'accessing' stamp: 'CamilloBruni 7/13/2011 18:50'!
resolveLooseRef: refPath
	| target |
	refPath exists ifFalse: [ ^ nil ].
	target := refPath readStreamDo: [ :stream | stream contents asString ].
	(target beginsWith: 'ref:') 
		ifTrue: [^ self resolveRef: (target allButFirst: 4) withBlanksTrimmed].
	^ GitObjectSignature from: target substrings first "Ignore other junk after the hash."! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 7/13/2011 18:51'!
resolveRef: name
"From the git-rev-parse manpage:
            1. if $GIT_DIR/<name> exists, that is what you mean (this is usually useful only for HEAD, FETCH_HEAD, ORIG_HEAD and MERGE_HEAD);
            2. otherwise, $GIT_DIR/refs/<name> if exists;
            3. otherwise, $GIT_DIR/refs/tags/<name> if exists;
            4. otherwise, $GIT_DIR/refs/heads/<name> if exists;
            5. otherwise, $GIT_DIR/refs/remotes/<name> if exists;
            6. otherwise, $GIT_DIR/refs/remotes/<name>/HEAD if exists."
	| havePackedRefs packedRefs basePath|	
	havePackedRefs := false.
	basePath := self baseDir .
	packedRefs := nil.
	#(	''
		'refs/'
		'refs/tags/'
		'refs/heads/'
		'refs/remotes/'
		"We don't do the last case (number 6) from the manual yet."
	) do: [:prefix |
		(self resolveLooseRef: (basePath resolve: (prefix , name)))
			ifNotNilDo: [ :result | ^ result].
		havePackedRefs ifFalse: [
				packedRefs := self packedRefs. 
				havePackedRefs := true].
		packedRefs ifNotNil: [
			self packedRefs at: prefix , name 
					ifPresent: [ :result | ^ result ] ] ].
	^ nil! !

!GitRepository methodsFor: 'private' stamp: 'CamilloBruni 8/9/2011 19:11'!
storeLooseObject: aGitStorableObject
	| zipStream compressed pathReference |
	pathReference := self baseDir resolve: (self looseObjectPathFor: aGitStorableObject signature).
	"we store an object with the same signature (happens with empty files)"
	pathReference exists ifTrue: [  ^ aGitStorableObject].
	
	zipStream := ZLibWriteStream on: (ByteArray new).
	zipStream nextPutAll: aGitStorableObject signature bytes; close.
	compressed := zipStream encodedStream contents.
	
	pathReference parent ensureDirectory.
	pathReference writeStreamDo: [ :stream | stream nextPutAll: compressed ].
	^aGitStorableObject! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:15'!
storeObject: anObject
"Stores a GitStorableObject in the repository and answers it."

	^self storeLooseObject: anObject! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 8/11/2011 22:16'!
tagNamed: aString
	^ self tagNamed: aString ifAbsent: [ 
		GitObjectNotFoundError 
			signal: 'Tag not found.'
			with: aString ] ! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 7/6/2011 16:12'!
tagNamed: aString ifAbsent: aBlock
	(self resolveRef: 'refs/tags/', aString) ifNotNilDo: [ :tagRef|
		 ^ self objectWithSignature: tagRef ifAbsent: aBlock ].
	^ aBlock value! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 7/11/2011 17:31'!
tagNames
	| tagRefs tagNames packedRefs|
	tagRefs := self tagsDir children.
	tagRefs isEmpty
		ifFalse: [ ^ tagRefs collect: [ :ref | ref basename ] ].
	packedRefs := self packedRefs.
	packedRefs ifNil: [ ^ OrderedCollection new ].
	tagNames := OrderedCollection new.
	packedRefs keysAndValuesDo: [ :refName :refSignature | 
		(refName includesSubString: 'tags') ifTrue: [
			tagNames add: (refName subStrings: '/') last ] ].
	^ tagNames ! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 8/12/2011 09:34'!
tags
	|  referencedObjects tags tagRefs packedRefs |
	tags := OrderedCollection new.
	self tagsDir exists ifFalse: [ ^ tags ].
	referencedObjects := self collectTagsFromPackedRefs.
	referencedObjects addAll: self collectTagsFromLooseRefs.
	
	referencedObjects keysAndValuesDo: [ :name :object |
		tags add: (object species = GitCommit
			ifTrue: [ "this is a lightweight tag"
				GitTag new
					name: name;
					object: object;
					yourself ]
			ifFalse: [ "this is a full tag and will be loaded later"
				 object ]) ].
	
	^ tags! !

!GitRepository methodsFor: 'locations' stamp: 'MaxLeske 7/23/2010 17:47'!
tagsDir
	^ self baseDir resolve: 'refs/tags'! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 11/2/2010 22:32'!
updateHeadToBranch: branchName
"Updates the reference in 'HEAD' to branchName"

	self basicUpdateRef: 'HEAD' to: 'ref: refs/heads/' , branchName! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 16:58'!
updateRef: refPath to: aGitObjectSignature
"Updates the file denoted by refPath to reference the object with the sha1 name in aGitObjectSignature.
If the file denoted by refPath does not yet exist it will be created."

	self basicUpdateRef: refPath to: aGitObjectSignature hexHash! !

!GitRepository methodsFor: 'public' stamp: 'CamilloBruni 8/9/2011 16:58'!
validate
	^ GitFSCK validate: self! !

!GitRepository methodsFor: 'public' stamp: 'MaxLeske 7/23/2010 15:10'!
workingDir
"Answers the reference the repository object was initialized on (as opposed to were the repository is located; see #baseDir)."

	^reference! !

Object subclass: #GitStamp
	instanceVariableNames: 'name email timeStamp'
	classVariableNames: 'DefaultEmail DefaultName'
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStamp commentStamp: '<historical>' prior: 0!
I am responsible for providing a name, an e-mail address and a timestamp in the format specified by git.

Instance Variables:
	name	<String>
		First and last name of the person referenced.
	email	<String>
		E-mail address of the person referenced.
	timeStamp	<TimeStamp>
		A timestamp denoting the time the operation this stamp is associated
		with was performed (e.g. a commit).!

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 09:59'!
defaultEmail
	DefaultEmail isEmptyOrNil 
		ifTrue: [ ^ GitStampEmailRequest signal ]
		ifFalse: [ ^ DefaultEmail ].
	^ self defaultEmail! !

!GitStamp class methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 05:33'!
defaultEmail: aString
	DefaultEmail := aString! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 10:00'!
defaultName
	DefaultName isEmptyOrNil 
		ifTrue: [ ^ GitStampNameRequest signal ]
		ifFalse: [ ^ DefaultName ].
	^ self defaultName! !

!GitStamp class methodsFor: 'accessing' stamp: 'lr 4/13/2010 16:21'!
defaultName: aString
	DefaultName := aString! !

!GitStamp class methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 05:37'!
ensureDefaultValues
	DefaultName ifNil: [
			self defaultName: 'John Doe'].
	DefaultEmail ifNil: [
			self defaultEmail: 'john@foo.com']! !

!GitStamp class methodsFor: 'instance creation' stamp: 'tonyg 2/23/2010 21:29'!
fromString: aString
	^ self readFrom: (ReadStream on: aString)! !

!GitStamp class methodsFor: 'instance creation' stamp: 'lr 4/13/2010 16:50'!
now
	^ self new 
		name: self defaultName; 
		email: self defaultEmail;
		yourself! !

!GitStamp class methodsFor: 'instance creation' stamp: 'MaxLeske 8/11/2011 22:22'!
readFrom: aStream
	| author email secondsSince1970 sign zoneOffset |
	author := (aStream upTo: $<) withBlanksTrimmed.
	email := (aStream upTo: $>) withBlanksTrimmed.
	aStream skipSeparators.
	secondsSince1970 := Integer readFrom: aStream.
	aStream skipSeparators.
	sign := aStream next.
	('-+' includes: sign) ifFalse: [ 
		self invalid: 'Invalid timezone sign found while loading a GitStamp object.' ].
	
	zoneOffset := Integer readFrom: aStream.
	zoneOffset := (zoneOffset // 100) hours + (zoneOffset \\ 100) minutes.
	sign = $- ifTrue: [zoneOffset := zoneOffset negated].
	
	^ self new
		name: author;
		email: email; 
		timeStamp: ((DateAndTime fromUnixTime: secondsSince1970) offset: zoneOffset) asTimeStamp;
		yourself! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 10:07'!
requestEmail
	self flag: 'this is basically a nightmare...'.
	((Smalltalk class canUnderstand: #isHeadless) and: [ Smalltalk isHeadless ])
		ifTrue: [ self error: 'Please set the default email using GitStamp defaultEmail:' ].
	^FillInTheBlankMorph 
		request: 'Please provide a valid e-mail address so that you can be contacted if necessary.'
		initialAnswer: 'user@domain.ext'! !

!GitStamp class methodsFor: 'accessing' stamp: 'MaxLeske 8/12/2011 10:06'!
requestName
	self flag: 'this is basically a nightmare...'.
	((Smalltalk class canUnderstand: #isHeadless) and: [ Smalltalk isHeadless ])
		ifTrue: [ self error: 'Please set the default name using GitStamp defaultName:' ].
	^FillInTheBlankMorph 
		request: 'Please provide your full name to identify your actions.'
		initialAnswer: 'Firstname Lastname'! !

!GitStamp methodsFor: 'private' stamp: 'tonyg 2/23/2010 21:24'!
cleanUp: aString
	^ aString copyWithoutAll: '<>', String lf! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
email
	^ email! !

!GitStamp methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2011 05:18'!
email: aString
	email := self cleanUp: aString! !

!GitStamp methodsFor: 'initialize-release' stamp: 'lr 4/13/2010 16:46'!
initialize
	super initialize.
	timeStamp := TimeStamp now! !

!GitStamp methodsFor: 'exception handling' stamp: 'MaxLeske 8/11/2011 22:22'!
invalid: aString
	GitInvalidObjectError signal: aString! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:22'!
name
	^ name! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:23'!
name: aString
	name := self cleanUp: aString! !

!GitStamp methodsFor: 'printing' stamp: 'lr 7/19/2010 13:48'!
printOn: aStream
	| offset |
	aStream
		nextPutAll: name;
		nextPutAll: ' <'; nextPutAll: email; nextPutAll: '> ';
		nextPutAll: timeStamp asUnixTime printString;
		nextPut: Character space.
	offset := timeStamp offset.
	"a negative Duration will return a negative SmallInteger for e.g. #hours.
	To print our custom string we need to prepend our own $- and work with a
	positive Duration."
	offset negative
		ifTrue: [aStream nextPut: $-. offset := offset negated]
		ifFalse: [aStream nextPut: $+].
	aStream
		nextPutAll: (offset hours printString forceTo: 2 paddingStartWith: $0);
		nextPutAll: (offset minutes printString forceTo: 2 paddingStartWith: $0).! !

!GitStamp methodsFor: 'accessing' stamp: 'tonyg 2/23/2010 21:21'!
timeStamp
	^ timeStamp! !

!GitStamp methodsFor: 'accessing' stamp: 'MaxLeske 5/25/2010 20:30'!
timeStamp: aTimeStamp
	timeStamp := aTimeStamp! !

Object subclass: #GitStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Plumbing'!

!GitStream commentStamp: '<historical>' prior: 0!
I am a wrapper for a WriteStream on a ByteArray. I provide convenience conversions for certain content that is written to the stream to simplify code (e.g. converting a String argument to a ByteArray before writing it on the stream).

Instance Variables:
	stream	<WriteStream>
		The internally used WriteStream!

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
contents
	^stream contents! !

!GitStream methodsFor: 'initialize-release' stamp: 'MaxLeske 4/23/2010 13:45'!
initialize
	stream := WriteStream on: ByteArray new! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 7/23/2010 15:25'!
lf
"Appends the linefeed character to the stream."

	self nextPut: Character lf! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:43'!
nextPut: aCharacter
	stream nextPut: aCharacter asInteger! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
nextPutAll: aString
	stream nextPutAll: aString asByteArray ! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 6/24/2010 10:55'!
nextPutBytes: aByteArray
	stream nextPutAll: aByteArray! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:51'!
nullByte
	self nextPut: 0! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 6/8/2010 22:33'!
size
	^stream size! !

!GitStream methodsFor: 'accessing' stamp: 'MaxLeske 4/23/2010 13:44'!
space
	self nextPut: Character space! !

Object subclass: #MCGitClassReader
	instanceVariableNames: 'classFolder definitions className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Git-Monticello'!

!MCGitClassReader class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 13:36'!
loadDefinitionsFrom: aFile
	^ self new readFrom: aFile! !

!MCGitClassReader class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 13:39'!
loadDefinitionsFrom: aFile into: aStream
	^ self new 
		definitions: aStream;
		readFrom: aFile! !

!MCGitClassReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 16:51'!
classInstVarNames
	self flag: 'TODO'.
	^ #()! !

!MCGitClassReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 13:39'!
definitions: aStream
	definitions := aStream! !

!MCGitClassReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 17:21'!
loadCassDefinition
	| tokens commentFile traitCompositionString lastIndex classTraitCompositionString classDefinitionStream|
	className := classFolder name copyUpToLast: $..
	classDefinitionStream := (classFolder reference / 'definition.st') readStream contents asString.
	tokens := Scanner new scanTokens: classDefinitionStream.
	classDefinitionStream := classDefinitionStream readStream.
	traitCompositionString := (classDefinitionStream
		match: 'uses:';
		upToAll: 'instanceVariableNames:') withBlanksTrimmed.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	
	(classFolder reference / 'metadefinition.st') ifExists: [ :file|
		classTraitCompositionString := ( file readStream
			match: 'uses:';
			upToAll: 'instanceVariableNames:') withBlanksTrimmed].
	classTraitCompositionString ifNil: [classTraitCompositionString := '{}'].
	lastIndex := tokens size.
	commentFile := classFolder reference / 'README.txt'.
	^ MCClassDefinition
		name: (tokens at: 3)
		superclassName: (tokens at: 1)
		traitComposition: traitCompositionString
		classTraitComposition: classTraitCompositionString
		category: (tokens at: lastIndex)
		instVarNames: ((tokens at: 5) findTokens: ' ')
		classVarNames: ((tokens at: 7) findTokens: ' ')
		poolDictionaryNames: ((tokens at: lastIndex - 2) findTokens: ' ')
		classInstVarNames: (self classInstVarNames)
		type: (self typeOfSubclass: (tokens at: 2))
		comment: commentFile readStream contents asString
		commentStamp: (TimeStamp now)! !

!MCGitClassReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 16:53'!
loadClassMethodDefinitions
	^ self loadMethodsFrom: (classFolder reference / 'classmethods') meta: true! !

!MCGitClassReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 16:52'!
loadDefinitions
	self loadCassDefinition.
	self loadInstanceMethodDefinitions.
	self loadClassMethodDefinitions.! !

!MCGitClassReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 16:52'!
loadInstanceMethodDefinitions
	^ self loadMethodsFrom: (classFolder reference / 'instancemethods') meta: false! !

!MCGitClassReader methodsFor: 'loading' stamp: 'CamilloBruni 7/12/2011 16:57'!
loadMethodsFrom: aMethodFolder meta: isMeta
	|methodFiles|
	aMethodFolder exists ifFalse: [ ^ #() ].
	methodFiles := aMethodFolder glob: [ :file| file name endsWith: '.st'].
	^ methodFiles collect: [ :methodSource|
		self loadingMethodFrom: methodSource meta: isMeta ]! !

!MCGitClassReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 13:38'!
readFrom: aFile
	classFolder := aFile.
	self loadDefinitions.
	^ definitions! !

!MCGitClassReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/12/2011 16:51'!
typeOfSubclass: aSymbol
	#(
		(subclass: normal)
		(variableSubclass: variable)
		(variableByteSubclass: bytes)
		(variableWordSubclass: words)
		(weakSubclass: weak)
		) do: [:ea | ea first = aSymbol ifTrue: [^ ea second]].
	self error: 'Unrecognized class definition'! !
